//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <map>
#include "types.h"

#pragma once

//	MOREVIT - I'm abbreviating Category here to be consistent with the
//		method name abbreviations used throughout the code. These
//		methods often come in long chains and I decided to take
//		some mercy on my fellow teammembers by shortening it.
//		In this limited case it actually improves readability.

//	Permanent category IDs
enum PERMANENT_CATS
{
	CAT_PERMANENT = 100,
	CAT_ALL = CAT_PERMANENT,
	CAT_UNCATEGORIZED,
	CAT_INCOMPLETE,
	CAT_COMPLETED,
	CAT_WAITING,
	CAT_DOWNLOADING,
	CAT_ERRONEOUS,
	CAT_PAUSED,
	CAT_STOPPED,

	CAT_LASTSTATUSCAT = CAT_STOPPED,	// Update this if you add a new status category

	CAT_VIDEO,
	CAT_AUDIO,
	CAT_ARCHIVES,
	CAT_CDIMAGES,

	CAT_LASTPERMANENTCAT,
	CAT_TOTALPERMANENTCATS = CAT_LASTPERMANENTCAT-CAT_PERMANENT, // NOTE: _total_ to distinguish it
																//		from _numPermanentCats_ which would be
																//		the number visible

	CAT_MAXPERMANENTCATS=155,
};

class CPreferences;
class CCatDialog;

#pragma pack(1)
class CCat
{
//	Making these two classes friends means we don't have to put public
//		setters on this class which would make our data members writable
//		by classes which shouldn't be mucking with them.
friend class	CPreferences;
friend class	CCatDialog;

public:
				CCat();
				CCat( LPCTSTR strTitle,
					  LPCTSTR strSavePath=NULL,
					  LPCTSTR strComment=NULL,
					  LPCTSTR strAutoCatExt=NULL );
private:
	CString		m_strSavePath;
	CString		m_strTitle;
	CString		m_strComment;
	COLORREF	m_crColor;
	uint8		m_iPriority;
	CString		m_strAutoCatExt;
	bool		m_bIsPermanent;
	uint8		m_iCatID;

	typedef CArray<CCat*,CCat*>		CCatArray;
	typedef std::map<uint8,uint8>	CCatIDMap;

	static uint8		g_nAllCatType;
	static CCatArray	g_arrCat;
	static CCatIDMap	g_mapCatID;
	static uint8		g_iNumPermanentCats;

public:
//	Accessors
	const CString	&GetPath() const		{ return m_strSavePath; }
	const CString	&GetTitle() const		{ return m_strTitle; }
	const CString	&GetComment() const		{ return m_strComment; }
	COLORREF		GetColor() const		{ return m_crColor; }
	uint8			GetPriority() const		{ return m_iPriority; }
	const CString	&GetAutoCatExt() const	{ return m_strAutoCatExt; }
	bool			IsPermanent() const		{ return m_bIsPermanent; }
	uint8			GetID() const			{ return m_iCatID; }
//
//	NOTE: Category indices are:				0 .. GetNumCats()-1
//		  Indices for user categories are:	1 .. GetNumUserCats()
//		  IDs for user categories are:		1 .. CAT_PERMANENT-1
//		  IDs for permanent categories are:	CAT_PERMANENT .. CAT_PERMANENT + GetNumPermanentCats()-1
	static void		Finalize();
	static int		AddCat(CCat* pCat, bool bAssignNewID = true);
	static bool		MoveCat(uint8 iFromIndex, uint8 iToIndex);
	static void		RemoveCatByIndex(int iIndex);
	static int		GetNumCats()		{ return g_arrCat.GetCount();}
	static int		GetNumUserCats()	{ return g_arrCat.GetCount() - g_iNumPermanentCats; }
	static const TCHAR	*GetCatPathByIndex(uint8 index)	{ return g_arrCat[index]->m_strSavePath;}
	static uint8	CatIndexToUserCatIndex(uint8 index) { return index - g_iNumPermanentCats + 1; }
	static uint8	UserCatIndexToCatIndex(uint8 userIndex) { return userIndex - 1 + g_iNumPermanentCats; }
	static COLORREF	GetCatColorByIndex(uint8 index)	{ if (index>=0 && index<g_arrCat.GetCount()) return g_arrCat[index]->m_crColor; else return 0;}
	static COLORREF	GetCatColorByID(uint8 catID)	{ CCat* pCat = GetCatByID(catID); return pCat == NULL ? 0 : pCat->GetColor(); }
	static int		GetAllCatType()	{ return g_nAllCatType;}
	static void		SetAllCatType(int in)	{ g_nAllCatType=in; }
	static bool		CatIndexIsValid(int iIndex) { return (iIndex >= 0 && iIndex < g_arrCat.GetCount()); }
	static uint8	GetNumPermanentCats()	{ return g_iNumPermanentCats; }
	static uint8	GetCatIDByIndex(int index) { return GetCatByIndex(index)!=NULL ? GetCatByIndex(index)->m_iCatID : 0; }
	static int		GetCatIndexByID(uint8 catID);
	static CCat		*GetUserCatByIndex(int userIndex) { int index = UserCatIndexToCatIndex(userIndex); return(GetCatByIndex(index));}
	static CCat		*GetCatByIndex(int index) { if (CatIndexIsValid(index)) return g_arrCat[index]; else return NULL;}
	static CCat		*GetCatByID(int iCatID) { return GetCatByIndex(GetCatIndexByID(iCatID)); }
	static uint8	GetNewCatID();
};
#pragma pack()
