//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// ClientDetailDialog.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "ClientDetailDialog.h"
#include "otherfunctions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


// CClientDetailDialog dialog

IMPLEMENT_DYNAMIC(CClientDetailDialog, CDialog)
CClientDetailDialog::CClientDetailDialog(CUpDownClient* client)
	: CDialog(CClientDetailDialog::IDD, 0)
{
	m_client = client;
}

CClientDetailDialog::~CClientDetailDialog()
{
}

void CClientDetailDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BOOL CClientDetailDialog::OnInitDialog()
{
	EMULE_TRY

	CDialog::OnInitDialog();
	Localize();
	
	// Track loaded sources name [Tarod]
	bool loadedSourceName = false ;
	if (m_client->GetUserName())
	{
		loadedSourceName = (_tcsstr(m_client->GetUserName(), GetResString(IDS_LOADEDSOURCE)) != NULL)
							|| (_tcsstr(m_client->GetUserName(), GetResString(IDS_EXCHANGEDSOURCE)) != NULL) ;
	}
	CString buffer;
	buffer = m_client->GetUserName();
	if (!buffer.IsEmpty())
		GetDlgItem(IDC_DNAME)->SetWindowText(buffer);
	else
		GetDlgItem(IDC_DNAME)->SetWindowText(_T("?"));
	
	if (m_client->HasValidHash() && !loadedSourceName)
	{
		buffer ="";
		CString buffer2;
		for (uint16 i = 0;i != 16;i++)
		{
			buffer2.Format(_T("%02X"),m_client->GetUserHash()[i]);
			buffer+=buffer2;
		}
		GetDlgItem(IDC_DHASH)->SetWindowText(buffer);
	}
	else
		GetDlgItem(IDC_DHASH)->SetWindowText(_T("?"));

	// [Lord KiRon]
	// Display client and client version //Cax2 - show plus version
	bool isPlus=(m_client->PlusVersion() && m_client->PlusVersion()<0x2600);
	if(isPlus)	//Cax2 - second check - don't display the 38e (pre v1?) mods
		GetDlgItem(IDC_DSOFT)->SetWindowText(GetClientNameString(m_client->GetClientSoft())+_T(" [")+GetClientNameAndVersionString(m_client)+_T("]"));
	else
		GetDlgItem(IDC_DSOFT)->SetWindowText(GetClientNameString(m_client->GetClientSoft()));
	
	switch(m_client->GetClientSoft())
	{
		case SO_EMULE:
		case SO_LMULE:
		case SO_OLDEMULE:
		case SO_CDONKEY:
		case SO_SHAREAZA:
			buffer.Format(_T("v%s"),GetClientVersionString(m_client->GetClientSoft(),m_client->GetMuleVersion()));
			break;
		case SO_UNKNOWN:
			buffer.Format(_T("???"));
			break;
		default:
			buffer.Format(_T("v%s"),GetClientVersionString(m_client->GetClientSoft(),m_client->GetVersion()));
			break;
	}
	GetDlgItem(IDC_DVERSION)->SetWindowText((CString) buffer+((m_client->GetModVersion().IsEmpty())? _T("") : _T(" [")+(CString) m_client->GetModVersion()+_T("]")));
	// [/Lord KiRon]

	buffer.Format(_T("%u (%s)"),m_client->GetUserID(),(m_client->HasLowID() ? GetResString(IDS_PRIOLOW):GetResString(IDS_PRIOHIGH)));
	GetDlgItem(IDC_DID)->SetWindowText(buffer);
	
	buffer.Format(_T("%s:%i"),m_client->GetFullIP(),m_client->GetUserPort());
	GetDlgItem(IDC_DIP)->SetWindowText(buffer);

	if (m_client->GetServerIP())
	{
		in_addr server;
		server.S_un.S_addr = m_client->GetServerIP();
		CString s(inet_ntoa(server));
		GetDlgItem(IDC_DSIP)->SetWindowText(s);
		
		CServer* cserver = theApp.serverlist->GetServerByAddress(s, m_client->GetServerPort()); 
		if (cserver)
			GetDlgItem(IDC_DSNAME)->SetWindowText(cserver->GetListName());
		else
			GetDlgItem(IDC_DSNAME)->SetWindowText(_T("?"));
	}
	else
	{
		GetDlgItem(IDC_DSIP)->SetWindowText(_T("?"));
		GetDlgItem(IDC_DSNAME)->SetWindowText(_T("?"));
	}

	CKnownFile* file = theApp.sharedfiles->GetFileByID(m_client->reqfileid);
	if (file)
	{
		if (m_client->GetUploadState() != US_UPLOADING) 
			GetDlgItem(IDC_CURRENTLY)->SetWindowText(GetResString(IDS_CURRENTLY));
		GetDlgItem(IDC_DDOWNLOADING)->SetWindowText(MakeStringEscaped(file->GetFileName()));
	}
	else
		GetDlgItem(IDC_DDOWNLOADING)->SetWindowText(_T("-"));

	GetDlgItem(IDC_DDUP)->SetWindowText(CastItoXBytes(m_client->GetTransferedDown()));

	GetDlgItem(IDC_DDOWN)->SetWindowText(CastItoXBytes(m_client->GetTransferedUp()));

	buffer.Format(_T("%.1f %s"),(float)m_client->GetDownloadDatarate()/1024,GetResString(IDS_KBYTESEC));
	GetDlgItem(IDC_DAVUR)->SetWindowText(buffer);

	buffer.Format(_T("%.1f %s"),max((float)m_client->GetDatarate()/1024,.0),GetResString(IDS_KBYTESEC));
	GetDlgItem(IDC_DAVDR)->SetWindowText(buffer);
	
	if (m_client->Credits())
	{
		GetDlgItem(IDC_DUPTOTAL)->SetWindowText(CastItoXBytes(m_client->Credits()->GetDownloadedTotal()));
		GetDlgItem(IDC_DDOWNTOTAL)->SetWindowText(CastItoXBytes(m_client->Credits()->GetUploadedTotal()));
		buffer.Format(_T("%.1f"),(float)m_client->Credits()->GetScoreRatio(m_client->GetIP()));
		GetDlgItem(IDC_DRATIO)->SetWindowText(buffer);
	
		if (theApp.clientcredits->CryptoAvailable())
		{
				switch(m_client->Credits()->GetCurrentIdentState(m_client->GetIP()))
				{
					case IS_NOTAVAILABLE:
						GetDlgItem(IDC_CDIDENT)->SetWindowText(GetResString(IDS_IDENTNOSUPPORT));
						break;
					case IS_IDFAILED:
					case IS_IDNEEDED:
					case IS_IDBADGUY:
						GetDlgItem(IDC_CDIDENT)->SetWindowText(GetResString(IDS_IDENTFAILED));
						break;
					case IS_IDENTIFIED:
						GetDlgItem(IDC_CDIDENT)->SetWindowText(GetResString(IDS_IDENTOK));
						break;
				}
		}
		else
			GetDlgItem(IDC_CDIDENT)->SetWindowText(GetResString(IDS_IDENTNOSUPPORT));
	}
	else
	{
		GetDlgItem(IDC_DDOWNTOTAL)->SetWindowText(_T("?"));
		GetDlgItem(IDC_DUPTOTAL)->SetWindowText(_T("?"));
		GetDlgItem(IDC_DRATIO)->SetWindowText(_T("?"));
		GetDlgItem(IDC_CDIDENT)->SetWindowText(_T("?"));
	}

    // Maverick
    if (m_client->IsRemoteInfoAvaiable())
	{
		buffer.Format(_T("%0.1f"),m_client->GetRemoteBaseModifier());
		GetDlgItem(IDC_DRATIO_REMOTE)->SetWindowText(buffer);
		buffer.Format(_T("%ld"),m_client->GetRemoteRatio());
		GetDlgItem(IDC_DRATING_REMOTE)->SetWindowText(buffer);
		buffer.Format(_T("%ld"),m_client->GetRemoteScore());
		GetDlgItem(IDC_DSCORE_REMOTE)->SetWindowText(buffer);
		if (!m_client->GetRemoteCommunityEnabled()) 
			GetDlgItem(IDC_COMUSER_REMOTE)->SetWindowText(GetResString(IDS_DISABLED));
		else if (m_client->GetRemoteCommunity()) 
			GetDlgItem(IDC_COMUSER_REMOTE)->SetWindowText(GetResString(IDS_YES));
		else
			GetDlgItem(IDC_COMUSER_REMOTE)->SetWindowText(GetResString(IDS_NO));
	}
	else
	{
		if (m_client->Credits())
		{
			buffer.Format(_T("%0.1f"),m_client->GetRemoteBaseModifier());
			GetDlgItem(IDC_DRATIO_REMOTE)->SetWindowText(buffer);
			buffer.Format(_T("%ld"),m_client->GetRemoteRatio());
			GetDlgItem(IDC_DRATING_REMOTE)->SetWindowText(buffer);
			buffer.Format(_T("%ld"),m_client->GetRemoteScore());
			GetDlgItem(IDC_DSCORE_REMOTE)->SetWindowText(buffer);
		}
		else
		{
			GetDlgItem(IDC_DSCORE_REMOTE)->SetWindowText(_T("?"));
			GetDlgItem(IDC_DRATING_REMOTE)->SetWindowText(_T("?"));
			GetDlgItem(IDC_DRATIO_REMOTE)->SetWindowText(_T("?"));
		}
		GetDlgItem(IDC_COMUSER_REMOTE)->SetWindowText(GetResString(IDS_DISABLED));
	}
    // end Maverick

	if (m_client->GetUserName() && !loadedSourceName)
	{
		buffer.Format(_T("%.1f"),(float)m_client->GetScore(false,m_client->IsDownloading(),true));
		GetDlgItem(IDC_DRATING)->SetWindowText(buffer);
	}
	else
		GetDlgItem(IDC_DRATING)->SetWindowText(_T("?"));

	if (m_client->GetUploadState() != US_NONE)
	{
		if (!m_client->GetFriendSlot())
		{
			buffer.Format(_T("%u"),m_client->GetScore(false,m_client->IsDownloading(),false));
			GetDlgItem(IDC_DSCORE)->SetWindowText(buffer);
		}
		else
			GetDlgItem(IDC_DSCORE)->SetWindowText(GetResString(IDS_FRIENDDETAIL));
	}
	else
		GetDlgItem(IDC_DSCORE)->SetWindowText(_T("-"));

	// DonGato (Community Sharing)
	
		//
		if (theApp.glob_prefs->CommunityEnabled())
		{
			if (m_client->IsCommunity())
				GetDlgItem(IDC_COMUSER)->SetWindowText(GetResString(IDS_YES));
			else
				GetDlgItem(IDC_COMUSER)->SetWindowText(GetResString(IDS_NO));
		}
		else
			GetDlgItem(IDC_COMUSER)->SetWindowText(GetResString(IDS_DISABLED));

	// START - enkeyDEV(Ottavio84) -Show download queue wait time-
	if (m_client->GetDownloadState() == DS_ONQUEUE)
		GetDlgItem(IDC_QUEUETIME)->SetWindowText(CastSecondsToHM(m_client->GetQueueWaitTime()/1000));
	else
		GetDlgItem(IDC_QUEUETIME)->SetWindowText(_T("-"));
	// END - enkeyDEV(Ottavio84) -Show download queue wait time-

	return true;

	EMULE_CATCH

	return false;
}

BEGIN_MESSAGE_MAP(CClientDetailDialog, CDialog)
END_MESSAGE_MAP()


// CClientDetailDialog message handlers
void CClientDetailDialog::Localize()
{
	EMULE_TRY

	GetDlgItem(IDC_STATIC30)->SetWindowText(GetResString(IDS_CD_GENERAL));
	GetDlgItem(IDC_STATIC31)->SetWindowText(GetResString(IDS_CD_UNAME));
	GetDlgItem(IDC_STATIC32)->SetWindowText(GetResString(IDS_CD_UHASH));
	GetDlgItem(IDC_STATIC33)->SetWindowText(GetResString(IDS_CD_CSOFT));
	GetDlgItem(IDC_STATIC34)->SetWindowText(GetResString(IDS_CD_UIP));
	GetDlgItem(IDC_STATIC35)->SetWindowText(GetResString(IDS_CD_SIP));
	GetDlgItem(IDC_STATIC36)->SetWindowText(GetResString(IDS_CD_VERSION));
	GetDlgItem(IDC_STATIC37)->SetWindowText(GetResString(IDS_CD_UID));
	GetDlgItem(IDC_STATIC38)->SetWindowText(GetResString(IDS_CD_SNAME));
	GetDlgItem(IDC_STATIC39)->SetWindowText(GetResString(IDS_CD_IDENT));
	GetDlgItem(IDC_STATIC40)->SetWindowText(GetResString(IDS_CD_TRANS));
	GetDlgItem(IDC_STATIC41)->SetWindowText(GetResString(IDS_CD_CDOWN));
	GetDlgItem(IDC_STATIC42)->SetWindowText(GetResString(IDS_CD_DOWN));
	GetDlgItem(IDC_STATIC43)->SetWindowText(GetResString(IDS_CD_ADOWN));
	GetDlgItem(IDC_STATIC44)->SetWindowText(GetResString(IDS_CD_TDOWN));
	GetDlgItem(IDC_STATIC45)->SetWindowText(GetResString(IDS_CD_UP));
	GetDlgItem(IDC_STATIC46)->SetWindowText(GetResString(IDS_CD_AUP));
	GetDlgItem(IDC_STATIC47)->SetWindowText(GetResString(IDS_CD_TUP));

	GetDlgItem(IDC_STATIC50)->SetWindowText(GetResString(IDS_CD_SCORES));
	GetDlgItem(IDC_STATIC51)->SetWindowText(GetResString(IDS_CD_MOD));
	GetDlgItem(IDC_STATIC52)->SetWindowText(GetResString(IDS_CD_RATING));
	GetDlgItem(IDC_STATIC53)->SetWindowText(GetResString(IDS_CD_USCORE));
	GetDlgItem(IDC_STATIC54)->SetWindowText(GetResString(IDS_COMUSER));

	GetDlgItem(IDC_STATIC55)->SetWindowText(GetResString(IDS_CD_SCORES_REMOTE));
	GetDlgItem(IDC_STATIC56)->SetWindowText(GetResString(IDS_CD_MOD));
	GetDlgItem(IDC_STATIC57)->SetWindowText(GetResString(IDS_CD_RATING));
	GetDlgItem(IDC_STATIC58)->SetWindowText(GetResString(IDS_COMUSER));
	GetDlgItem(IDC_STATIC59)->SetWindowText(GetResString(IDS_CD_USCORE));
	GetDlgItem(IDC_QUEUETIMELBL)->SetWindowText(GetResString(IDS_QUEUETIMELBL)); // EC
	GetDlgItem(IDOK)->SetWindowText(GetResString(IDS_FD_CLOSE));

	SetWindowText(GetResString(IDS_CD_TITLE));

	EMULE_CATCH
}
