//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// ListenSocket.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "ListenSocket.h"
#include "opcodes.h"
#include "KnownFile.h"
#include "sharedfilelist.h"
#include "uploadqueue.h"
#include "updownclient.h"
#include "clientlist.h"
#include "memcpy_amd.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifdef OLD_SOCKETS_ENABLED

// CClientReqSocket
CClientReqSocket::CClientReqSocket(CPreferences* in_prefs,CUpDownClient* in_client){
	app_prefs = in_prefs;
	client = in_client;
	if (in_client)
		client->socket = this;
	theApp.listensocket->AddSocket(this);
	ResetTimeOutTimer();
	deletethis = false;
	deltimer = 0;
}


CClientReqSocket::~CClientReqSocket(){
	if (client)
		client->socket = 0;
	client = 0;
	theApp.listensocket->RemoveSocket(this);

	DEBUG_ONLY (theApp.clientlist->Debug_SocketDeleted(this));
}

void CClientReqSocket::ResetTimeOutTimer(){
	timeout_timer = ::GetTickCount();
};

bool CClientReqSocket::CheckTimeOut(){
	if (::GetTickCount() - timeout_timer > CONNECTION_TIMEOUT){
		timeout_timer = ::GetTickCount();
		Disconnect();
		return true;
	}
	return false;
};

void CClientReqSocket::OnClose(int nErrorCode){
	ASSERT (theApp.listensocket->IsValidSocket(this));
	CEMSocket::OnClose(nErrorCode);
	Disconnect();
};

void CClientReqSocket::Disconnect(){
	AsyncSelect(0);
	byConnected = ES_DISCONNECTED;
    if(client && !theApp.clientlist->Debug_IsValidClient(client)) { // obaldin: a quick fix for a strange bug
        AddDebugLogLine(false, _T("Invalid client in CClientReqSocket::Disconnect()"));
        client = 0;
    }
	if (!client)
		Safe_Delete();
	else
		client->Disconnected();
};

void CClientReqSocket::Delete_Timed(){
// it seems that MFC Sockets call socketfunctions after they are deleted, even if the socket is closed
// and select(0) is set. So we need to wait some time to make sure this doesn't happens
	if (::GetTickCount() - deltimer > 10000)
		delete this;
}

void CClientReqSocket::Safe_Delete(){
	ASSERT (theApp.listensocket->IsValidSocket(this));
	AsyncSelect(0);
	deltimer = ::GetTickCount();
	if (m_SocketData.hSocket != INVALID_SOCKET) // deadlake PROXYSUPPORT - changed to AsyncSocketEx
		ShutDown(2);
	if (client)
		client->socket = 0;
	client = 0;
	byConnected = ES_DISCONNECTED;
	deletethis = true;
}

bool CClientReqSocket::ProcessPacket(char* packet, uint32 size, uint8 opcode)
{
	try{
		try{
			if (!client && opcode != OP_HELLO)
				throw GetResString(IDS_ERR_NOHELLO);
			
			switch(opcode){
				case OP_HELLOANSWER:{
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					client->ProcessHelloAnswer(packet,size);
					if (client)
						client->ConnectionEstablished();
					break;
				}
				case OP_HELLO:{
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					bool newClient = false;
					if (!client){
						// create new client to save standart informations
						client = new CUpDownClient(this);
						newClient = true;
					}
					try {
						client->ProcessHelloPacket(packet,size);
					}
					catch(...) {
						if (newClient)
							//eklmn: reset client to NULL in order don't let CUpDownClient::Disconnected be processed
							//------ for a client which is not in the list of clients.
							safe_delete(client);
						throw;
					}
					// now we check if we now this client already. if yes this socket will
					// be attached to the known client, the new client will be deleted
					// and the var. "client" will point to the known client.
					// if not we keep our new-constructed client ;)
					if (theApp.clientlist->AttachToAlreadyKnown(&client,this)){
						// update the old client informations
						client->ProcessHelloPacket(packet,size);
						client->DisableL2HAC(); //<<-- enkeyDEV(th1) -L2HAC- lowid side
					}
					else {
						theApp.clientlist->AddClient(client);
						client->SetCommentDirty();
					}

					// if IP is filtered, dont reply but disconnect...
					/*
					if (theApp.ipfilter->IsFiltered(client->GetIP())) {
						AddDebugLogLine(false,_T("Filtered: %s %s "), client->GetFullIP(),theApp.ipfilter->GetLastHit());
						client->Disconnected();
						theApp.stat_filteredclients++;
						break;
					}
					*/

					// send a response packet with standart informations
					if (client->GetClientSoft() == SO_EMULE && client->GetHashType() == SO_EMULE)
						client->SendMuleInfoPacket(false);
					
					client->SendHelloAnswer();
					if (client)
						client->ConnectionEstablished();

					break;
				}

				case OP_FILEREQUEST:
				{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					// check failed(error + scaning) file requests
					if (client->GetFailedFileRequests() >= 3)
					{
						theApp.ipfilter->AddTemporaryBannedIP(client->GetIP());
						AddDebugLogLine(false,_T("Client %s (%s) added to filtered clients due to file scanning"),client->GetFullIP(),GetClientNameAndVersionString(client));
						Disconnect();
						break;
					}	

					if (size == 16 || (size > 16 && client->GetExtendedRequestsVersion() > 0))
					{
						if (!client->GetWaitStartTime())
							client->SetWaitStartTime();
						uchar reqfileid[16];
						md4cpy(reqfileid,packet);
						CKnownFile* reqfile = theApp.sharedfiles->GetFileByID(reqfileid);
						if (!reqfile)
						{
							client->UpdateFailedFileRequests();
							// if we've just started a download we may want to use that client as a source
							CKnownFile* partfile = theApp.downloadqueue->GetFileByID(reqfileid);
							if (partfile && partfile->IsPartFile())
								if( theApp.glob_prefs->GetMaxSourcePerFile() > ((CPartFile*)partfile)->GetSourceCount() ) //<<--
									theApp.downloadqueue->CheckAndAddKnownSource((CPartFile*)partfile,client);
								
							// check if client are using our userhas (???)
							if (!memcmp(theApp.glob_prefs->GetUserHash(), client->GetUserHash(),16)) 
								AddDebugLogLine(false,_T("Client %s (%s) is using our userhash"),client->GetUserName(),client->GetFullIP());
						}
						if (!reqfile)
						{
							// DbT:FileRequest
							// send file request no such file packet (0x48)
							Packet* replypacket = new Packet(OP_FILEREQANSNOFIL, 16);
							md4cpy(replypacket->pBuffer, packet);
							theApp.uploadqueue->AddUpDataOverheadFileRequest(replypacket->size);
							SendPacket(replypacket, true);
							// DbT:End
							break;
						}
						// if wer are downloading this file, this could be a new source
						if (reqfile->IsPartFile())
							if( theApp.glob_prefs->GetMaxSourcePerFile() > ((CPartFile*)reqfile)->GetSourceCount() ) //<<--
								theApp.downloadqueue->CheckAndAddKnownSource((CPartFile*)reqfile,client);
						
						// check to see if this is a new file they are asking for
						if(md4cmp(client->reqfileid, packet) != 0)
							client->SetCommentDirty();

						// send filename etc
						client->ResetFailedFileRequests();
						client->SetUploadFileID((uchar*)packet);
						CSafeMemFile data(128);
						data.Write(reqfile->GetFileHash(),16);
						uint16 namelength = (uint16)_tcslen(reqfile->GetFileName());
						data.Write(&namelength,2);
						data.Write(reqfile->GetFileName(),namelength);
						// TODO: Don't let 'ProcessUpFileStatus' re-process the entire packet and search the fileid
						// again in 'sharedfiles' -> waste of time.
						client->ProcessUpFileStatus(packet,size);
						Packet* packet = new Packet(&data);
						packet->opcode = OP_FILEREQANSWER;
						theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
						SendPacket(packet,true);
						client->SendCommentInfo(reqfile);
					}
					else
					{
						client->UpdateFailedFileRequests();
						throw CString(GetResString(IDS_ERR_WRONGPACKAGESIZE)+ _T(" (OP_FILEREQUEST)"));
					}
					break;
				}
				case OP_FILEREQANSNOFIL:{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
				// DbT:FileRequest
				if (size == 16) {
					// if that client do not have my file maybe has another different
					CPartFile* reqfile = theApp.downloadqueue->GetFileByID((uchar*)packet);
					if (!reqfile)
						break;

					// we try to swap to another file ignoring no needed parts files
					if (client) switch (client->GetDownloadState()) {
						case DS_ONQUEUE:
						case DS_NONEEDEDPARTS:
						if (!client->SwapToAnotherFile(NULL)) {	//eklmn: bugfix(08): A4AF switching (???)
							theApp.downloadqueue->RemoveSource(client, true);
						}
						break;
					}
					break;
				}
				throw CString(GetResString(IDS_ERR_WRONGPACKAGESIZE)+ _T(" (OP_FILEREQANSNOFIL)")); 
				break;
				// DbT:End
				}
				case OP_FILEREQANSWER:{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					client->ProcessFileInfo(packet,size);
					break;
				}
				case OP_FILESTATUS:{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					client->ProcessFileStatus(packet,size);
					break;
				}
				case OP_STARTUPLOADREQ:
				{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					// DbT:StartUploadReq
					if( size == 16 )
					{
						uchar reqfileid[16];
						md4cpy(reqfileid,packet);
						CKnownFile* reqfile = theApp.sharedfiles->GetFileByID(reqfileid);
						if (reqfile)
						{
							// if wer are downloading this file, this could be a new source
							if (reqfile->IsPartFile())
								if( theApp.glob_prefs->GetMaxSourcePerFile() > ((CPartFile*)reqfile)->GetSourceCount() ) //<<--
									theApp.downloadqueue->CheckAndAddKnownSource((CPartFile*)reqfile,client);

							// check to see if this is a new file they are asking for
							if(md4cmp(client->GetUploadFileID(), packet) != 0)
								client->SetCommentDirty();

							client->SetUploadFileID((uchar*)packet);
							client->SendCommentInfo(reqfile);
						}
					}
					theApp.uploadqueue->AddClientToQueue(client);
					break;
					// DbT:End
				}
				case OP_QUEUERANK:
				{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					CSafeMemFile data((BYTE*)packet,size);
					uint32 rank;
					data.Read(&rank,4);
					client->SetRemoteQueueRank(rank);					
					// eklmn: switch DL state in case newrank != 0
					if (rank && client->GetDownloadState() == DS_DOWNLOADING)
						client->SetDownloadState(DS_ONQUEUE);
					break;
				}
				case OP_ACCEPTUPLOADREQ:
				{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					// modified by sivka [dont download if paused]
					if(client->reqfile && theApp.downloadqueue->IsPartFile(client->reqfile))
					{
						// eklmn: don't ask for paused files
						if (client->GetDownloadState() == DS_ONQUEUE && !client->reqfile->IsPaused() 
						&& (client->reqfile->GetStatus(false) == PS_READY || client->reqfile->GetStatus(false) == PS_EMPTY))
						{
							client->SetDownloadState(DS_DOWNLOADING);
							client->SetLastDownPart(0xFFFF); // eklmn: feature(00): ICR 2.3 reset part before new download 
							client->SendBlockRequests();
						}
						else // added by sivka [dont download if paused]
						{
							Packet* packet = new Packet(OP_CANCELTRANSFER,0);
							theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
							//client->socket->SendPacket(packet,false,true); //eklmn: bugfix(04): 
							client->socket->SendPacket(packet,true,true); //eklmn: bugfix(05): Listen soket OP_ACCEPTUPLOADREQ
							client->SetDownloadState(DS_ONQUEUE);
						}
					}
					else
						throw CString(GetResString(IDS_ERR_WRONGFILEID)+ _T(" (OP_ACCEPTUPLOADREQ => !reqfile)"));
					break;
				}
				case OP_REQUESTPARTS:{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					CSafeMemFile data((BYTE*)packet,size);
					uchar reqfilehash[16];
					data.Read(reqfilehash,16);
					Requested_Block_Struct* reqblock1 = new Requested_Block_Struct;
					Requested_Block_Struct* reqblock2 = new Requested_Block_Struct;
					Requested_Block_Struct* reqblock3 = new Requested_Block_Struct;
					data.Read(&reqblock1->StartOffset,4);
					data.Read(&reqblock2->StartOffset,4);
					data.Read(&reqblock3->StartOffset,4);
					data.Read(&reqblock1->EndOffset,4);
					data.Read(&reqblock2->EndOffset,4);
					data.Read(&reqblock3->EndOffset,4);
					md4cpy(&reqblock1->FileID,reqfilehash);
					md4cpy(&reqblock2->FileID,reqfilehash);
					md4cpy(&reqblock3->FileID,reqfilehash);
					if (reqblock1->EndOffset-reqblock1->StartOffset == 0)			
						delete reqblock1;
					else
						client->AddReqBlock(reqblock1);

					if (reqblock2->EndOffset-reqblock2->StartOffset == 0)			
						delete reqblock2;
					else
						client->AddReqBlock(reqblock2);

					if (reqblock3->EndOffset-reqblock3->StartOffset == 0)			
						delete reqblock3;
					else
						client->AddReqBlock(reqblock3);
					break;
				}
				case OP_CANCELTRANSFER:{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					theApp.uploadqueue->RemoveFromUploadQueue(client,ETS_CANCELED);
					theApp.emuledlg->AddDebugLogLine(false, _T("%s: Upload session ended due to a canceled transfer."), client->GetUserName());
					client->SetUploadFileID(NULL);
					break;
				}
				case OP_END_OF_DOWNLOAD:
				{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					if (size>=16 && !md4cmp(client->GetUploadFileID(),packet)){
						theApp.uploadqueue->RemoveFromUploadQueue(client,ETS_END_OF_DOWNLOAD);
						AddDebugLogLine(false, _T("%s: Upload session ended due ended transfer."), client->GetUserName());
						client->SetUploadFileID(NULL);
					}
					break;
				}				
				case OP_HASHSETREQUEST:{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					if (size != 16)
						throw GetResString(IDS_ERR_WRONGHPACKAGESIZE);
					client->SendHashsetPacket(packet);
					break;
				}
				case OP_HASHSETANSWER:{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
                    //if (client->GetDownloadState() != DS_REQHASHSET)
                    //    throw CString("unwanted hashset");
					client->ProcessHashSet(packet,size);
					break;
				}
				case OP_SENDINGPART:
				{
//					theApp.downloadqueue->AddDownDataOverheadOther(0, 24);
					if (client->reqfile && (client->reqfile->GetStatus() == PS_READY || client->reqfile->GetStatus() == PS_EMPTY))
					{
						client->ProcessBlockPacket(packet,size);
						if ((client->reqfile != NULL) &&
							(client->reqfile->GetStatus()==PS_PAUSED 
							|| client->reqfile->GetStatus()==PS_STOPPED
							|| client->reqfile->GetStatus()==PS_ERROR))
						{
							Packet* packet = new Packet(OP_CANCELTRANSFER,0);
							theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
							client->socket->SendPacket(packet,true,true);
						}
					}
					else{
						Packet* packet = new Packet(OP_CANCELTRANSFER,0);
						theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
						client->socket->SendPacket(packet,true,true);
						client->SetDownloadState(client->reqfile==NULL ? DS_NONE : DS_ONQUEUE);
					}
					break;
				}
				case OP_OUTOFPARTREQS:{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					if (client->GetDownloadState() == DS_DOWNLOADING)
						client->SetDownloadState(DS_ONQUEUE);
					break;
				}
				case OP_SETREQFILEID:
				{
					theApp.downloadqueue->AddDownDataOverheadFileRequest(size);
					// DbT:FileRequest
					if (size == 16)
					{
						if (!client->GetWaitStartTime()) 
							client->SetWaitStartTime();

						CKnownFile* reqfile = theApp.sharedfiles->GetFileByID((uchar*)packet);

						/*
						if (!reqfile)
						{
							// Need to send an OP_FILESTATUS for getting an answer for OP_REQUESTSOURCES!!!
							reqfile = theApp.downloadqueue->GetFileByID((uchar*)packet);
						}
						*/
						if (!reqfile)
						{
							client->UpdateFailedFileRequests();
							if (client->GetFailedFileRequests() >= 3)
							{
								theApp.ipfilter->AddTemporaryBannedIP(client->GetIP());
								AddDebugLogLine(false,_T("Client %s (%s) added to filtered clients due to file scanning"),client->GetFullIP(),GetClientNameAndVersionString(client));
								Disconnect();
								break;
							}	
						
							// send file request no such file packet (0x48)
							Packet* replypacket = new Packet(OP_FILEREQANSNOFIL, 16);
							md4cpy(replypacket->pBuffer, packet);
							theApp.uploadqueue->AddUpDataOverheadFileRequest(replypacket->size);
							SendPacket(replypacket, true);
							break;
						}

						// eklmn: reset Failed Request Counter
						client->ResetFailedFileRequests();
						// if we are downloading this file, this could be a new source
						if (reqfile->IsPartFile())
							if( theApp.glob_prefs->GetMaxSourcePerFile() > ((CPartFile*)reqfile)->GetSourceCount() )
								theApp.downloadqueue->CheckAndAddKnownSource((CPartFile*)reqfile,client); 

						// check to see if this is a new file they are asking for
						if(md4cmp(client->GetUploadFileID(), packet) != 0)
							client->SetCommentDirty();

						//send filestatus
						client->SetUploadFileID((uchar*)packet);
						//md4cpy(client->reqfileid,packet);
						CSafeMemFile data(16+16);
						data.Write(reqfile->GetFileHash(),16);
						if (reqfile->IsPartFile())
							((CPartFile*)reqfile)->WritePartStatus(&data);
#if JUMPSTART
                    				else if (reqfile->GetJumpstartEnabled())
							reqfile->WriteJumpstartPartStatus(client, &data);
#endif // JUMPSTART
						//--- xrmb:partprio ---
						else if(reqfile->HasHiddenParts())
							reqfile->WritePartStatus(&data);
						//--- :xrmb ---
						else 
						{
							uint32 null = 0;
							data.Write(&null,3);
						}

						Packet* packet = new Packet(&data);
						packet->opcode = OP_FILESTATUS;
						theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
						SendPacket(packet,true);
					}
					else
						throw CString(GetResString(IDS_ERR_WRONGPACKAGESIZE)+ _T(" (OP_SETREQFILEID)"));
					break;
					// DbT:End
				}
				case OP_CHANGE_CLIENT_ID:
				{
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					CSafeMemFile data((BYTE*)packet, size);
					uint32 nNewUserID;
					data.Read(&nNewUserID, 4);
					uint32 nNewServerIP;
					data.Read(&nNewServerIP, 4);
					if (nNewUserID < 16777216){ // client changed server and gots a LowID
						CServer* pNewServer = theApp.serverlist->GetServerByIP(nNewServerIP);
						if (pNewServer != NULL){
							client->SetUserID(nNewUserID); // update UserID only if we know the server
							client->SetServerIP(nNewServerIP);
							client->SetServerPort(pNewServer->GetPort());
						}
					}
					else if (nNewUserID == client->GetIP()){ // client changed server and gots a HighID(IP)
						client->SetUserID(nNewUserID);
						CServer* pNewServer = theApp.serverlist->GetServerByIP(nNewServerIP);
						if (pNewServer != NULL){
							client->SetServerIP(nNewServerIP);
							client->SetServerPort(pNewServer->GetPort());
						}
					}					
					break;
				}

				case OP_CHANGE_SLOT:
				{
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					// sometimes sent by Hybrid
					break;
				}
				case OP_MESSAGE:{
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					uint16 length;
					#ifndef AMD
					memcpy(&length,packet,2);
					#else
					memcpy_amd(&length,packet,2);
					#endif
					if (length+2 != size)
						throw CString("invalid message packet");
					char* message = new char[length+1];
					#ifndef AMD
					memcpy(message,packet+2,length);
					#else
					memcpy_amd(message,packet+2,length);
					#endif
					message[length] = 0;
					theApp.emuledlg->chatwnd.chatselector.ProcessMessage(client,message);
					delete[] message;
					break;
				}
				case OP_ASKSHAREDFILES:	{	// client wants to know what we have in share, let's see if we allow him to know that
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					
					// IP banned, no answer for this request
					if (client->IsBanned() ){
						break;						
					}

					CPtrList list;
					if (theApp.glob_prefs->CanSeeShares() == SEE_SHARE_EVERYBODY ||
						(theApp.glob_prefs->CanSeeShares() == SEE_SHARE_FRIENDS && client->IsFriend() ) )
					{
						CCKey bufKey;
						CKnownFile* cur_file;
						for (POSITION pos = theApp.sharedfiles->m_Files_map.GetStartPosition();pos != 0;){
							theApp.sharedfiles->m_Files_map.GetNextAssoc(pos,bufKey,cur_file);
							list.AddTail((void*&)cur_file);
						}
						AddDebugLogLine(false,IDS_REQ_SHAREDFILES,client->GetUserName(),client->GetUserID(),GetResString(IDS_ACCEPTED) );
					} else AddDebugLogLine(false,IDS_REQ_SHAREDFILES,client->GetUserName(),client->GetUserID(),GetResString(IDS_DENIED) );

					// now create the memfile for the packet
					uint32 iTotalCount = list.GetCount();
					CSafeMemFile tempfile(80);
					tempfile.Write(&iTotalCount, 4);
					while (list.GetCount())
					{
						theApp.sharedfiles->CreateOfferedFilePacket(*(CKnownFile*)list.GetHead(), tempfile);
						list.RemoveHead();
					}
					// create a packet and send it
					Packet* replypacket = new Packet(&tempfile);
					replypacket->opcode = OP_ASKSHAREDFILESANSWER;
					theApp.uploadqueue->AddUpDataOverheadOther(replypacket->size);
					SendPacket(replypacket, true, true);
					break;
				}
				case OP_ASKSHAREDFILESANSWER:{
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					client->ProcessSharedFileList(packet,size);
					break;
				}
				case OP_ASKSHAREDDIRS:
				{
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					ASSERT( size == 0 );
					
					// IP banned, no answer for this request
					if (client->IsBanned() )	break;						

					if (theApp.glob_prefs->CanSeeShares() == SEE_SHARE_EVERYBODY ||
						(theApp.glob_prefs->CanSeeShares() == SEE_SHARE_FRIENDS && client->IsFriend() ) )
					{
						AddDebugLogLine(false,IDS_SHAREDREQ1,client->GetUserName(),client->GetUserID(),GetResString(IDS_ACCEPTED) );
 
						CString strDir,strTest;

						// virtual folder list
						CArray<CString, CString&> arFolders;
						CMapStringToString &mapVDirs= theApp.sharedfiles->GetSharedVDirForList();
						POSITION pos= mapVDirs.GetStartPosition();
						while (pos)
						{
							CString strKey, strDir;
							mapVDirs.GetNextAssoc(pos, strKey, strDir);
							int i= 0;
							while ((i < arFolders.GetCount()) && (strKey.CompareNoCase(arFolders.GetAt(i)) > 0))
								i++;
							arFolders.InsertAt(i, strKey);
						}

						//build packet
						CSafeMemFile tempfile(80);
						uint32 uDirs = arFolders.GetCount();
						tempfile.Write(&uDirs, 4);
						for (int ix= 0; ix < arFolders.GetCount(); ix++)
						{
							const CString &strDir = arFolders.GetAt(ix);
							uint16 cnt = strDir.GetLength();
							tempfile.Write(&cnt, 2);
							tempfile.Write((LPCTSTR)strDir, cnt);
						}
						Packet* replypacket = new Packet(&tempfile);
						replypacket->opcode = OP_ASKSHAREDDIRSANS;
						theApp.uploadqueue->AddUpDataOverheadOther(replypacket->size);
						SendPacket(replypacket, true, true);
					}
					else
					{
						AddDebugLogLine(false,IDS_SHAREDREQ1,client->GetUserName(),client->GetUserID(),GetResString(IDS_DENIED) );
						Packet* replypacket = new Packet(OP_ASKSHAREDDENIEDANS, 0);
						theApp.uploadqueue->AddUpDataOverheadOther(replypacket->size);
						SendPacket(replypacket, true, true);
					}
					break;
				}
				case OP_ASKSHAREDFILESDIR:
				{
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					
					// IP banned, no answer for this request
					if (client->IsBanned() ) break;

					CSafeMemFile data((uchar*)packet, size);
					uint16 cnt;
					data.Read(&cnt, 2);
					CString strReqDir, strReqVDir;		// #zegzav:vdir4list
					data.Read(strReqVDir.GetBuffer(cnt), cnt);
					strReqVDir.ReleaseBuffer(cnt);
					PathRemoveBackslash(strReqVDir.GetBuffer());
					strReqVDir.ReleaseBuffer();
 
					// retrieve real dir from virtual dir
					CMapStringToString &mapVDirs= theApp.sharedfiles->GetSharedVDirForList();	// #zegzav:vdir4list
					bool bReqVDir= mapVDirs.Lookup(strReqVDir, strReqDir);

					if  ( (bReqVDir) &&
						  (theApp.glob_prefs->CanSeeShares() == SEE_SHARE_EVERYBODY ||
						   (theApp.glob_prefs->CanSeeShares() == SEE_SHARE_FRIENDS && client->IsFriend() ) ) )
					{
						AddDebugLogLine(false,IDS_SHAREDREQ2,client->GetUserName(),client->GetUserID(),strReqVDir,strReqDir,GetResString(IDS_ACCEPTED) );
						ASSERT( data.GetPosition() == data.GetLength() );

						CCKey bufKey;
						CKnownFile* cur_file;
						CTypedPtrList<CPtrList, CKnownFile*> list;
						
						for (POSITION pos = theApp.sharedfiles->m_Files_map.GetStartPosition();pos != 0;)
						{
							theApp.sharedfiles->m_Files_map.GetNextAssoc(pos, bufKey, cur_file);
							CString strSharedFileDir(cur_file->GetPath());
							PathRemoveBackslash(strSharedFileDir.GetBuffer());
							strSharedFileDir.ReleaseBuffer();
							if (strReqDir.CompareNoCase(strSharedFileDir) == 0) list.AddTail(cur_file);
						}
 
						// Currently we are sending each shared directory, even if it does not contain any files.
						// Because of this we also have to send an empty shared files list..
						/*if (list.GetCount())*/
						{
							CSafeMemFile tempfile(80);
							uint16 cnt = strReqVDir.GetLength();
							tempfile.Write(&cnt, 2);
							tempfile.Write((LPCTSTR)strReqVDir, cnt);
							uint32 uFiles = list.GetCount();
							tempfile.Write(&uFiles, 4);
							while (list.GetCount())
							{
								theApp.sharedfiles->CreateOfferedFilePacket(*list.GetHead(), tempfile);
								list.RemoveHead();
							}

							Packet* replypacket = new Packet(&tempfile);
							replypacket->opcode = OP_ASKSHAREDFILESDIRANS;
							theApp.uploadqueue->AddUpDataOverheadOther(replypacket->size);
							SendPacket(replypacket, true, true);
						}
					}
					else
					{
						AddDebugLogLine(false,IDS_SHAREDREQ2,client->GetUserName(),client->GetUserID(),strReqVDir,strReqDir,GetResString(IDS_DENIED) );
						Packet* replypacket = new Packet(OP_ASKSHAREDDENIEDANS, 0);
						theApp.uploadqueue->AddUpDataOverheadOther(replypacket->size);
						SendPacket(replypacket, true, true);
					}
					break;
				}
				case OP_ASKSHAREDDIRSANS:
				{
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					if (client->GetFileListRequested() == 1)
					{
						CSafeMemFile data((uchar*)packet, size);
						uint32 uDirs;
						data.Read(&uDirs, 4);
						for (UINT i = 0; i < uDirs; i++)
						{
							uint16 cnt;
							data.Read(&cnt, 2);
							CString strDir;
							data.Read(strDir.GetBuffer(cnt), cnt);
							strDir.ReleaseBuffer(cnt);
							// Better send the received and untouched directory string back to that client
							//PathRemoveBackslash(strDir.GetBuffer());
							//strDir.ReleaseBuffer();
							AddDebugLogLine(false,IDS_SHAREDANSW,client->GetUserName(),client->GetUserID(),strDir);

							CSafeMemFile tempfile(80);
							cnt = strDir.GetLength();
							tempfile.Write(&cnt, 2);
							tempfile.Write((LPCTSTR)strDir, cnt);
							Packet* replypacket = new Packet(&tempfile);
							replypacket->opcode = OP_ASKSHAREDFILESDIR;
							theApp.uploadqueue->AddUpDataOverheadOther(replypacket->size);
							SendPacket(replypacket, true, true);
						}

						ASSERT( data.GetPosition() == data.GetLength() );
						client->SetFileListRequested(uDirs);
					}
					else
						AddDebugLogLine(false,IDS_SHAREDANSW2,client->GetUserName(),client->GetUserID());

					break;
				}

				case OP_ASKSHAREDFILESDIRANS:
				{
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					CSafeMemFile data((uchar*)packet, size);
					uint16 cnt;
					data.Read(&cnt, 2);
					CString strDir;
					data.Read(strDir.GetBuffer(cnt), cnt);
					strDir.ReleaseBuffer(cnt);
					PathRemoveBackslash(strDir.GetBuffer());
					strDir.ReleaseBuffer();
					if (client->GetFileListRequested() > 0)
					{
						AddDebugLogLine(false,IDS_SHAREDINFO1,client->GetUserName(),client->GetUserID(),strDir);
						client->ProcessSharedFileList(packet + data.GetPosition(), size - data.GetPosition(), strDir);
						if (client->GetFileListRequested() == 0)
							AddDebugLogLine(false,IDS_SHAREDINFO2,client->GetUserName(),client->GetUserID());
					}
					else
						AddDebugLogLine(false,IDS_SHAREDANSW3,client->GetUserName(),client->GetUserID(),strDir);
					break;
				}
				case OP_ASKSHAREDDENIEDANS:
				{
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					ASSERT( size == 0 );
					AddLogLine(true,IDS_SHAREDREQDENIED,client->GetUserName(),client->GetUserID());
					client->SetFileListRequested(0);
					break;
				}

				case OP_UNKNOWNx6A:
				case OP_UNKNOWNx7A:
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					break;
				default:
					theApp.downloadqueue->AddDownDataOverheadOther(size);
					AddDebugLogLine(false,_T("Unknown opcode: %i %x from %s"), opcode, opcode, GetClientNameAndVersionString(client));
					PacketToDebugLogLine(_T("Unknown opcode"), packet, size, opcode);	// DbT:Logging
			}
		}
		catch(CFileException* error){
			OUTPUT_DEBUG_TRACE();
			error->Delete();	//mf
			throw GetResString(IDS_ERR_INVALIDPACKAGE);
		}
		catch(CMemoryException* error){
			OUTPUT_DEBUG_TRACE();
			error->Delete();
			throw CString(_T("Memory exception"));
		}
	}
	catch(CString error){
		OUTPUT_DEBUG_TRACE();
		if (client){
			client->SetDownloadState(DS_ERROR);	
			//TODO write this into a debugfile
			AddDebugLogLine(false,GetResString(IDS_ERR_CLIENTERROR)+ _T(" (%s)")
			,client->GetUserName(),client->GetFullIP(),error.GetBuffer(), GetClientNameAndVersionString(client));
			PacketToDebugLogLine(_T("Unknown client opcode"), packet, size, opcode);	// DbT:Logging
		}
		else {
			AddDebugLogLine(false,IDS_ERR_BADCLIENTACTION,error.GetBuffer());
			PacketToDebugLogLine(_T("Unknown protocol opcode"), packet, size, opcode);
		}
			
		Disconnect();
		return false;
	}
	return true;
}

bool CClientReqSocket::ProcessExtPacket(char* packet, uint32 size, uint8 opcode)
{
	try{
		try{
		if (!client)
			throw GetResString(IDS_ERR_UNKNOWNCLIENTACTION);
		switch(opcode){
			case OP_EMULEINFO:{
				theApp.downloadqueue->AddDownDataOverheadOther(size);
				client->ProcessMuleInfoPacket(packet,size);
				if (client->IsLeecherType())
					client->Ban(_T("AUTOMATED MESSAGE: You're being banned because your client is leeching!"), "Leecher MOD");
				else
				client->SendMuleInfoPacket(true);
				break;
			}
			case OP_EMULEINFOANSWER:{
				theApp.downloadqueue->AddDownDataOverheadOther(size);
				client->ProcessMuleInfoPacket(packet,size);
				if (client->IsLeecherType())
					client->Ban(_T("AUTOMATED MESSAGE: You're being banned because your client is leeching!"), "Leecher MOD");
				break;
			}
				case OP_SECIDENTSTATE:{
					client->ProcessSecIdentStatePacket((uchar*)packet,size);
					if (client->GetSecureIdentState() == IS_SIGNATURENEEDED)
						client->SendSignaturePacket();
					else if (client->GetSecureIdentState() == IS_KEYANDSIGNEEDED){
						client->SendPublicKeyPacket();
						client->SendSignaturePacket();
					}
					break;
				}
				case OP_PUBLICKEY:{
					if (client->IsBanned() ){
						break;						
					}
					client->ProcessPublicKeyPacket((uchar*)packet,size);
					break;
				}
  				case OP_SIGNATURE:{
					client->ProcessSignaturePacket((uchar*)packet,size);
					DEBUG_ONLY(AddDebugLogLine(false,"-->Signature Received"));
					break;
				}
			
			case OP_COMPRESSEDPART:
			{
//				theApp.downloadqueue->AddDownDataOverheadOther(24);
				if (client->reqfile && (client->reqfile->GetStatus()==PS_READY || client->reqfile->GetStatus()==PS_EMPTY))
				{
					client->ProcessBlockPacket(packet,size,true);
					if ((client->reqfile != NULL) &&
						(client->reqfile->GetStatus()==PS_PAUSED 
						|| client->reqfile->GetStatus()==PS_STOPPED
						|| client->reqfile->GetStatus()==PS_ERROR))
					{
						Packet* packet = new Packet(OP_CANCELTRANSFER,0);
						theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
						client->socket->SendPacket(packet,true,true);
					}
				}
				else
				{
					Packet* packet = new Packet(OP_CANCELTRANSFER,0);
					theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
					client->socket->SendPacket(packet,true,true);
					client->SetDownloadState(client->reqfile==NULL ? DS_NONE : DS_ONQUEUE);
				}
				break;
			}
			case OP_QUEUERANKING:{
				theApp.downloadqueue->AddDownDataOverheadOther(size);
				if (size != 12)
					throw GetResString(IDS_ERR_BADSIZE);
				uint16 newrank;
				#ifndef AMD
				memcpy(&newrank,packet+0,2);
				#else
				memcpy_amd(&newrank,packet+0,2);
				#endif
				client->SetRemoteQueueFull(false);
				client->SetRemoteQueueRank(newrank);
				// eklmn: switch DL state in case newrank != 0
				if (newrank && client->GetDownloadState() == DS_DOWNLOADING)
					client->SetDownloadState(DS_ONQUEUE);
				break;
			}
 			case OP_REQUESTSOURCES:{
				theApp.downloadqueue->AddDownDataOverheadSourceExchange(size);
				if (client->GetSourceExchangeVersion() >= 1)
				{
					if(size != 16)
						throw GetResString(IDS_ERR_BADSIZE);
	
					//first check shared file list, then download list
					CKnownFile* file = theApp.sharedfiles->GetFileByID((uchar*)packet);
					if(!file)
						file = theApp.downloadqueue->GetFileByID((uchar*)packet);
	
					if(file) 
					{
						DWORD dwTimePassed = ::GetTickCount() - client->GetLastSrcReqTime() + CONNECTION_LATENCY;
						bool bNeverAskedBefore = client->GetLastSrcReqTime() == 0;
	
						if( 
						    //if not complete and file is rare, allow once every 10 minutes
						    ( file->IsPartFile() &&
						     ((CPartFile*)file)->GetSourceCount() <= RARE_FILE * 2 &&
						      (bNeverAskedBefore || dwTimePassed > SOURCECLIENTREASK)
						    ) ||
						    //OR if file is not rare or if file is complete, allow every 90 minutes
						    ( (bNeverAskedBefore || dwTimePassed > SOURCECLIENTREASK * MINCOMMONPENALTY) )
						  ) {
	
							client->SetLastSrcReqTime();
							Packet* tosend = file->CreateSrcInfoPacket(client);
							if(tosend){
								theApp.uploadqueue->AddUpDataOverheadSourceExchange(tosend->size);
								SendPacket(tosend, true, true);
								//AddDebugLogLine(false, "RCV:Source Request User(%s) File(%s)", client->GetUserName(), file->GetFileName());
							}
						}
						}
				}
				break;
			}
 			case OP_ANSWERSOURCES:{
				theApp.downloadqueue->AddDownDataOverheadSourceExchange(size);
				try{
					CSafeMemFile data((BYTE*)packet,size);
					uchar hash[16];
					data.Read(hash,16);
					CKnownFile* file = theApp.downloadqueue->GetFileByID((uchar*)packet);
					if(file){
						if (file->IsPartFile()){
							//set the client's answer time
							client->SetLastSrcAnswerTime();
							//and set the file's last answer time
							((CPartFile*)file)->SetLastAnsweredTime();
	
							//v1:- eklmn: support Source Exchange v2
							((CPartFile*)file)->AddClientSources(&data, client->GetSourceExchangeVersion());
						}
					}
				}
				catch(CFileException* error){
					OUTPUT_DEBUG_TRACE();
					error->Delete();
					throw GetResString(IDS_ERR_INVALIDPACKAGE);
				}
				break;
			}
			case OP_FILEDESC:{
				theApp.downloadqueue->AddDownDataOverheadOther(size);
				client->ProcessMuleCommentPacket(packet,size);
				break;
			}
			/*case OP_VERIFYUPSREQ:{
				CUpDownClient* vuclient = theApp.clientlist->VUGetRandomClient();
				if (vuclient){
					Packet* answer = new Packet(OP_VERIFYUPSANSWER,6,OP_EMULEPROT);
					uint32 ip = vuclient->GetIP();
					memcpy(answer->pBuffer,ip,4);
					uint16 udpport = vuclient->GetUDPPort();
					memcpy(answer->pBuffer,ip,4);
					theApp.uploadqueue->AddUpDataRateMSOverhead(packet->size);
					SendPacket(answer,true,true);
				}
			}
			case OP_VERIFYUPSANSWER:{
				if (size != 12)
					throw CString("invalid size (OP_VERIFYUPSANSWER)");
			}*/
			default:
				theApp.downloadqueue->AddDownDataOverheadOther(size);
				AddDebugLogLine(false,_T("unknown extended opcode: %i %x from %s"),opcode,opcode, GetClientNameAndVersionString(client));
			  
			}
		}
		catch(CFileException* error){
			OUTPUT_DEBUG_TRACE();
			error->Delete();
			throw GetResString(IDS_ERR_INVALIDPACKAGE);
		}
	}
	catch(CString error){
		OUTPUT_DEBUG_TRACE();
        if(!client)
		    AddDebugLogLine(false,IDS_ERR_BADCLIENTACTION,error.GetBuffer());
        else
		    AddDebugLogLine(false,GetResString(IDS_ERR_BADCLIENTACTION)+ _T(" (%s)"),error.GetBuffer(), GetClientNameAndVersionString(client));
		if (client) client->SetDownloadState(DS_ERROR);		
//		Disconnect();
		return false;
	}
	return true;
}

void CClientReqSocket::OnInit(){
	uint8 tv = 1;
	//SetSockOpt(SO_DONTLINGER,&tv,sizeof(BOOL));
}

void CClientReqSocket::OnSend(int nErrorCode){
	ResetTimeOutTimer();
	CEMSocket::OnSend(nErrorCode);
}

void CClientReqSocket::OnError(int nErrorCode){

	if (client)
		AddDebugLogLine(false,IDS_ERR_BADCLIENT2
		,client->GetUserName(),client->GetFullIP(),nErrorCode);
	else
		AddDebugLogLine(false,IDS_ERR_BADCLIENTACTION,"CClientReqSocket::OnError");			
	Disconnect();
}

void CClientReqSocket::PacketReceived(Packet* packet)
{
	//v- eklmn: feature(01): Filter clients, which are violating protocol
#ifdef OLD_SOCKETS_ENABLED	
	SOCKADDR_IN sockAddr;
	memset(&sockAddr, 0, sizeof(sockAddr));
	uint32 nSockAddrLen = sizeof(sockAddr);
	GetPeerName((SOCKADDR*)&sockAddr,(int*)&nSockAddrLen);
	uint32 UserIP = sockAddr.sin_addr.S_un.S_addr;
	CString FullUserIP(inet_ntoa(sockAddr.sin_addr));
	if(theApp.ipfilter->IsFiltered(UserIP)) 
	{
		//AddDebugLogLine(false,_T("Filtered: %s %s "), FullUserIP,theApp.ipfilter->GetLastHit());
		theApp.stat_filteredclients++;
		//eklmn: reset client to NULL in order don't let CUpDownClient::Disconnected be processed for a client which is not in the list of clients.
		client = NULL;
		Disconnect();
		return;
	}
#endif //OLD_SOCKETS_ENABLED
	//^- eklmn: feature(01): Filter clients, which are violating protocol

	switch (packet->prot)
	{
		case OP_EDONKEYPROT:
			ProcessPacket(packet->pBuffer,packet->size,packet->opcode);
			break;
		case OP_PACKEDPROT:
			if (!packet->UnPackPacket()){
				ASSERT ( false );
				break;
			}
		case OP_EMULEPROT:
			ProcessExtPacket(packet->pBuffer,packet->size,packet->opcode);
			break;
		default:;
	}
}

void CClientReqSocket::OnReceive(int nErrorCode)
{
	ResetTimeOutTimer();
	CEMSocket::OnReceive(nErrorCode);
}

bool CClientReqSocket::Create(){
	theApp.listensocket->AddConnection();
	BOOL result = CAsyncSocketEx::Create(0,SOCK_STREAM,FD_WRITE|FD_READ|FD_CLOSE); // deadlake PROXYSUPPORT - changed to AsyncSocketEx
	OnInit();
	return result;
}



// CListenSocket
// CListenSocket member functions
CListenSocket::CListenSocket(CPreferences* in_prefs){
	app_prefs = in_prefs;
	opensockets = 0;
	maxconnectionreached = 0;
	m_OpenSocketsInterval = 0;
	m_nPeningConnections = 0;
}

CListenSocket::~CListenSocket(){
	Close();
	KillAllSockets();
}

bool CListenSocket::StartListening(){
	bListening = true;
	return (this->Create(app_prefs->GetPort(),SOCK_STREAM,FD_ACCEPT) && this->Listen());
}

void CListenSocket::ReStartListening(){
	bListening = true;
	if (m_nPeningConnections){
		m_nPeningConnections--;
		OnAccept(0);
	}
}

void CListenSocket::StopListening(){
	bListening = false;
	maxconnectionreached++;
}


void CListenSocket::OnAccept(int nErrorCode)
{
	if (!nErrorCode)
	{
		m_nPeningConnections++;
		if (m_nPeningConnections < 1){
			ASSERT ( false );
			m_nPeningConnections = 1;
		}
		if (TooManySockets(true) && !theApp.serverconnect->IsConnecting()){
			StopListening();
			return;
		}
		else if ( bListening == false )
			ReStartListening(); //If the client is still at maxconnections, this will allow it to go above it.. But if you don't, you will get a lowID on all servers.
	
		while (m_nPeningConnections)
		{
			m_nPeningConnections--;
			CClientReqSocket* newclient = new CClientReqSocket(app_prefs);
			if (!Accept(*newclient))
				newclient->Safe_Delete();
			else
			{
				newclient->AsyncSelect(FD_WRITE|FD_READ|FD_CLOSE);
				newclient->OnInit();
			}
			AddConnection();
		}
//		if (TooManySockets(true) && !theApp.serverconnect->IsConnecting())
//			StopListening();
	}

}

void CListenSocket::Process(){
	POSITION pos2;
	m_OpenSocketsInterval = 0;
	opensockets = 0;
	for(POSITION pos1 = socket_list.GetHeadPosition(); ( pos2 = pos1 ) != NULL; ){
		socket_list.GetNext(pos1);
		CClientReqSocket* cur_sock = socket_list.GetAt(pos2);
		opensockets++;

	   if (cur_sock->deletethis){
		   if (cur_sock->m_SocketData.hSocket != INVALID_SOCKET){ // deadlake PROXYSUPPORT - changed to AsyncSocketEx
				cur_sock->Close();
		   }
		   else{
			   cur_sock->Delete_Timed();;
		   }
	   }
	   else
		socket_list.GetAt( pos2 )->CheckTimeOut();
   }
   if ( (GetOpenSockets()+5 < app_prefs->GetMaxConnections() || theApp.serverconnect->IsConnecting()) && !bListening)
	   ReStartListening();
}

void CListenSocket::RecalculateStats(){
	memset(m_ConnectionStates,0,6);
	POSITION pos1,pos2;
	for(pos1 = socket_list.GetHeadPosition(); ( pos2 = pos1 ) != NULL; ){
		socket_list.GetNext(pos1);
		CClientReqSocket* cur_sock = socket_list.GetAt(pos2);
		switch (cur_sock->GetConState()){
			case ES_DISCONNECTED:
				m_ConnectionStates[0]++;
				break;
			case ES_NOTCONNECTED:
				m_ConnectionStates[1]++;
				break;
			case ES_CONNECTED:
				m_ConnectionStates[2]++;
				break;
		}
	}
}

void CListenSocket::AddSocket(CClientReqSocket* toadd){
	socket_list.AddTail(toadd);
}


void CListenSocket::RemoveSocket(CClientReqSocket* todel)
{
	POSITION todel_pos;
	todel_pos = socket_list.Find(todel);
	if (todel)
		socket_list.RemoveAt(todel_pos);
}

void CListenSocket::KillAllSockets(){
	for (POSITION pos = socket_list.GetHeadPosition();pos != 0;pos = socket_list.GetHeadPosition()){
		CClientReqSocket* cur_socket = socket_list.GetAt(pos);
		if (cur_socket->client)
			delete cur_socket->client;
		else
			delete cur_socket;
	}
}

void CListenSocket::AddConnection(){
	m_OpenSocketsInterval++;
	opensockets++;
}

bool CListenSocket::TooManySockets(bool bIgnoreInterval){
	if (GetOpenSockets() > app_prefs->GetMaxConnections() 
		|| (m_OpenSocketsInterval > (theApp.glob_prefs->GetMaxConperFive()*theApp.emuledlg->statisticswnd.GetMaxConperFiveModifier()) 
		&& !bIgnoreInterval) )
	{
		return true;
	}
	else
		return false;
}

bool CListenSocket::IsValidSocket(CClientReqSocket* totest){
	return socket_list.Find(totest);
}

void CListenSocket::Debug_ClientDeleted(CUpDownClient* deleted){
	POSITION pos1, pos2;
	for (pos1 = socket_list.GetHeadPosition();( pos2 = pos1 ) != NULL;){
		socket_list.GetNext(pos1);
		CClientReqSocket* cur_sock = socket_list.GetAt(pos2);
		if (!AfxIsValidAddress(cur_sock, sizeof(CClientReqSocket))) {
			AfxDebugBreak(); 
		}
		if (cur_sock->client == deleted){
			AfxDebugBreak();
		}
	}
}
#endif //OLD_SOCKETS_ENABLED
