//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "partfile.h"
#include "emule.h"
#include "updownclient.h"
#include <math.h>
#include "ED2KLink.h"
#include "Preview.h"
#include "ini2.h"
#include "ArchiveRecovery.h"
#include <sys/stat.h>
#include <io.h>
#include "Smtp.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


// Barry - use this constant for both places
#define PROGRESS_HEIGHT 4	//netwolf 06.05.03 (was 3)
/*	Maella - removed mem leak
CBarShader CPartFile::s_LoadBar(PROGRESS_HEIGHT); // Barry - was 5
CBarShader CPartFile::s_ChunkBar(16); 
*/
CMutex CPartFile::s_ComplitingMutex;

CPartFile::CPartFile()
{
	Init();
}

CPartFile::CPartFile(CSearchFile* searchresult)
{
	Init();

	EMULE_TRY

	#ifndef AMD
	md4cpy(filehash,searchresult->GetFileHash());
	#else
	memcpy_amd(filehash,searchresult->GetFileHash(),16);
	#endif
	for (int i = 0; i != searchresult->taglist.GetCount();i++){
		switch (searchresult->taglist[i]->GetSpecialTag()){
			case FT_FILENAME:{
				filename = searchresult->taglist[i]->GetStringValue();
				break;
			}
			case FT_FILESIZE:{
				filesize = searchresult->taglist[i]->GetIntValue();
				break;
			}
			default:
				CTag* newtag = new CTag(*searchresult->taglist[i]);
				taglist.Add(newtag);
		}
	}
	// Don't ask for a hashset, if there is none.
	if (filesize < PARTSIZE)
		hashsetneeded = false;
	CreatePartFile();

	EMULE_CATCH
}

CPartFile::CPartFile(CString edonkeylink)
{
	Init();

	EMULE_TRY

	CED2KLink* pLink = 0;
	try {
		pLink = CED2KLink::CreateLinkFromUrl(edonkeylink);
		_ASSERT( pLink != 0 );
		CED2KFileLink* pFileLink = pLink->GetFileLink();
		if (pFileLink==0) 
			throw GetResString(IDS_ERR_NOTAFILELINK);
		InitializeFromLink(pFileLink);
	} catch (CString error) {
		OUTPUT_DEBUG_TRACE();
		CString buffer;
		buffer.Format(GetResString(IDS_ERR_INVALIDLINK),error.GetBuffer());
		AddLogLine(true, IDS_ERR_LINKERROR, buffer);
		SetStatus(PS_ERROR);
	}
	delete pLink;

	EMULE_CATCH
}

void CPartFile::InitializeFromLink(CED2KFileLink* fileLink)
{
	Init();

	EMULE_TRY

	try{
		filename = CString( fileLink->GetName() );
		filesize = fileLink->GetSize();
		// Ultras: prevent to donload an empty file
		if( !filesize ) {
			AddLogLine( true, IDS_SKIPZEROLENGTHFILE, filename );
			SetStatus(PS_ERROR);
			return;
		}
		// Don't ask for a hashset, if there is none.
		if (filesize < PARTSIZE)
			hashsetneeded = false;
		#ifndef AMD
		memcpy(filehash,fileLink->GetHashKey(),sizeof(filehash));
		#else
		memcpy_amd(filehash,fileLink->GetHashKey(),sizeof(filehash));
		#endif
		if (!theApp.downloadqueue->IsFileExisting(filehash))
			CreatePartFile();
		else
			SetStatus(PS_ERROR);
		}
	catch(CString error){
		OUTPUT_DEBUG_TRACE();
		CString buffer;
		buffer.Format(GetResString(IDS_ERR_INVALIDLINK),error.GetBuffer());
		AddLogLine(true, IDS_ERR_LINKERROR, buffer);
		SetStatus(PS_ERROR);
	}

	EMULE_CATCH
}

CPartFile::CPartFile(CED2KFileLink* fileLink)
{
	Init();
	InitializeFromLink(fileLink);
}

void CPartFile::Init()
{
	EMULE_TRY

	tempdir	=	"";	
	newdate = true;
	lastsearchtime = 0;
	lastpurgetime = ::GetTickCount();
	paused = false;
	stopped = false;
	SetStatus(PS_EMPTY);
	transfered = 0;
	priority = PR_NORMAL;
	srcarevisible = false;
	//eklmn(khaos): reset a statistical variables
	m_iSrcNNP =	0;
	m_iSrcOnQueue =	0;
	m_iSrcHighQR = 0;
	m_iSrcConnecting = 0;
	m_iSrcTooManyConns = 0;
	m_iSrcConnected = 0;
	m_iSrcTimeout = 0;
	m_iSrcConnViaServer = 0;
	m_iSrcUnknown =	0;
	m_iSrcRecHash = 0;
	m_iSrcLowToLow =	 0;
	m_iSrcQueueFull = 0;
	m_iSrcNoUpload = 0;
	m_iSrcBanned = 0;
	m_iSrcError = 0;
	m_iSrcA4AF = 0;
	datarate = 0;
	hashsetneeded = true;
	count = 0;
	percentcompleted = 0;
	partmetfilename = "";
	completedsize=0;

	m_bPreviewing=false;
	lastseencomplete=NULL;
	availablePartsCount=0;
	m_ClientSrcAnswered = 0;
	m_LastNoNeededCheck = 0;
	m_iRate=0;
	m_strComment="";
	m_nTotalBufferData = 0;
	m_nLastBufferFlushTime = 0;
	m_bPercentUpdated = false;
	m_bRecoveringArchive = false;
	m_iGainDueToCompression = 0;
	m_iLostDueToCorruption = 0;
	m_iTotalPacketsSavedDueToICH = 0;
	m_nSavedReduceDownload = 0;
	hasRating=false;
	hasComment=false;
	m_bDiscardSuperCompressed = false;

	// Tarod
	if (theApp.glob_prefs->IsDAPEnabled())
		m_bAutoPriority = true;
	else
		m_bAutoPriority = false;

	m_LastRemovedTime = 0;
	m_nLastCompleteSrcCount= 0;			// netwolf: complete sources (zegzav) 13.05.03
	m_bUpdateCompleteSrcCount= false;
	m_LastDownTransfer = CTime::GetCurrentTime();
	//bond006 (smoothed "remaining time" calculation) //SyruS
	m_SessionStartTime = m_LastDownTransfer;
	m_SessionStartSize = 0;
	m_AvgDataRate = 0;
	m_StartTimeReset = false;
	m_iCategory = 0;

	EMULE_CATCH
}

CPartFile::~CPartFile()
{
	EMULE_TRY

	// Barry - Ensure all buffered data is written
	FlushBuffer();
	//taab BEGIN fix - minimize rehashing of files //netwolf 03.05.03 (dont know if we really need it)
	if (m_hpartfile.m_hFile != INVALID_HANDLE_VALUE)
	{
		m_hpartfile.Close();
		// Update met file
		SavePartFile();
	}
	//taab END fix	
	m_SrcpartFrequency.RemoveAll();
	m_ReqParts.RemoveAll();
	m_BlockedParts.RemoveAll();
	for (POSITION pos = gaplist.GetHeadPosition();pos != 0;gaplist.GetNext(pos))
		delete gaplist.GetAt(pos);

	EMULE_CATCH
}

void CPartFile::CreatePartFile()
{
	EMULE_TRY

	tempdir = theApp.glob_prefs->GetTempDir();	// added by InterCeptor (multiple tempdirs) 19.11.02
	if (tempdir.Right(1) == '\\')
		tempdir.Truncate(tempdir.GetLength() - 1);	// added by InterCeptor (check for \\) 19.11.02
	directory = tempdir;	// #zegzav:vdir4list (OP_ASKSHAREDFILESDIR)
  	int i = 0; 
	CString filename; 
	do 
	{ 
		i++; 
		filename.Format(_T("%s\\%03i.part"), tempdir, i); 
	
	}while(PathFileExists(filename)); 
	
	partmetfilename.Format(_T("%03i.part.met"),i); 	
	fullname.Format(_T("%s\\%s"),tempdir,partmetfilename);
	CString buffer = partmetfilename.Left(partmetfilename.GetLength() - 4);	
	CTag* partnametag = new CTag(FT_PARTFILENAME,buffer);	
	taglist.Add(partnametag);
	
	Gap_Struct* gap = new Gap_Struct;
	gap->start = 0;
	gap->end = filesize-1;
	gaplist.AddTail(gap);

	CString partfull = fullname.Left(fullname.GetLength() - 4);
	if (!m_hpartfile.Open(partfull,CFile::modeCreate|CFile::modeReadWrite|CFile::shareDenyWrite|CFile::osSequentialScan)){
		AddLogLine(false, IDS_ERR_CREATEPARTFILE);
		SetStatus(PS_ERROR);
	}

	const uint16 partCount = GetPartCount();
	m_SrcpartFrequency.SetSize(partCount);
	m_BlockedParts.SetSize(partCount);	// eklmn: ICR 2.9
	m_ReqParts.SetSize(partCount);	//eklmn: ICR 2.10 
	for (uint32 i = 0; i != partCount;i++) {
		m_SrcpartFrequency.Add(0);
		m_BlockedParts[i] = NULL;
		m_ReqParts[i] = 0;		// ICR 2.10
	}
	paused = false;

	// Before we save file we need to stablish UAP by Tarod
	if (theApp.glob_prefs->IsUAPEnabled()) {
		CKnownFile::SetAutoPriority(true);
		CKnownFile::UpdateUploadAutoPriority();
		theApp.emuledlg->sharedfileswnd.sharedfilesctrl.UpdateItem((CKnownFile*)this);
	}
	// End Tarod
	
	// SLUGFILLER: hashlistRecheck
	if(filesize <= PARTSIZE )
		hashsetneeded = false;
	// SLUGFILLER: hashlistRecheck

	//SyruS: cleanup
	if (theApp.glob_prefs->GetAutoFilenameCleanup()) SetFileName( CleanupFilename(GetFileName()).GetBuffer() );

	SavePartFile();

	EMULE_CATCH
}

bool CPartFile::LoadPartFile(LPCTSTR in_directory,LPCTSTR in_filename)
{
	EMULE_TRY

	tempdir = in_directory;	// added by InterCeptor (multiple tempdirs) 19.11.02
    CMap<uint16,uint16,Gap_Struct*,Gap_Struct*> gap_map; // safe Load Gaps [SlugFiller]
	transfered = 0;
	partmetfilename = in_filename;
	directory = in_directory;
	if (directory.Right(1) == '\\')
		directory.Truncate(directory.GetLength() - 1);	// added by InterCeptor (check for \\) 19.11.02
	
	fullname.Format(_T("%s\\%s"),directory,partmetfilename);
	CSafeFile file;
	try{
		// readfile data form part.met file
		if (!file.Open(fullname,CFile::modeRead)){
			AddLogLine(false, IDS_ERR_OPENMET, partmetfilename,filename);
			return false;
		}
		uint8 version;
		file.Read(&version,1);
		if (version != PARTFILE_VERSION){
			if (file.m_hFile != INVALID_HANDLE_VALUE) file.Close();	//eklmn: bugfix(21): *.part.met was't closed after loading (aka "corrupted file problem")
			AddLogLine(false, IDS_ERR_BADMETVERSION,partmetfilename,filename);
			return false;
		}
		LoadDateFromFile(file);
        if(!LoadHashsetFromFile(file,false)) {
            AddLogLine(true,IDS_ERR_HASHSETCORR,in_filename);
        }
		LoadMovieMode(); // DonGato - Moview Preview Mode

		uint32 tagcount;
		file.Read(&tagcount,4);
		for (uint32 j = 0; j != tagcount;j++){
			CTag* newtag = new CTag(file);
			switch(newtag->GetSpecialTag()){
				case FT_FILENAME:{
					if(newtag->GetStringValue().IsEmpty()) {
						AddLogLine(true, IDS_ERR_METCORRUPT, partmetfilename,filename);
						delete newtag;
						return false;
					}
					filename = newtag->GetStringValue();
					delete newtag;
					break;
				}
				case FT_LASTSEENCOMPLETE: {
					lastseencomplete = newtag->GetIntValue();
					delete newtag;
					break;
				}
				case FT_FILESIZE:{
					filesize = newtag->GetIntValue();
					delete newtag;
					break;
				}
				case FT_TRANSFERED:{
					transfered = newtag->GetIntValue();
					delete newtag;
					break;
				}
				case FT_PRIORITY:{
					priority = newtag->GetIntValue();
					delete newtag;

					// Added by Tarod for auto-priority
					if (priority == PR_AUTO)
						SetAutoPriority(true);
					else
						SetAutoPriority(false);
					break;
				}
				case FT_STATUS:{
					paused = newtag->GetIntValue();
					delete newtag;
					break;
				}
				case FT_ULPRIORITY: {
					if ((uint8)newtag->GetIntValue() == PR_AUTO) {
						CKnownFile::SetAutoPriority(true);
						CKnownFile::UpdateUploadAutoPriority();
					} else {
						CKnownFile::SetAutoPriority(false);
						CKnownFile::SetPriority((uint8)newtag->GetIntValue());
					}
					delete newtag;
					break;
				}
				case FT_CATEGORY:
				{
					m_iCategory = newtag->GetIntValue();
					delete newtag;
					break;
				}
				default:{
					// Start Changes by Slugfiller for better exception handling
					if ((!newtag->GetSpecialTag()) && (newtag->GetTagName()[0] == FT_GAPSTART ||
					     newtag->GetTagName()[0] == FT_GAPEND)){
						Gap_Struct* gap;
						CString s = newtag->GetTagName();
						s = s.Right(s.GetLength()-1);
						uint16 gapkey = _tstoi(s);
						if (!gap_map.Lookup(gapkey, gap))
						{
							gap = new Gap_Struct;
							gap_map.SetAt(gapkey, gap);
							gap->start = -1;
							gap->end = -1;
						}
						if (newtag->GetTagName()[0] == FT_GAPSTART)
							gap->start = newtag->GetIntValue();
						if (newtag->GetTagName()[0] == FT_GAPEND)
							gap->end = newtag->GetIntValue()-1;
						delete newtag;
					// End Changes by Slugfiller for better exception handling
					}
					else
						taglist.Add(newtag);
				}
					
				// START - enkeyDEV(Ottavio84) -ChangeDir-
				CString dfpath = CString(fullname) + CString(".dir");
				CStdioFile f;
				if (f.Open(dfpath, CFile::modeRead | CFile::typeText)) {
					f.ReadString(m_alternativepath);
					f.Close();			
				}
				// END - enkeyDEV(Ottavio84) -ChangeDir-					
			}

		}
		if (file.m_hFile != INVALID_HANDLE_VALUE) file.Close();	//eklmn: bugfix(21): *.part.met was't closed after loading (aka "corrupted file problem")
	}
	catch(CFileException* error){
		OUTPUT_DEBUG_TRACE();
		if (error->m_cause == CFileException::endOfFile)
			AddLogLine(true, IDS_ERR_METCORRUPT, partmetfilename,filename);
		else{			
			AddLogLine(true, IDS_ERR_FILEERROR,partmetfilename,filename,error);
		}
		error->Delete();		
		for (POSITION pos = gap_map.GetStartPosition(); pos != NULL; ) {
			Gap_Struct* gap;
			uint16 gapkey;
 			gap_map.GetNextAssoc(pos, gapkey, gap);
			delete gap;
		}
		return false;
	}

	// Now to flush the map into the list (Slugfiller)
	for (POSITION pos = gap_map.GetStartPosition(); pos != NULL; ){
		Gap_Struct* gap;
		uint16 gapkey;
 		gap_map.GetNextAssoc(pos, gapkey, gap);
		//SyruS (0.29a) SLUGFILLER: SafeHash - revised code, and extra safety
		/*if (gap->start >= 0 && gap->end >=0 && gap->start <= gap->end)
        		gaplist.AddTail(gap); // All tags accounted for
    		else
			delete gap; // Some of the tags were missing*/
		if (gap->start != -1 && gap->end != -1 && gap->start <= gap->end && gap->start < filesize){
			if (gap->end >= filesize)
				gap->end = filesize-1; // Clipping
			AddGap(gap->start, gap->end); // All tags accounted for, use safe adding
		}
		delete gap;
		// SLUGFILLER: SafeHash
  }

	EMULE_TRY
	//--- xrmb:keepPartFileStats ---
	//--- i hope that is a safe point to load the stats ---
	LoadPartFileStats();
	//--- :xrmb ---
	EMULE_CATCH

	//check if this is a backup
	if(fullname.Right(7).CompareNoCase(_T(".backup")) == 0) {
		fullname.Truncate(fullname.GetLength()-7);
	}

	// open permanent handle
	CString searchpath = fullname.Left(fullname.GetLength() - 4);	
	if (!m_hpartfile.Open(searchpath,CFile::modeReadWrite|CFile::shareDenyWrite|CFile::osSequentialScan)){
		AddLogLine(false,IDS_ERR_FILEOPEN,fullname,filename);		
		return false;
	}			

	//SyruS (0.29a) SLUGFILLER: SafeHash - final safety, make sure any missing part of the file is gap
	if (m_hpartfile.GetLength() < filesize)
		AddGap(m_hpartfile.GetLength(), filesize-1);
	// Goes both ways - Partfile should never be too large
	if (m_hpartfile.GetLength() > filesize){
		TRACE("Partfile \"%s\" is too large! Truncating %I64u bytes.\n", GetFileName(), m_hpartfile.GetLength() - filesize);
		m_hpartfile.SetLength(filesize);
	}
	// SLUGFILLER: SafeHash

	const uint16 partCount = GetPartCount();
	m_SrcpartFrequency.SetSize(partCount);
	m_BlockedParts.SetSize(partCount);	//eklmn: ICR2.9+
	m_ReqParts.SetSize(partCount); //eklmn: ICR 2.10
	for (uint32 i = 0; i != partCount; i++) {
		m_SrcpartFrequency.Add(0);
		m_BlockedParts[i] = NULL;
		m_ReqParts[i] = 0;		// ICR 2.10
	}
	SetStatus(PS_READY);
	// check hashcount, filesatus etc
	CSingleLock Lock(&m_hashlistMutex,TRUE);
	INT_PTR iSize = hashlist.GetCount();
	Lock.Unlock();
	if ( iSize < GetPartCount() && GetFileSize() >= PARTSIZE){
		SetStatus(PS_EMPTY);
		hashsetneeded = true;
		return true;
	}
	else {
		hashsetneeded = false;
		bool empty = true;
		for (int i = 0; i != iSize; i++){
			if (IsComplete(i*PARTSIZE,((i+1)*PARTSIZE)-1))
				empty = false;
		}
		if (empty)
			SetStatus(PS_EMPTY);
	}

	if (gaplist.IsEmpty()){	// is this file complete already?
		CompleteFile(false);
		return true;
	}

	// check date of .part file - if its wrong, rehash file
	CFileStatus filestatus;
	m_hpartfile.GetStatus(filestatus); // this; "...returns m_attribute without high-order flags" indicates a known MFC bug, wonder how many unknown there are... :)
        m_LastDownTransfer = filestatus.m_mtime;
		if (date != mktime(filestatus.m_mtime.GetLocalTm())){
			AddLogLine(false,IDS_ERR_REHASH,filename);
			// rehash
			SetStatus(PS_WAITINGFORHASH);
		    CAddFileThread* addfilethread = (CAddFileThread*) AfxBeginThread(RUNTIME_CLASS(CAddFileThread), THREAD_PRIORITY_NORMAL+ theApp.glob_prefs->GetMainProcessPriority(),0, CREATE_SUSPENDED);
		    addfilethread->SetValues(0,directory,m_hpartfile.GetFileName().GetBuffer(),this);
		    addfilethread->ResumeThread();
		}

	UpdateCompletedInfos();
	if ( completedsize > transfered )
		m_iGainDueToCompression = completedsize - transfered;
	else if ( completedsize != transfered )
		m_iLostDueToCorruption = transfered - completedsize;
	//bond006 (smoothed "remaining time" calculation) //SyruS
	m_SessionStartSize = completedsize;
	if (paused) SetStartTimeReset(true);

	return true;

	EMULE_CATCH

	return false;
}

bool CPartFile::SavePartFile()
{
	EMULE_TRY

	switch (status){
		case PS_WAITINGFORHASH:
		case PS_HASHING:
		case PS_COMPLETING:
			return false;
	}
	FILE* file = 0;
	try{
		//backup part.met file
		CString bakname;
		bakname.Format(_T("%s\\%s.bak"), theApp.glob_prefs->GetTempDir(), GetPartMetFileName());
		::CopyFile(fullname, bakname, FALSE);
		
		//get filedate
		CFileFind ff;
		CString searchpath = fullname.Left(fullname.GetLength() - 4);		
		bool end = !ff.FindFile(searchpath,0);		
		if (!end)
				ff.FindNextFile();
		if (end || ff.IsDirectory())
			throw GetResString(IDS_ERR_PART_FNF);
		CTime lwtime;
		ff.GetLastWriteTime(lwtime);
		date = mktime(lwtime.GetLocalTm());
		ff.Close();
		uint32 lsc = mktime(lastseencomplete.GetLocalTm());
		CString backupName(fullname);
		backupName.Append(_T(".backup"));

		_tremove(backupName);
		_trename(fullname, backupName);

		// readfile data form part.met file
		file = _tfopen(fullname,_T("wbS"));
		if (!file)
			throw GetResString(IDS_ERR_OPENMETFILE);
		//version
		uint8 version = PARTFILE_VERSION;
		fwrite(&version,1,1,file);
		//date
		fwrite(&date,4,1,file);
		//hash
		fwrite(&filehash,16,1,file);
		CSingleLock Lock(&m_hashlistMutex,TRUE);
		uint16 parts = hashlist.GetCount();
		fwrite(&parts,2,1,file);
		for (int x = 0; x != parts; x++)
			fwrite(hashlist[x],16,1,file);
		Lock.Unlock();
		//tags
		uint32 tagcount = taglist.GetCount()+8+(gaplist.GetCount()*2);
		fwrite(&tagcount,4,1,file);
		CTag nametag(FT_FILENAME,filename);
		nametag.WriteTagToFile(file);		
		CTag sizetag(FT_FILESIZE,filesize);
		sizetag.WriteTagToFile(file);		
		CTag transtag(FT_TRANSFERED,transfered);
		transtag.WriteTagToFile(file);		
		CTag statustag(FT_STATUS,(paused)? 1:0);
		statustag.WriteTagToFile(file);		
		// Modified by Tarod for DAP		
		uint8 priority_value;
		if (IsAutoPrioritized())
			priority_value = PR_AUTO;
		else 
			priority_value = priority;
		CTag prioritytag(FT_PRIORITY,priority_value);
		prioritytag.WriteTagToFile(file);		
		// End by Tarod
		CTag lsctag(FT_LASTSEENCOMPLETE,lsc);
		lsctag.WriteTagToFile(file);		

		// Modified by Tarod for UAP		
		uint8 ulpriority_value;
		if (CKnownFile::IsAutoPrioritized())
			ulpriority_value = PR_AUTO;
		else
			ulpriority_value = m_iPriority;		
		CTag ulprioritytag(FT_ULPRIORITY,ulpriority_value);
		ulprioritytag.WriteTagToFile(file);

		CTag categorytag(FT_CATEGORY, m_iCategory);
		categorytag.WriteTagToFile(file);
	
		// End by Tarod
		for (uint32 j = 0; j != taglist.GetCount();j++)
			taglist[j]->WriteTagToFile(file);
		//gaps
		TCHAR namebuffer[10];
		TCHAR *number = &namebuffer[1];
		uint16 i_pos = 0;
		for (POSITION pos = gaplist.GetHeadPosition();pos != 0;gaplist.GetNext(pos)){
			_itot(i_pos,number,10);
			namebuffer[0] = FT_GAPSTART;
			CTag gapstarttag(namebuffer,gaplist.GetAt(pos)->start);
			gapstarttag.WriteTagToFile(file);
			// gap start = first missing byte but gap ends = first non-missing byte in edonkey
			// but I think its easier to user the real limits
			namebuffer[0] = FT_GAPEND;
			CTag gapendtag(namebuffer,(gaplist.GetAt(pos)->end)+1);
			gapendtag.WriteTagToFile(file);			
			i_pos++;
		}

		// START - enkeyDEV(Ottavio84) -ChangeDir-
		if (!m_alternativepath.IsEmpty()) {
			CString dfpath = CString(fullname) + CString(_T(".dir"));
			CStdioFile f;
			if (f.Open(dfpath, CFile::modeCreate | CFile::modeWrite | CFile::typeText)) {
				f.WriteString((LPCTSTR)m_alternativepath);
				f.Close();			
			}
		}
		// END - enkeyDEV(Ottavio84) -ChangeDir-

		if (ferror(file))
			throw CString(_T("unexpected write error"));

		_tremove(backupName);
	}	
	catch(char* error){
		OUTPUT_DEBUG_TRACE();
		if (file)
			fclose(file);
		AddLogLine(false,IDS_ERR_SAVEMET,error,partmetfilename,filename);
		delete [] error;
		return false;
	}
	catch(CString error){
		OUTPUT_DEBUG_TRACE();
		if (file)
			fclose(file);
		AddLogLine(false,IDS_ERR_SAVEMET,error.GetBuffer(),partmetfilename,filename);
		return false;
	}
	// v1:- eklmn: bugfix(27): by shutdown force OS flush data direct to disk [SlugFiller]
	if ( !theApp.emuledlg->IsRunning() ) _commit(_fileno(file));
	fclose(file);
	return true;

	EMULE_CATCH

	return false;
}

void CPartFile::PartFileHashFinished(CKnownFile* result)
{
	EMULE_TRY

	newdate = true;
	bool errorfound = false;
	CSingleLock Lock(&m_hashlistMutex,TRUE);
	INT_PTR hashSize = hashlist.GetSize();
	for (uint32 i = 0; i != hashSize; i++){
		if (IsComplete(i*PARTSIZE,((i+1)*PARTSIZE)-1)){
			if (!(result->GetPartHash(i) && !md4cmp(result->GetPartHash(i),GetPartHash(i)))){
				AddLogLine(false,IDS_ERR_FOUNDCORRUPTION,i+1,filename);		
				AddGap(i*PARTSIZE,((((i+1)*PARTSIZE)-1) >= filesize) ? filesize-1 : ((i+1)*PARTSIZE)-1);
				errorfound = true;
			}
		}
	}
	Lock.Unlock();
	delete result;
	if (!errorfound){
		if (status == PS_COMPLETING){
			CompleteFile(true);
			return;
		}
		else
			AddLogLine(false,IDS_HASHINGDONE,filename);
	}
	else{
		SetStatus(PS_READY);
		SavePartFile();
		return;
	}
	SetStatus(PS_READY);
	SavePartFile();
	theApp.sharedfiles->SafeAddKFile(this);

	EMULE_CATCH
}

void CPartFile::AddGap(uint32 start, uint32 end)
{
	EMULE_TRY

	POSITION pos1, pos2;
	for (pos1 = gaplist.GetHeadPosition();(pos2 = pos1) != NULL;){
		Gap_Struct* cur_gap = gaplist.GetNext(pos1);
		if (cur_gap->start >= start && cur_gap->end <= end){ // this gap is inside the new gap - delete
			gaplist.RemoveAt(pos2);
			delete cur_gap;
			continue;
		}
		else if (cur_gap->start >= start && cur_gap->start <= end){// a part of this gap is in the new gap - extend limit and delete
			end = cur_gap->end;
			gaplist.RemoveAt(pos2);
			delete cur_gap;
			continue;
		}
		else if (cur_gap->end <= end && cur_gap->end >= start){// a part of this gap is in the new gap - extend limit and delete
			start = cur_gap->start;
			gaplist.RemoveAt(pos2);
			delete cur_gap;
			continue;
		}
		else if (start >= cur_gap->start && end <= cur_gap->end){// new gap is already inside this gap - return
			return;
		}
	}
	Gap_Struct* new_gap = new Gap_Struct;
	new_gap->start = start;
	new_gap->end = end;
	gaplist.AddTail(new_gap);
	//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
	UpdateDisplayedInfo();
	newdate = true;

	EMULE_CATCH
}

bool CPartFile::IsComplete(uint32 start, uint32 end)
{
	EMULE_TRY

	if (end >= filesize)
		end = filesize-1;
	for (POSITION pos = gaplist.GetHeadPosition();pos != 0;gaplist.GetNext(pos)){
		Gap_Struct* cur_gap = gaplist.GetAt(pos);
		if ((cur_gap->start >= start && cur_gap->end <= end)||(cur_gap->start >= start 
			&& cur_gap->start <= end)||(cur_gap->end <= end && cur_gap->end >= start)
			||(start >= cur_gap->start && end <= cur_gap->end)){
				return false;	
			}
	}
	return true;

	EMULE_CATCH

	return false;
}

bool CPartFile::IsPureGap(uint32 start, uint32 end)
{
	EMULE_TRY

	if (end >= filesize)
		end = filesize-1;
	for (POSITION pos = gaplist.GetHeadPosition();pos != 0;gaplist.GetNext(pos)){
		Gap_Struct* cur_gap = gaplist.GetAt(pos);
		if (start >= cur_gap->start  && end <= cur_gap->end ){
			return true;
		}
	}

	EMULE_CATCH

	return false;
}

bool CPartFile::IsAlreadyRequested(uint32 start, uint32 end)
{
	EMULE_TRY

	for (POSITION pos =  requestedblocks_list.GetHeadPosition();pos != 0; requestedblocks_list.GetNext(pos)){
		Requested_Block_Struct* cur_block =  requestedblocks_list.GetAt(pos);
		if ((start <= cur_block->EndOffset) && (end >= cur_block->StartOffset))
			return true;
	}

	EMULE_CATCH

	return false;
}


bool CPartFile::GetNextEmptyBlockInPart(uint16 partNumber, Requested_Block_Struct *result)
{
	EMULE_TRY

	Gap_Struct *firstGap;
	Gap_Struct *currentGap;
	uint32 end;
	uint32 blockLimit;

	// Find start of this part
	uint32 partStart = (PARTSIZE * partNumber);
	uint32 start = partStart;

	// What is the end limit of this block, i.e. can't go outside part (or filesize)
	uint32 partEnd = (PARTSIZE * (partNumber + 1)) - 1;
	if (partEnd >= GetFileSize())
		partEnd = GetFileSize() - 1;

	// Loop until find a suitable gap and return true, or no more gaps and return false
	while (true)
	{
		firstGap = NULL;

		// Find the first gap from the start position
		for (POSITION pos = gaplist.GetHeadPosition(); pos != 0; gaplist.GetNext(pos))
		{
			currentGap = gaplist.GetAt(pos);
			// Want gaps that overlap start<->partEnd
			if ((currentGap->start <= partEnd) && (currentGap->end >= start))
			{
				// Is this the first gap?
				if ((firstGap == NULL) || (currentGap->start < firstGap->start))
					firstGap = currentGap;
			}
		}

		// If no gaps after start, exit
		if (firstGap == NULL)
			return false;

		// Update start position if gap starts after current pos
		if (start < firstGap->start)
			start = firstGap->start;

		// If this is not within part, exit
		if (start > partEnd)
			return false;

		// Find end, keeping within the max block size and the part limit
		end = firstGap->end;
		blockLimit = partStart + (EMBLOCKSIZE * (((start - partStart) / EMBLOCKSIZE) + 1)) - 1;
		if (end > blockLimit)
			end = blockLimit;
		if (end > partEnd)
			end = partEnd;
    
		// If this gap has not already been requested, we have found a valid entry
		if (!IsAlreadyRequested(start, end))
		{
			// Was this block to be returned
			if (result != NULL)
			{
				result->StartOffset = start;
				result->EndOffset = end;
				#ifndef AMD
				md4cpy(result->FileID, GetFileHash());
				#else
				memcpy_amd(result->FileID, GetFileHash(), 16);
				#endif
				result->transferred = 0;
			}
			return true;
		}
		else
		{
			// Reposition to end of that gap
			start = end + 1;
		}

		// If tried all gaps then break out of the loop
		if (end == partEnd)
			break;
	}

	EMULE_CATCH

	// No suitable gap found
	return false;
}


void CPartFile::GetGapsInPart(CUpDownClient* sender, uint16 numPart, Requested_Block_Struct** newblocks, uint16* count) 	//jicxicmic
{
	EMULE_TRY

	// Calculate offsets of the Part
	const uint32 uPartStart = numPart * PARTSIZE;
	const uint32 uPartEnd  = ((GetFileSize() - 1) < (uPartStart + PARTSIZE - 1)) ? (GetFileSize() - 1) : (uPartStart + PARTSIZE - 1);

	int newblockcount = 0;

	// Retrieve Gap(s) matching the part
	for (POSITION pos = gaplist.GetHeadPosition(); pos != NULL;)
	{
		Gap_Struct *CurGap = gaplist.GetNext(pos);

		// Check if Gap is inside the limit
		if (CurGap->start > uPartEnd || CurGap->end < uPartStart)
			continue;

		// Calculate offsets of the Gap within the part
		uint32 uGapStart = (CurGap->start < uPartStart) ? uPartStart : CurGap->start;
		const uint32 uGapEnd = (CurGap->end > uPartEnd) ? uPartEnd : CurGap->end;

		// Create n request block(s)
		while (uGapEnd >= uGapStart && newblockcount < *count)
		{
			// Calculate offsets of the block
			const uint32 startOffset = uGapStart;
			const uint32 endOffset = ((uGapStart + EMBLOCKSIZE - 1) > uGapEnd) ? uGapEnd : (uGapStart + EMBLOCKSIZE - 1); 

			// Prepare offset of next block
			uGapStart = endOffset + 1;

			// Check if block has already been requested
			if (IsAlreadyRequested(startOffset, endOffset) == false) 
			{
				// Create 1 request block
				Requested_Block_Struct* block = new Requested_Block_Struct;
				block->StartOffset = startOffset;
				block->EndOffset = endOffset;   
				// block->packedsize = 0;
				block->transferred = 0;
				#ifndef AMD
				md4cpy(block->FileID, GetFileHash());
				#else
				memcpy_amd(block->FileID, GetFileHash(), 16);
				#endif

				// Flag the block as 'requested'
				requestedblocks_list.AddTail(block);

				// Return the block to the source
				newblocks[newblockcount] = block;
				newblockcount++;
			}
		}
	}
	*count = newblockcount;

	return;

	EMULE_CATCH

	*count = 0;
}

void CPartFile::FillGap(uint32 start, uint32 end)
{
	EMULE_TRY

	POSITION pos1, pos2;
	for (pos1 = gaplist.GetHeadPosition();(pos2 = pos1) != NULL;){
		Gap_Struct* cur_gap = gaplist.GetNext(pos1);
		if (cur_gap->start >= start && cur_gap->end <= end){ // our part fills this gap completly
			gaplist.RemoveAt(pos2);
			delete cur_gap;
			continue;
		}
		else if (cur_gap->start >= start && cur_gap->start <= end){// a part of this gap is in the part - set limit
			cur_gap->start = end+1;
		}
		else if (cur_gap->end <= end && cur_gap->end >= start){// a part of this gap is in the part - set limit
			cur_gap->end = start-1;
		}
		else if (start >= cur_gap->start && end <= cur_gap->end){
			uint32 buffer = cur_gap->end;
			cur_gap->end = start-1;
			cur_gap = new Gap_Struct;
			cur_gap->start = end+1;
			cur_gap->end = buffer;
			gaplist.InsertAfter(pos1,cur_gap);
			break;	// Lord KiRon
		}
	}

	UpdateCompletedInfos();
	//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
	UpdateDisplayedInfo();
	newdate = true;
	// Barry - The met file is now updated in FlushBuffer()
	//SavePartFile();

	EMULE_CATCH
}

void CPartFile::UpdateCompletedInfos() 
{
	EMULE_TRY

   	uint32 allgaps = 0; 

	for (POSITION pos = gaplist.GetHeadPosition();pos !=  0;gaplist.GetNext(pos)){ 
		Gap_Struct* cur_gap = gaplist.GetAt(pos); 
		allgaps += cur_gap->end - cur_gap->start;
	}

	if ((gaplist.GetCount() || requestedblocks_list.GetCount()))
	{ 
			percentcompleted = ((1.0f-(float)allgaps/filesize)) * 100; 
			completedsize = (uint32)((filesize-(allgaps+1))); 
	} else {
		percentcompleted = 100;
		completedsize=filesize;
	}

	EMULE_CATCH
}

void CPartFile::DrawStatusBar(CDC* dc, RECT* rect, bool bFlat)
{
	EMULE_TRY

	const COLORREF crProgress=(bFlat)?RGB(0, 150, 0):RGB(0, 192, 0);
	COLORREF crHave;
	COLORREF crPending;
	COLORREF crMissing;

	// SLUGFILLER: grayPause - Colors by status
	if(GetStatus() == PS_EMPTY || GetStatus() == PS_READY)
	{
		crHave=(bFlat)?RGB(0, 0, 0):RGB(95, 95, 95);
		crPending = RGB(255, 208, 0);
		crMissing = RGB(255, 0, 0);
	} else {
		crHave=(bFlat)?RGB(105, 105, 105):RGB(142, 142, 142);
		crPending = RGB(255, 240, 142);
		crMissing = RGB(255, 142, 142);
	}
	// SLUGFILLER

	CBarShader s_ChunkBar(rect->bottom - rect->top, rect->right - rect->left);
	s_ChunkBar.SetFileSize(filesize); 
	s_ChunkBar.Fill(crHave); 

	uint32 allgaps = 0;

	if(status == PS_COMPLETE || status == PS_COMPLETING) {
		s_ChunkBar.FillRange(0, filesize, crProgress); 
		s_ChunkBar.Draw(dc, rect->left, rect->top, bFlat); 
		percentcompleted = 100;
		completedsize=filesize;
		return; 
	} 

	// red gaps 
	for (POSITION pos = gaplist.GetHeadPosition();pos !=  0;gaplist.GetNext(pos)){ 
		Gap_Struct* cur_gap = gaplist.GetAt(pos); 
		allgaps += cur_gap->end - cur_gap->start;
		bool gapdone = false; 
		uint32 gapstart = cur_gap->start; 
		uint32 gapend = cur_gap->end; 
		for (uint32 i = 0; i != GetPartCount(); i++){ 
			if (gapstart >= i*PARTSIZE && gapstart <=  (i+1)*PARTSIZE){ // is in this part? 
				if (gapend <= (i+1)*PARTSIZE) 
				gapdone = true; 
				else{ 
				gapend = (i+1)*PARTSIZE; // and next  part 
				} 
				// paint 
				COLORREF color; 
				if (m_SrcpartFrequency.GetCount() >= (INT_PTR)i && m_SrcpartFrequency[i])  // frequency? 
				// SLUGFILLER: grayPause
				{
					if(status == PS_EMPTY || status == PS_READY)
					    color = RGB(0,
								    (210-(22*(m_SrcpartFrequency[(uint16)i]-1)) <  0)? 0:210-(22*(m_SrcpartFrequency[(uint16)i]-1))
								    ,255);
				    else
						color = RGB(100,
									(210-(22*(m_SrcpartFrequency[(uint16)i]-1)) <  0)? 0:255-(22*(m_SrcpartFrequency[(uint16)i]-1))
									,255);
				}
				// SLUGFILLER: grayPause
				else 
					color = crMissing; 

				s_ChunkBar.FillRange(gapstart, gapend + 1,  color); 

				if (gapdone) // finished? 
				break; 
				else{ 
				gapstart = gapend; 
				gapend = cur_gap->end; 
				} 
			} 
		} 
	} 

	// yellow pending parts 
	for (POSITION pos = requestedblocks_list.GetHeadPosition();pos !=  0;requestedblocks_list.GetNext(pos))
	{
		Requested_Block_Struct* block =  requestedblocks_list.GetAt(pos); 
		s_ChunkBar.FillRange((block->StartOffset + block->transferred), block->EndOffset,  crPending);
	} 

	s_ChunkBar.Draw(dc, rect->left, rect->top, bFlat); 

	//--- xrmb:confirmeddownload ---
	if((gaplist.GetCount() || requestedblocks_list.GetCount()))
	{
		// all this here should be done in the Process, not in the drawing!!! its a waste of cpu time, or maybe not :)
		uint16	completedParts=0;
		confirmedsize=0;
		for(uint32 i=0; i!=GetPartCount(); i++)
		{
			uint32	end=(i+1)*PARTSIZE-1;
			bool	lastChunk=false;
			//--- last part? ---
			if(end>filesize)
			{
				end=filesize;
				lastChunk=true;
			}

			if(IsComplete(i*PARTSIZE, end))
			{
				completedParts++;

				if(lastChunk==false)
					confirmedsize+=PARTSIZE;
				else
					confirmedsize+=filesize % PARTSIZE;
			}
		}
		
		completedsize = (uint32)((filesize-(allgaps+1)));

		percentcompleted = (float)completedsize/filesize*100;
		percentconfirmed = (float)confirmedsize/filesize*100;
	}
	else 
	{
		percentcompleted = 100;
		percentconfirmed = 100;
		completedsize=filesize;
	}

	// green progress 
	RECT gaprect; 
	gaprect.top = rect->top; 
	gaprect.bottom = gaprect.top + PROGRESS_HEIGHT; // Barry - was 4
	gaprect.left = rect->left; 

	uint32	w=rect->right-rect->left+1;
	uint32	wc=(uint32)(percentconfirmed/100*w+0.5f);
	uint32	wp=(uint32)(percentcompleted/100*w+0.5f);

	if(!bFlat) 
	{
		CBarShader s_LoadBar(PROGRESS_HEIGHT); // No use of static element
		s_LoadBar.SetWidth(wp);
		s_LoadBar.SetFileSize(completedsize);
		s_LoadBar.Fill(crPending);
		s_LoadBar.FillRange(0, confirmedsize, crProgress);
      	s_LoadBar.Draw(dc, gaprect.left, gaprect.top, false); 
	} 
	else 
	{
		int	left=rect->left;

		// green
		//gaprect.left =  left;
		gaprect.right = left+wc;
      		dc->FillRect(&gaprect, &CBrush(crProgress)); 

		// yellow
		gaprect.left = left+wc;
		gaprect.right = left+wp;
		dc->FillRect(&gaprect, &CBrush(crPending));

 		//draw gray progress only if flat 
		gaprect.left = left+wp;
		gaprect.right = left+w;
      		dc->FillRect(&gaprect, &CBrush(RGB(224,224,224)));
	} 
	//--- :xrmb ---

	EMULE_CATCH
}

void CPartFile::WritePartStatus(CFile* file)
{
	EMULE_TRY

	uint16 parts = GetPartCount();
	file->Write(&parts,2);
	uint16 done = 0;
	while (done != parts){
		uint8 towrite = 0;
		for (uint32 i = 0;i != 8;i++){
			//--- xrmb:partprio ---
			if (GetPartStatus(done)==PR_PART_ON && IsComplete(done*PARTSIZE,((done+1)*PARTSIZE)-1))
			//--- :xrmb ---
				towrite |= (1<<i);

			done++;
			if (done == parts)
				break;
		}
		file->Write(&towrite,1);
	}

	EMULE_CATCH
}
// DbT:StartUploadReq
bool CPartFile::HasParts()
{
	EMULE_TRY

	uint16 parts = GetPartCount();
	for (uint16 i=0; i < parts; i++) {
		if(GetPartStatus(i)!=PR_PART_OFF && IsComplete(i*PARTSIZE,((i+1)*PARTSIZE)-1))
			return true;
	}

	EMULE_CATCH

	return false;
}
// DbT:End

// netwolf: complete sources (zegzav) 03.05.03 BEGIN
void CPartFile::WriteCompleteSourcesCount(CFile* file)
	{
	uint16 completecount= GetCompleteSourcesCount();
	file->Write(&completecount,2);
	}
// netwolf: complete sources (zegzav) 03.05.03 END

int CPartFile::GetValidSourcesCount() 
{
	EMULE_TRY

	int counter=0;
	POSITION pos1,pos2;
	for (int sl=0;sl<SOURCESSLOTS;sl++) if (!srclists[sl].IsEmpty())
	for (pos1 = srclists[sl].GetHeadPosition();( pos2 = pos1 ) != NULL;){
		srclists[sl].GetNext(pos1);
		CUpDownClient* cur_src = srclists[sl].GetAt(pos2);
		if (cur_src->GetDownloadState()!=DS_ONQUEUE && cur_src->GetDownloadState()!=DS_DOWNLOADING &&
			cur_src->GetDownloadState()!=DS_NONEEDEDPARTS) counter++;
	}
	return counter;

	EMULE_CATCH

	return 0;
}

uint16 CPartFile::GetNotCurrentSourcesCount()
{
	EMULE_TRY

	uint16 counter=0;

	POSITION pos1,pos2;
		for (int sl=0;sl<SOURCESSLOTS;sl++) if (!srclists[sl].IsEmpty())
		for (pos1 = srclists[sl].GetHeadPosition();( pos2 = pos1 ) != NULL;){
			srclists[sl].GetNext(pos1);
			CUpDownClient* cur_src = srclists[sl].GetAt(pos2);
		if (cur_src->GetDownloadState()!=DS_ONQUEUE && cur_src->GetDownloadState()!=DS_DOWNLOADING) counter++;
	}
	return counter;

	EMULE_CATCH

	return 0;
}
// netwolf: complete sources (zegzav) - BEGIN
uint16	CPartFile::GetCompleteSourcesCount()
{
	if (!m_bUpdateCompleteSrcCount)
		return m_nLastCompleteSrcCount;

	uint16 counter=0;

	POSITION pos1,pos2;
	for (int sl=0;sl<SOURCESSLOTS;sl++) if (!srclists[sl].IsEmpty())
	for (pos1 = srclists[sl].GetHeadPosition();( pos2 = pos1 ) != NULL;){
		srclists[sl].GetNext(pos1);
		CUpDownClient* cur_src = srclists[sl].GetAt(pos2);
		if (cur_src->IsCompleteSource())
			counter++;
	}
	m_nLastCompleteSrcCount= counter;
	m_bUpdateCompleteSrcCount= false;
	return counter;
}
// netwolf: complete sources (zegzav) - END


uint8 CPartFile::GetStatus(bool ignorepause)
{
	EMULE_TRY

	if (!paused || status == PS_ERROR || status == PS_COMPLETING || status == PS_COMPLETE || ignorepause)
		return status;
	else
	{
		if(stopped)
			return PS_STOPPED;
		else
			return PS_PAUSED;
	}

	EMULE_CATCH
}

//Inserted by Vorlost for load rarest parts first, code by Denominator 
uint16 CPartFile::GetRemainingBlocksInPart(uint16 partnumber)
{
	EMULE_TRY

	uint16 result = 0; 
	for (uint32 i = 0; i != ceil((float)PARTSIZE/EMBLOCKSIZE);i++){ 
		uint32 start = (PARTSIZE*partnumber) + i*EMBLOCKSIZE; 
		uint32 end    = (PARTSIZE*partnumber) + ((i+1)*EMBLOCKSIZE)-1; 
		if (end >= PARTSIZE*((uint32)partnumber+1)) 
			end = (PARTSIZE*(partnumber+1))-1; 
		if (start >= GetFileSize()) 
			break; 
		if (end >= GetFileSize()) 
			end = GetFileSize()-1; 
		if ( !IsComplete(start,end) && !IsAlreadyRequested(start,end)) 
			result++; 
	} 
	return result; 

	EMULE_CATCH

	return 0;
} 
//End of code insertion by Vorlost for loading rarest parts first

// Added by Tarod -remove no needed sources from download queue
//
// [In]	number_sources_to_remove: Number of sources to be removed from list or 0xffffffff if we want all removed
void CPartFile::RemoveNoNeededSources(uint32 number_sources_to_remove)
{
	EMULE_TRY

	POSITION pos1, pos2;
	uint32 removed = 0;
    for (int sl=0;sl<SOURCESSLOTS;sl++) if (!srclists[sl].IsEmpty())
	for (pos1 = srclists[sl].GetHeadPosition(); (pos2 = pos1) != NULL && removed < number_sources_to_remove;) {
		srclists[sl].GetNext(pos1);
		CUpDownClient* cur_src = srclists[sl].GetAt(pos2);
		if ((cur_src->GetDownloadState() == DS_NONEEDEDPARTS) || (cur_src->IsRemoteQueueFull()))  //netwolf: katsyonak change
		{
			theApp.downloadqueue->RemoveSource(cur_src);
			removed ++; // Count removed source
		}
	}

	EMULE_CATCH

	lastpurgetime = ::GetTickCount();
}

uint32 CPartFile::Process(uint32 reducedownload/*in percent*/, uint32 iteration)
{
	EMULE_TRY

	uint16	i_TcntTransferring = GetTransferingSrcCount(); // DonGato: for category filters
	
	DWORD dwCurTick = ::GetTickCount();
	CUpDownClient* cur_src;
	bool sockets_usage_high;
	bool bIsClientConnected;
	bool bReaskDownload;
	POSITION pos1, pos2;
	uint32 cur_datarate;
	//eklmn: initialize temp variables
	datarate = 0;

	// If buffer size exceeds limit, or if not written within time limit, flush data
	if ((m_nTotalBufferData > ((uint32)theApp.glob_prefs->GetFileBufferSize())*30000) 
		|| (dwCurTick > (m_nLastBufferFlushTime + BUFFER_TIME_LIMIT))) //netwolf: increased FileBufferSize
	{
		// Avoid flushing while copying preview file
		if (!m_bPreviewing)
			FlushBuffer();
	}

#ifdef OLD_SOCKETS_ENABLED
	 sockets_usage_high = theApp.listensocket->GetOpenSockets() > (theApp.glob_prefs->GetMaxConnections() * 0.75);
#endif //OLD_SOCKETS_ENABLED

	// calculate datarate, set limit etc.
	// 9 iteration for download & 1 iteration for other files
	if (iteration < 10 && !m_downloadingSourceList.IsEmpty())
	{
		for (pos1 = m_downloadingSourceList.GetHeadPosition(); pos1 !=NULL;)
		{
			cur_src = m_downloadingSourceList.GetNext(pos1);
			if (cur_src)
			{
				cur_datarate = cur_src->CalculateDownloadRate();
				datarate+=cur_datarate;
				if(reducedownload)
				{
					uint32 limit = reducedownload*cur_datarate/1000;
					if(limit<1000 && reducedownload == 200)
						limit +=1000;
					else if(limit<1)
						limit = 1;
#ifdef OLD_SOCKETS_ENABLED					
					cur_src->socket->SetDownloadLimit(limit);
#endif //OLD_SOCKETS_ENABLED
				}
				else if ( m_nSavedReduceDownload != reducedownload )
#ifdef OLD_SOCKETS_ENABLED					
					cur_src->socket->DisableDownloadLimit();
#endif //OLD_SOCKETS_ENABLED			
				m_nSavedReduceDownload  = reducedownload;
			}
		}
	}
	else 
	{
		// eklmn (khaos): initialize temp variables only if we need it
		bool		alreadyCounted =		false;
		uint16	i_TcntNNP =				0;
		uint16	i_TcntOnQueue =			0;
		uint16	i_TcntHighQR =			0;
		uint16	i_TcntConnecting =		0;
		uint16	i_TcntTooManyConns =	0;
		uint16	i_TcntConnected =		0;
		uint16	i_TcntTimeout =			0;
		uint16	i_TcntConnViaSrv =		0;
		uint16	i_TcntUnknown =		0;
		uint16	i_TcntRecHash =			0;
		uint16	i_TcntLowToLow =		0;
		uint16	i_TcntQueueFull =		0;
		uint16	i_TcntNoUpload =		0;
		uint16	i_TcntBanned =			0;
		uint16	i_TcntErr =				0;
		// <-----khaos-
	
		for (uint32 sl = 0; sl < SOURCESSLOTS; sl++)
		{
			if (!srclists[sl].IsEmpty())
			{
				for (pos1 = srclists[sl].GetHeadPosition();( pos2 = pos1 ) != NULL;)
				{	
					srclists[sl].GetNext(pos1);
					cur_src = srclists[sl].GetAt(pos2);
					
					// eklmn: check a NULL case
					if (!cur_src) continue;

					alreadyCounted = false;
					bIsClientConnected = true;
					bReaskDownload = false;
					uint32 usethisfilereasktime = FILEREASKTIME;
					
					switch (cur_src->GetDownloadState())
					{
						case DS_DOWNLOADING:
						{
							// DonGato: needed to update the source count 
							// GetTransferingSrcCount() isn't updated inside this function.
							i_TcntTransferring++; 
							cur_datarate = cur_src->CalculateDownloadRate();
							datarate += cur_datarate;
							if (reducedownload && cur_src->GetDownloadState() == DS_DOWNLOADING) 
							{
								uint32 limit = reducedownload*cur_datarate/1000;
								if (limit < 1000 && reducedownload == 200)
									limit += 1000;
								else if (limit < 1)
									limit = 1;
#ifdef OLD_SOCKETS_ENABLED
								cur_src->socket->SetDownloadLimit(limit);
#endif //OLD_SOCKETS_ENABLED
							}
#ifdef OLD_SOCKETS_ENABLED
							else if( m_nSavedReduceDownload != reducedownload )
								cur_src->socket->DisableDownloadLimit();
#endif //OLD_SOCKETS_ENABLED
							m_nSavedReduceDownload  = reducedownload;
							break;
						}
						case DS_BANNED:
							if(!alreadyCounted) {i_TcntBanned++; alreadyCounted = true;}
							break;

						case DS_ERROR:
							if(!alreadyCounted) {i_TcntErr++; alreadyCounted = true;}
							break;

						case DS_CONNECTED:
							// if socket doesn't exist or isn't connected
#ifdef OLD_SOCKETS_ENABLED
							if(cur_src->socket)
							{
								if (!cur_src->socket->IsConnected())
								{
									cur_src->SetDownloadState(DS_NONE);
									if (!alreadyCounted) { i_TcntUnknown++; alreadyCounted = true; }
									break;
								}
							}
							else
							{
								cur_src->SetDownloadState(DS_NONE);
								if (!alreadyCounted) { i_TcntUnknown++; alreadyCounted = true; }
								break;
							}
#endif //OLD_SOCKETS_ENABLED

							if (dwCurTick - cur_src->GetEnteredConnectedState() > CONNECTION_TIMEOUT + 20000)
							{
								if(!alreadyCounted) { i_TcntTimeout++; alreadyCounted = true; }
								theApp.downloadqueue->RemoveSource(cur_src);
								break;
							}
							if(!alreadyCounted) {i_TcntConnected++; alreadyCounted = true;}
							bReaskDownload = true;
							break;

						case DS_LOWTOLOWIP:	// if we now have a high ip we can ask
							if(!alreadyCounted) { i_TcntLowToLow++; alreadyCounted = true; }
							if( ((dwCurTick - lastpurgetime) > PURGE_TIME) 
								&& (GetSourceCount() >= (theApp.glob_prefs->GetMaxSourcePerFile()*.8 )) )
							{
								theApp.downloadqueue->RemoveSource(cur_src);
								lastpurgetime = dwCurTick;
								break;
							}
#ifdef OLD_SOCKETS_ENABLED
							if (theApp.serverconnect->IsLowID())
								break;
#endif //OLD_SOCKETS_ENABLED

						case DS_NONEEDEDPARTS:
						{ 
							if(!alreadyCounted) { i_TcntNNP++; alreadyCounted = true; }
							if ( ((dwCurTick - lastpurgetime) > 40000) 
								&& (GetSourceCount() >= (theApp.glob_prefs->GetMaxSourcePerFile()*.8 )) )
								if( !cur_src->SwapToAnotherFile(NULL,true) ) //eklmn: bugfix(08): A4AF switching
								{
									theApp.downloadqueue->RemoveSource(cur_src);
									lastpurgetime = dwCurTick;
									break; 
								}

							// doubled reasktime for no needed parts - save connections and traffic
							//Vorlost, noneededparts-sources will be checked filereasktime*4 when 3/4 of max. sockets are open 
							if (sockets_usage_high)
								usethisfilereasktime = FILEREASKTIME * 2;
							//End Vorlost for Expanded Filereasktime 

							// doubled reasktime for no needed parts - save connections and traffic
							if (!((!cur_src->GetLastAskedTime()) || (dwCurTick - cur_src->GetLastAskedTime()) > usethisfilereasktime*2))
								break;
						}
						case DS_ONQUEUE:
						{
							if( cur_src->IsRemoteQueueFull() )
							{
								if(!alreadyCounted) { i_TcntQueueFull++; alreadyCounted = true; }
								if( ((dwCurTick - lastpurgetime) > 60000) 
									&& (GetSourceCount() >= (theApp.glob_prefs->GetMaxSourcePerFile()*.8 )) )
								{
									theApp.downloadqueue->RemoveSource( cur_src );
									lastpurgetime = dwCurTick;
									break;
								}
							}
							// Check Remote queue position (HighQR)
							else if (theApp.glob_prefs->UseRemoteQueueLimit()
								&& (cur_src->GetRemoteQueueRank() > theApp.glob_prefs->GetRemoteQueueLimit())) 
							{
								if (!alreadyCounted) { i_TcntHighQR++; alreadyCounted = true; }
								theApp.downloadqueue->RemoveSource(cur_src);
								break;
							}
							else if (!alreadyCounted) { i_TcntOnQueue++; alreadyCounted = true; }
							
							if ( cur_src->GetRemoteQueueRank() && sockets_usage_high )  //Vorlost QueueRank > 250 --> Filereasktime=45minutes
							{
								if ( cur_src->GetRemoteQueueRank() > 250 )
									usethisfilereasktime = FILEREASKTIME * 2.5 ; //Vorlost
								else 
									usethisfilereasktime = FILEREASKTIME * 2; //Vorlost (QR 1-250 = 36minutes)
							}

							// LANCAST (moosetea) - We reask much faster if lancast user, UDP happens in 10 seconds
							if (cur_src->isOnLAN())
							{
								if ((!cur_src->GetLastAskedTime()) 
									|| (::GetTickCount() - cur_src->GetLastAskedTime()) > LANCASTFILEREASKTIMEUDP)
								{
									cur_src->UDPReaskForDownload();
								}
							}
							else 
							{
#ifdef OLD_SOCKETS_ENABLED
								if (theApp.serverconnect->IsConnected() 
									&& ((!cur_src->GetLastAskedTime()) 
									|| (dwCurTick - cur_src->GetLastAskedTime()) > usethisfilereasktime-20000))
								{
									cur_src->UDPReaskForDownload();
								}
#endif //OLD_SOCKETS_ENABLED
							}

							// START enkeyDEV(th1) -L2HAC- highid side, "prepare" phase
							// (emulates a "Connecting via server..." without socket timeout)
#ifdef OLD_SOCKETS_ENABLED
							if (theApp.serverconnect->IsConnected()
								&& (cur_src->GetLastAskedTime())
								&& ((dwCurTick - cur_src->GetLastAskedTime()) > (FILEREASKTIME - L2HAC_PREPARE_PRECEDE))
								&& cur_src->HasLowID()
								&& cur_src->GetL2HACTime()
								&& !theApp.serverconnect->IsLowID())
							{
								cur_src->SetDownloadState(DS_WAITCALLBACK);
							}
#endif //OLD_SOCKETS_ENABLED
							// END enkeyDEV(th1) -L2HAC-
						}
						case DS_CONNECTING:
							if(!alreadyCounted) { i_TcntConnecting++; alreadyCounted = true; }
						case DS_TOOMANYCONNS:
							if(!alreadyCounted) { i_TcntTooManyConns++; alreadyCounted = true; }
						case DS_NONE:
							if(!alreadyCounted) { i_TcntNoUpload++; alreadyCounted = true; }
						case DS_WAITCALLBACK:
							if(!alreadyCounted) { i_TcntConnViaSrv++; alreadyCounted = true; }
							bReaskDownload = true;
							break;
							
						case DS_REQHASHSET:
							i_TcntRecHash++;	
							break;
							
						default:
							i_TcntUnknown++;
					}// switch (state)
#ifdef OLD_SOCKETS_ENABLED
					if (bReaskDownload && theApp.serverconnect->IsConnected())
					{
						// LANCAST (moosetea) - Because reask much faster if lancast user ie we may reask as soon as we get disconnected
						// , we only use TCP on a LAN, because emule may try to send a TCP request and UDP request at the same time!
						if (cur_src->isOnLAN())
							usethisfilereasktime = LANCASTFILEREASKTIMETCP;
						//Vorlost for Expanded FILEREASKTIME if QR is known and 3/4 Sockets are open 
						if (!cur_src->GetLastAskedTime() || (dwCurTick - cur_src->GetLastAskedTime()) > usethisfilereasktime)
							bIsClientConnected = cur_src->AskForDownload();
					}
#endif //OLD_SOCKETS_ENABLED
					cur_src = NULL;	// reset a variable
				}// end: pos = srclist...
			}//end: srclist.Empty
		}// for (slots)

		// -khaos--+++> Set the members using temp vars.  We have
		//				to use temporary vars so that the disp counts
		//				don't get screwed up during processing.
		// we don't need to count this, cause we can get a transfering sources over m_downloadingSourceList.GetCount()
		//m_iSrcTransferring=	i_TcntTransferring;
		m_iSrcNNP =				i_TcntNNP;
		m_iSrcOnQueue =			i_TcntOnQueue;
		m_iSrcHighQR =			i_TcntHighQR;
		m_iSrcConnecting =		i_TcntConnecting;
		m_iSrcTooManyConns =	i_TcntTooManyConns;
		m_iSrcConnected =		i_TcntConnected;
		m_iSrcTimeout = 		i_TcntTimeout;
		m_iSrcConnViaServer =	i_TcntConnViaSrv;
		m_iSrcUnknown =			i_TcntUnknown;
		m_iSrcRecHash =			i_TcntRecHash;
		m_iSrcLowToLow =		i_TcntLowToLow;
		m_iSrcQueueFull =		i_TcntQueueFull;
		m_iSrcNoUpload =		i_TcntNoUpload;
		m_iSrcBanned =			i_TcntBanned;
		m_iSrcError =			i_TcntErr;
		m_iSrcA4AF =			A4AF_srclist.GetCount();
		// <-----khaos-

		// swap No needed partfiles if possible
		// v:- eklmn: A4AF management
		// Modified by Tarod -now handles A4AF too keeping usage of CPU less and do not forcing files
		// that has No Needed Parts to download letting them swap to another file
		if (((!m_LastNoNeededCheck) || (dwCurTick - m_LastNoNeededCheck) > 10000))
		{
			m_LastNoNeededCheck = dwCurTick;
			for (int sl=0;sl<SOURCESSLOTS;sl++) 
				if (!srclists[sl].IsEmpty())
					for (pos1 = srclists[sl].GetHeadPosition(); (pos2 = pos1 ) != NULL;) 
					{
						srclists[sl].GetNext(pos1);
						cur_src = srclists[sl].GetAt(pos2);
						// check if current source has a part for this file
						if (cur_src->GetDownloadState() == DS_NONEEDEDPARTS) 
						{
							// check if A4AF Auto file is enable
							if (theApp.downloadqueue->GetA4AFAutoFile() == NULL)
								// A4AF auto is disable => switch to any file
								cur_src->SwapToAnotherFile(NULL,true);	//eklmn: bugfix(08): A4AF switching
							else if (theApp.downloadqueue->GetA4AFAutoFile() != this)
								// A4AF auto is enable & isn't current file => switch to A4AF auto file
								cur_src->SwapToAnotherFile(theApp.downloadqueue->GetA4AFAutoFile());
						}
					} // end for (pos =
		} // end if
	
		// if this "File" is A4AF auto & A4AF list isn't empty,
		// then check A4AF list and swap source to "this", if souce isn't NNS
		if (!A4AF_srclist.IsEmpty() && theApp.downloadqueue->GetA4AFAutoFile() == this) 
		{
			//eklmn: use a two-pointer loop to prevent exception due to swaping (delete)
			for (pos1 = A4AF_srclist.GetHeadPosition(); (pos2 = pos1 ) != NULL;)
			{
				A4AF_srclist.GetNext(pos1);
				cur_src = A4AF_srclist.GetAt(pos2);
				if (cur_src->m_OtherNoNeeded_list.Find(this) == 0)
					cur_src->SwapToAnotherFile(this);
			}
		}
		// ^- eklmn: A4AF management

		// check if we want new sources from server
		//uint16 test = theApp.glob_prefs->GetMaxSourcePerFileSoft();
#ifdef OLD_SOCKETS_ENABLED
		if (( (!lastsearchtime) 
			|| (dwCurTick - lastsearchtime) > SERVERREASKTIME) 
			&& theApp.serverconnect->IsConnected()
			&& theApp.glob_prefs->GetMaxSourcePerFileSoft() > GetSourceCount() )
		{
			//local server
			lastsearchtime = dwCurTick;
			Packet* packet = new Packet(OP_GETSOURCES,16);
			#ifndef AMD
			md4cpy(packet->pBuffer,filehash);
			#else
			memcpy_amd(packet->pBuffer,filehash,16);
			#endif
			theApp.uploadqueue->AddUpDataOverheadServer(packet->size);
			theApp.serverconnect->SendPacket(packet,true);
			//AddDebugLogLine(false, "Send:Source Request Server File(%s)", GetFileName());
		}
#endif //OLD_SOCKETS_ENABLED

		// eklmn: call SaveSources only one time (in case if there isn't DL)
		// START enkeyDEV(kei-kun) -SLS preferences-
#ifdef OLD_SOCKETS_ENABLED
		if (iteration && theApp.glob_prefs->SLSEnable() && m_downloadingSourceList.IsEmpty() && theApp.serverconnect->IsConnected()) 
		{
			m_sourcesaver.Process(this, theApp.glob_prefs->SLSMaxSourcesPerFile()); //<<-- enkeyDEV(Ottavio84) -New SLS-
		}
#endif //OLD_SOCKETS_ENABLED
		// END enkeyDEV(kei-kun) -SLS preferences-

		// calculate datarate, set limit etc. (every 5sec)
		count++;
		if (count == 5)
		{
			count = 0;
			//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
			UpdateDisplayedInfo();
			if (m_bPercentUpdated == false)
				UpdateCompletedInfos();
			m_bPercentUpdated = false;
		}
	}

	// DonGato: for category filters
	if (GetTransferingSrcCount() != i_TcntTransferring)
	{
		if (theApp.emuledlg->transferwnd.m_wndDownloadList.m_nCurTab==0)
			theApp.emuledlg->transferwnd.m_wndDownloadList.ChangeCategory(0); 
		else
			UpdateDisplayedInfo(true);
		
		if (theApp.glob_prefs->ShowCatTabInfos())
			theApp.emuledlg->transferwnd.UpdateCatTabTitles();
	}
	
	return datarate;

	EMULE_CATCH

	return 0;
}

void CPartFile::AddSources(CMemFile& sources,uint32 serverip, uint16 serverport)
{
	EMULE_TRY
	
	//if this part file is paused or complete/completing no sources needed any more (Cax2 + SyruS check)
	if (GetStatus() == PS_PAUSED 
		|| GetStatus() == PS_STOPPED
		|| GetStatus() == PS_COMPLETE
		|| GetStatus() == PS_COMPLETING) 
		return;
	
	uint8 count;
	uint8 debug_lowiddropped = 0;
	uint8 debug_possiblesources = 0;
	sources.Read(&count,1);
//	if (theApp.glob_prefs->IsRemoveNNSEnabled())
    {
	    // Added by Tarod - if there is needed place for more sources just remove NNS
	    int source_count_plus_new = GetSourceCount() + count;
	    if (source_count_plus_new > theApp.glob_prefs->GetMaxSourcePerFile()
		    && ((lastpurgetime == 0) || (::GetTickCount() - lastpurgetime) > PURGE_TIME))
		    RemoveNoNeededSources(source_count_plus_new - theApp.glob_prefs->GetMaxSourcePerFile());
	}
	for (int i = 0;i != count;i++){
		uint32 userid;
		sources.Read(&userid,4);
		uint16 port;
		sources.Read(&port,2);
		// check first if we are this source

		// MOD Note: Do not change this part - Merkur
#ifdef OLD_SOCKETS_ENABLED
		if (theApp.serverconnect->GetClientID() < 16777216 && theApp.serverconnect->IsConnected()){
			if ((theApp.serverconnect->GetClientID() == userid) && inet_addr(theApp.serverconnect->GetCurrentServer()->GetFullIP()) == serverip)
				continue;
		}
		else if (theApp.serverconnect->GetClientID() == userid)
			continue;
		else if (userid < 16777216 && !theApp.serverconnect->IsLocalServer(serverip,serverport)){
			debug_lowiddropped++;
			continue;
		}
#endif //OLD_SOCKETS_ENABLED
		// MOD Note - end

		if( theApp.glob_prefs->GetMaxSourcePerFile() > GetSourceCount() ){
			debug_possiblesources++;
			CUpDownClient* newsource = new CUpDownClient(port,userid,serverip,serverport,this);
			theApp.downloadqueue->CheckAndAddSource(this,newsource);
		} 
	}
//	AddDebugLogLine(false,"RCV: %i sources from server, %i low id dropped, %i possible sources",count,debug_lowiddropped,debug_possiblesources);

	EMULE_CATCH
}

// v- eklmn: changes(00): NewSrcPartsInfo & UpdateAvailablePartsCount merged (CPU load)
void CPartFile::NewSrcPartsInfo()
{
	EMULE_TRY
	// Cache part count
	uint16 iPartCount = GetPartCount();
	uint8 availablecounter = 0;
	uint8 MaxAvailableParts = 0;

	// Increase size if necessary
	if(m_SrcpartFrequency.GetSize() < iPartCount){
		m_SrcpartFrequency.SetSize(iPartCount);
	}
	// Reset part counters & median
	for(int i = 0; i < iPartCount; i++){
		m_SrcpartFrequency[i] = 0;
	}
	median = 0;     // <- eklmn: feature(00): ICR 
	CUpDownClient* cur_src;

	// Calculate a median position
	int M_pos = (int)ceil((float)iPartCount/2) - 1;	// <- eklmn: feature(00): ICR

	POSITION pos1,pos2;
	for(int sl=0; sl<SOURCESSLOTS; ++sl) {
		if (!srclists[sl].IsEmpty()) {
			for (pos1 = srclists[sl].GetHeadPosition();( pos2 = pos1 ) != NULL;){
				srclists[sl].GetNext(pos1);
				cur_src = srclists[sl].GetAt(pos2);
				int j = 0; // <- eklmn: feature(00): ICR v2
				availablecounter = 0;
				for (int i = 0; i < iPartCount; i++){
					if (cur_src->IsPartAvailable(i)) {
						if (j == M_pos) median++;       // <- eklmn: feature(00): ICR
						j++;                            // <- eklmn: feature(00): ICR
						m_SrcpartFrequency[i] +=1;
						availablecounter++;
					} // end: IsPartAv...
				}
				if (availablecounter>MaxAvailableParts) MaxAvailableParts = availablecounter;
			}
		}
	}
	//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);

	m_MaxSrcPartFrequency = 0;	// <- eklmn: feature(00): ICR
	for(int i = 0; i < iPartCount; i++)
		if (m_SrcpartFrequency[i] > m_MaxSrcPartFrequency) m_MaxSrcPartFrequency = m_SrcpartFrequency[i];

	if (iPartCount == MaxAvailableParts && availablePartsCount < iPartCount)
		lastseencomplete = CTime::GetCurrentTime();
	availablePartsCount = MaxAvailableParts;
	m_bUpdateCompleteSrcCount= true;	// netwolf: zegzav:completesrc
	
	UpdateDisplayedInfo();


	EMULE_CATCH
}
// ^- eklmn: changes(00): NewSrcPartsInfo & UpdateAvailablePartsCount merged (CPU load)

// v- eklmn: feature(00): Intelligent Chunk Request(ICR v2.10, 3.07.2003) for eMule+
uint16 CPartFile::GetNextRequestedBlock(CUpDownClient* sender)	//Cax2 - changed the type of function, left algorithm the same
{
	EMULE_TRY

	if (sender == NULL)
		return 0xffff;

	// Cache some values => speed improvement
	const int partCount = GetPartCount();
	int16 i_AvailableNeededParts = 0;
	int16 i_CurBlockedParts =0;
	const uint8 PreviewMode = GetMovieMode();
  	// Mode 0=NoPreview,1=AVI(1F,1L),2=AVI(2F,1L),3=AVI(3F,1L)
  	// Mode 4=MPG(1F),5=MPG(2F),6=MPG(3F),7=AVI/MPG(2F,2L) F=First,L=Last
  	uint8* partsav = sender->GetPartStatus();
	uint16 goodpart = sender->GetLastDownPart();

	_Bvector gettingParts;
	sender->ShowDownloadingParts(gettingParts);
	if((goodpart != 0xffff) && GetNextEmptyBlockInPart(goodpart,0)) {
		//AddDebugLogLine(false,"Priority 0 (always choose currently downloaded part until it will be finished). File: %s, Part : %u", this->GetFileName(), goodpart);
	 	return sender->GetLastDownPart();
	}
	 else {
	 	sender->SetLastDownPart(0xffff);
	}

	// Cunstruct Lists of block sorted by priority. Use number of parts to create limited lists and
	// keep data in list in same memory area (speed improvements)
	//CList<int> liRarestReqParts(partCount);	// priority 1 (highest)
	CList<int> liUniqueParts(partCount);    // priority 2
	CList<int> liRarestParts(partCount);	// priority 4
	CList<int> liMovieParts(5);            	// priority 5 => only first+last parts
	//CList<int> liReqParts(partCount);	// priority 6
	CList<int> liRandomParts(partCount);	// priority 7
	//----------------------------------------------------
	CList<uint16,uint16> m_CurBlockedParts;
	uint32 LastPartSize = 0;

	// scaning for requsted chunks in order to find how much they filled.
	for (int i = 0; i < partCount ; i++) {
		if (sender->IsPartAvailable(i) == true)
		{	
			m_ReqParts[i] = GetBytesInPart(i,true);
			if (m_ReqParts[i] != 0) i_AvailableNeededParts++;
		}
		else
			m_ReqParts[i] = 0xFFFFFFFF; // mark parts as not available to save CPU load
		// scan for blocked part
		if (m_BlockedParts[i] != NULL && m_BlockedParts[i]->GetDownloadState() == DS_DOWNLOADING)
			i_CurBlockedParts++;
		else
			m_BlockedParts[i] = NULL;
	} // end: scanning (for ...)

	// define condition of rareness
	uint16 uRarestPart = (uint16)floor(0.1*median); // 10% from median
	if (uRarestPart < 3) uRarestPart = 3;

	// Build lists
	uint16 MostCompletedUniqueReqPart, MostCompletedRarestReqPart, MostCompletedReqPart;
       uint32 m_MostCompletedUniqueReqPart = PARTSIZE;
	uint32 m_MostCompletedRarestReqPart = PARTSIZE;
	uint32 m_MostCompletedReqPart = PARTSIZE;
	for (int i = 0; i < partCount ; i++) {
		if (m_ReqParts[i] != 0xFFFFFFFF && m_ReqParts[i] != 0) 
		{
		        // Unique part (only one part exist, try to download it first in order to prevent
                        // file disappearing)
			if (m_SrcpartFrequency[i] == 1) {
				if (m_ReqParts[i] > (PARTSIZE - EMBLOCKSIZE)) // limit change to prevent consecutive download due overlaping
					liUniqueParts.AddHead(i);
				else {
					if (m_ReqParts[i] < m_MostCompletedUniqueReqPart) {
						MostCompletedUniqueReqPart = i;
						m_MostCompletedUniqueReqPart = m_ReqParts[i];
					}
				}
			}
			// ------------------------ Rarest Parts -------------------------
			if (median >= (uRarestPart + m_SrcpartFrequency[i]) ) {
				if (m_ReqParts[i] == PARTSIZE) {
					for (int j=0; j < (median-m_SrcpartFrequency[i]+1); j++)  liRarestParts.AddHead(i);
				}
				else {
					if (m_ReqParts[i] < m_MostCompletedRarestReqPart) {
						MostCompletedRarestReqPart = i;
						m_MostCompletedRarestReqPart = m_ReqParts[i];
					}
				}
			} // end rarest
			// --------------------------- Movie or Archive ---------------------------
			switch (PreviewMode) {
				case 3:  // 3=AVI(3F,1L)
					if (partCount > 3 && (i == partCount - 1)) liMovieParts.AddHead(i);
				case 6: // 6=MPG(3F)
					if (i == 2 && partCount > 2) liMovieParts.AddHead(2);
					if (i == 1 && partCount > 1) liMovieParts.AddHead(1);
					if (i == 0) liMovieParts.AddHead(0);
					break;
				case 7: // 7=AVI/MPG(2F,2L)
					if (partCount > 3 && (i == partCount - 2)) liMovieParts.AddHead(i);
				case 2: // 2=AVI(2F,1L)
					if (partCount > 2 && (i == partCount - 1)) liMovieParts.AddHead(i);
				case 5: // 5=MPG(2F)
					if (i == 1 && partCount > 1) liMovieParts.AddHead(1);
					if (i == 0) liMovieParts.AddHead(0);
					break;
				case 1: // 1=AVI(1F,1L)
					if (partCount > 1 && (i == partCount - 1)) liMovieParts.AddHead(i);
				case 4: // 4=MPG(1F)
					if (i == 0) liMovieParts.AddHead(0);
			}
			// --------------------- widespreaded parts ---------------------
			// check blocked parts
			if (m_BlockedParts[i] != NULL && i_AvailableNeededParts>i_CurBlockedParts) continue;
			if (m_ReqParts[i] == PARTSIZE) 
			{
				// Unstarted
				for (int j=0; j < (m_MaxSrcPartFrequency-m_SrcpartFrequency[i]+1); j++)
					liRandomParts.AddHead(i);
			}
			else 
			{
				if (m_ReqParts[i] < m_MostCompletedReqPart) 
				{
					MostCompletedReqPart	= i;
					m_MostCompletedReqPart = m_ReqParts[i];
				}
			} // end widespreaded
		} // end: if....
	} // end for

	// Select final result
	// ------------------------------ Unique & Requesred --------------------------------------------------
	// Try to complete a UniquePart and MostDownloaded as soon as possible to become a new source
	// it doesn't matter which one download, if there are parts same filled.
        if (m_MostCompletedUniqueReqPart != PARTSIZE && liMovieParts.IsEmpty()) 
        {
            goodpart = MostCompletedUniqueReqPart;
            //AddDebugLogLine(false,"Priority 1 (Unique unifinished part). File: %s, Part : %u", this->GetFileName(), goodpart);
        }
        // ------------------------------ Unique & Unrequesred ------------------------------------------------
        else if (!liUniqueParts.IsEmpty() && liMovieParts.IsEmpty()) 
        {
	  	// If there is only one unique part, then take it, otherwise take it random.
			if (liUniqueParts.GetCount() == 1)
            	goodpart = liUniqueParts.GetHead();
		else 
		{
				const uint16 randomness = (uint16)ROUND(((float)rand()/RAND_MAX)*(liUniqueParts.GetCount()-1));
				goodpart = liUniqueParts.GetAt(liUniqueParts.FindIndex(randomness));
        	}
	        //AddDebugLogLine(false,"Priority 2 (Unique unrequested part). File: %s, Part : %u", this->GetFileName(), goodpart);
		}
		// ------------------------------ Rarest & Requesred --------------------------------------------------
		// Try to complete a RarestPart and MostDownloaded as soon as possible to become a new source
		// it doesn't matter which one download, if there are parts same filled.
       else if (m_MostCompletedRarestReqPart != PARTSIZE) 
       {
            goodpart = MostCompletedRarestReqPart;
            //AddDebugLogLine(false,"Priority 3 (Rarest unifinished part). File: %s, Part : %u", this->GetFileName(), goodpart);
		}
        // ------------------------------ Rarest & Unrequesred ------------------------------------------------
        else if (liRarestParts.IsEmpty() == false) 
        {
	  	// If there is only one rarest part, then take it, otherwise take it random.
		if (liRarestParts.GetCount() == 1)
			goodpart = liRarestParts.GetHead();
		else 
		{
			const uint16 randomness = (uint16)ROUND(((float)rand()/RAND_MAX)*(liRarestParts.GetCount()-1));
			goodpart = liRarestParts.GetAt(liRarestParts.FindIndex(randomness));
		}
                //AddDebugLogLine(false,"Priority 4 (Rarest unrequested part). File: %s, Part : %u", this->GetFileName(), goodpart);
        }
        // ---------------------------------- Movies Part  -----------------------------------------------------
        else if (liMovieParts.IsEmpty() == false) 
        {
            goodpart = liMovieParts.GetHead();
            //AddDebugLogLine(false,"Priority 5 (Preview part). File: %s, Part : %u", this->GetFileName(), goodpart);
        }
        // --------------------------------------- Requesred ----------------------------------------------------
        // Try to complete a UnfinishedPart and MostDownloaded as soon as possible to become a new source
   	// it doesn't matter which one download, if there are parts same filled.
        else if (m_MostCompletedReqPart != PARTSIZE) 
        {
			goodpart = MostCompletedReqPart;
			m_BlockedParts[goodpart] = sender;
            //AddDebugLogLine(false,"Priority 6 (Widespread unifinished part). File: %s, Part : %u", this->GetFileName(), goodpart);
        }
        // -------------------------------- Unrequesred & Random -------------------------------------------------
        else if (liRandomParts.IsEmpty() == false) 
        {
	   		const uint16 randomness = (uint16)ROUND(((float)rand()/RAND_MAX)*(liRandomParts.GetCount()-1));
			goodpart = liRandomParts.GetAt(liRandomParts.FindIndex(randomness));
			m_BlockedParts[goodpart] = sender;
			//AddDebugLogLine(false,"Priority 7 (Widespread unrequested part). File: %s, Part : %u", this->GetFileName(), goodpart);
        }
	else
		goodpart = 0xFFFF;

	//AddDebugLogLine(false,"ICR-Test: File: %s, A-parts : %u, B-Parts %u. Choosen Part: %u", this->GetFileName(), i_AvailableNeededParts, m_CurBlockedParts.GetCount(),goodpart);

	// clear lists
	if (!liUniqueParts.IsEmpty()) liRarestParts.RemoveAll();
	if (!liRarestParts.IsEmpty()) liRarestParts.RemoveAll();
	if (!liMovieParts.IsEmpty()) liMovieParts.RemoveAll();
	if (!liRandomParts.IsEmpty()) liRandomParts.RemoveAll();
	if (!m_CurBlockedParts.IsEmpty()) m_CurBlockedParts.RemoveAll();

	sender->SetLastDownPart(goodpart);
	return goodpart;

	EMULE_CATCH

	return 0xffff;
}

uint32 CPartFile::GetBytesInPart(uint16 partnumber, bool left2DL, bool dontRecalculate)
{
	EMULE_TRY
		
	if (partnumber >= GetPartCount()) 
		return (left2DL ? 0 : 0xFFFFFFFF);
		
	uint32 PartSize;
	uint32 left2DLSize;
	
	const uint32 uStart = partnumber* PARTSIZE;
	const uint32 uEnd   = ((filesize - 1) < (uStart + PARTSIZE - 1)) ? (filesize - 1) : (uStart + PARTSIZE - 1);

	if (dontRecalculate)
	{
		PartSize =  m_ReqParts[partnumber];
		left2DLSize = (uEnd -uStart +1) - PartSize;
	}
	else
	{
		// Set part size to fullfilled
		PartSize = left2DLSize= uEnd -uStart + 1;
		for (POSITION pos = gaplist.GetHeadPosition(); pos != 0; gaplist.GetNext(pos) ) 
		{
			Gap_Struct* cur_gap = gaplist.GetAt(pos);
			// Check if Gap is into the limit
			if (cur_gap->end < uStart || cur_gap->start > uEnd) 
				continue;
			if (cur_gap->start <= uStart && cur_gap->end > uStart && cur_gap->end < uEnd) 
			{
				PartSize -= cur_gap->end - uStart;
				continue;
			}
			if (cur_gap->start >= uStart && cur_gap->end <= uEnd) 
			{
				PartSize -= cur_gap->end - cur_gap->start;
				continue;
			}
			if (cur_gap->start <= uStart && cur_gap->end >= uEnd) 
			{
				PartSize = 0;
				continue;
			}
			if (cur_gap->start > uStart && cur_gap->start < uEnd && cur_gap->end >= uEnd) 
			{
				PartSize -= uEnd - cur_gap->start;
			}
	}
	}

	left2DLSize -= PartSize; 

	if (left2DL)
		return left2DLSize;
	else
		return PartSize;
	
	EMULE_CATCH
		
	return (left2DL ? 0 : 0xFFFFFFFF);
	
}
// ^- eklmn: feature(00): Intelligent Chunk Request(ICR v2.10, 3.07.2003) for eMule+


void  CPartFile::RemoveBlockFromList(uint32 start,uint32 end)
{
	EMULE_TRY

	POSITION pos1,pos2;
	for (pos1 = requestedblocks_list.GetHeadPosition(); ( pos2 = pos1 ) != NULL; ){
	   requestedblocks_list.GetNext(pos1);
		if (requestedblocks_list.GetAt(pos2)->StartOffset <= start && requestedblocks_list.GetAt(pos2)->EndOffset >= end)
			requestedblocks_list.RemoveAt(pos2);
	}

	EMULE_CATCH
}

void CPartFile::RemoveAllRequestedBlocks(void)
{
	EMULE_TRY

	requestedblocks_list.RemoveAll();

	EMULE_CATCH
}

void CPartFile::CompleteFile(bool bIsHashingDone)
{
	EMULE_TRY

	StopFile();		//Cax2 - completing file bugfix....

	//if( srcarevisible )  //Done in StopFile()
	//Show completing instead of paused
	SetStatus(PS_COMPLETING);
	UpdateDisplayedInfo(true);
	
	if (!bIsHashingDone){
		datarate = 0;
		CString partfileb = partmetfilename.Left(partmetfilename.GetLength()-4);
		CAddFileThread* addfilethread = (CAddFileThread*) AfxBeginThread(RUNTIME_CLASS(CAddFileThread), THREAD_PRIORITY_BELOW_NORMAL+ theApp.glob_prefs->GetMainProcessPriority(),0, CREATE_SUSPENDED);
		addfilethread->SetValues(0,tempdir,partfileb,this);
		addfilethread->ResumeThread();	
		return;
	}
	else {
		CWinThread *pThread = AfxBeginThread((AFX_THREADPROC )CompleteThreadProc,this, THREAD_PRIORITY_BELOW_NORMAL+ theApp.glob_prefs->GetMainProcessPriority(),0, CREATE_SUSPENDED);
		if (pThread)
		{
			pThread->ResumeThread();
		} else {
			throw CString(_T("Error creating file complete thread")); // move this to resources
		}
	}
	GetAvgDatarate(true);	//SyruS do update on completing files

	if (theApp.glob_prefs->ShowCatTabInfos())
		theApp.emuledlg->transferwnd.UpdateCatTabTitles();

	UpdateDisplayedInfo(true);
	EMULE_CATCH
}

static void GenerateNewFileName(const CString& outputDir, CString& newname)
{
	ASSERT(!newname.IsEmpty()); //name should never be 0

	int namecount = 0;

	TCHAR p_fname[_MAX_FNAME];
	TCHAR p_fext[_MAX_EXT];
	_tsplitpath(newname, NULL, NULL, p_fname, p_fext);

	CString fname(p_fname);
	CString fext(p_fext);
	//search for matching ()s and check if it contains a number inside
	if (fname.Right(1) == ')') {
		int ll = fname.GetLength();
		int ob = fname.ReverseFind('(');		
		if (ob != -1 && ob + 2 != ll) { //check for ) existence or () situation
			bool found = true;
			for (int i = ob + 1; i < ll - 1; i++) {	//check for digits inside brackets			
				if (!_istdigit(fname[i])) {
					found = false;
					break;
				}
			}
			if (found) {				
				namecount = _ttoi(fname.Mid(ob + 1, ll - ob - 2));
				fname.Truncate(ob);
			}
		}
	}
		
	do
	{ 
       	namecount++; 
		newname.Format(_T("%s\\%s(%d)%s"), outputDir, fname, namecount, fext); 
    }
    while(PathFileExists(newname));    
}

// Lord KiRon - using threads for file completion
BOOL CPartFile::PerformFileComplete(void) 
{
	EMULE_TRY

	// Get current category
	int cat = GetCategory();

	//eklmn: wait until file will be free
	while (theApp.uploadqueue->IsFileUploading(this) == true)
	{
		// do nothing, just wait
		Sleep(0); // release the CPU for other processes 
	}

	// -khaos--+++> Save the lost/gain stats!
	theApp.glob_prefs->Add2LostFromCorruption(m_iLostDueToCorruption);
	theApp.glob_prefs->Add2SavedFromCompression(m_iGainDueToCompression);
	// <-----khaos-

	CSingleLock(&m_FileCompleteMutex,TRUE); // will be unlocked on exit
	CString partfilename = fullname.Left(fullname.GetLength() - 4);
	
	CString newname;
	newname.Format(_T("%s\\%s"),GetOutputDir(), theApp.StripInvalidFilenameChars(GetFileName()));

	// close permanent handle
	try{
		if (m_hpartfile.m_hFile != INVALID_HANDLE_VALUE)
			m_hpartfile.Close();
	}
	catch(CFileException* error){
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(false,_T("Failed to close permanent handle %s - %s"),GetFileName(),GetErrorMessage(error));
		error->Delete();
	}
	
	bool renamed = false;
	if(PathFileExists(newname))
	{
		renamed = true;
		GenerateNewFileName(GetOutputDir(), newname);
	} 

	// Lord KiRon
	BOOL bSlowComplete   =   theApp.glob_prefs->SlowComplete();
	int  BlockSize   =   theApp.glob_prefs->SlowCompleteBlockSize()*1024; 
       
    // we do this only if SlowComplete enabled 
    // AND the files are on the different HDs (paratitions) - check by the drive leter 
    // AND any of path is not in URL/Network format (has : as second leter) 
    if (bSlowComplete && (_toupper(partfilename[0]) != _toupper(newname[0]) && partfilename[1] == _T(':') 
         && newname[1] == _T(':'))) 
    {
    	CFile fInputFile; 
        CFile fOutputFile; 
        try 
        { 
      		CString strTempNew   =   CString(newname); 
            strTempNew += _T(".$$$"); 
            // We going to write everithing into the temporary file at same place the newname located 
            // and after we finished - rename , this will take care of syncronization problem with SharedFiles 
            if (!fInputFile.Open(partfilename,CFile::modeRead | CFile::shareDenyNone | CFile::typeBinary 
                | CFile::osSequentialScan)) 
            { 
               	// let catch to care about closing everything 
               	throw CString(GetResString(IDS_ERR_SLOWREAD)+_T(" (")+partfilename+_T(")"));
               	return FALSE; 
            } 
            if (!fOutputFile.Open(strTempNew,CFile::modeWrite | CFile::shareExclusive | CFile::osSequentialScan | CFile::modeCreate)) 
            { 
            	fInputFile.Close(); 
            	// let catch to care about closing everything 
				throw CString(GetResString(IDS_ERR_SLOWWRITE)+_T(" (")+strTempNew+_T(")"));
               	return FALSE; 
            } 

            DWORD dwRead = 0; 
            char *buffer = new char[BlockSize]; 
            if (!buffer) 
            { 
               	// let catch to care about closing everything 
               	throw CString(GetResString(IDS_ERR_OUTMEM)); 
               	return FALSE; 
            } 

            // Most of the work done here 
            // just copy the file data from one to another 
            do 
            { 
            	dwRead = fInputFile.Read(buffer, BlockSize ); 
               	fOutputFile.Write(buffer, dwRead); 
               	Sleep(0); // release the CPU for other processes 
               	// may be real sleep needed here , not sure 
            } 
            while (dwRead > 0); 
            delete []buffer; 

            fInputFile.Close();
            fOutputFile.Close();
            // Rename temporary file back to normal name 
            if (_trename(strTempNew,newname))
            { 
            	// let catch to care about closing everything 
               	throw CString(GetResString(IDS_ERR_RENFAIL)); 
               	return FALSE; 
            }
            CFile::Remove(partfilename); // This removes old Part file
		} 
        catch(CString str) 
        {          	
            AddLogLine(true,str); 
            paused = true; 
            SetStatus(PS_ERROR); 
            return FALSE; 
        } 
        catch(...) 
        {             
			// Purity - Sending message for erroneous file
			CString MessageText;
			MessageText.Format(GetResString(IDS_ERR_COMPLETIONFAILED) + _T(" ") + GetFileName());
			theApp.smtpconnection->SendMail(MessageText, theApp.glob_prefs->GetNotifierPopOnImportantError(), theApp.glob_prefs->IsSMTPWarningEnabled());
			theApp.emuledlg->ShowNotifier(MessageText, TBN_IMPORTANTEVENT, false, theApp.glob_prefs->GetNotifierPopOnImportantError());
            AddLogLine(true,IDS_ERR_COMPLETIONFAILED,GetFileName()); 
            paused = true; 
            SetStatus(PS_ERROR); 
            return FALSE; 
        } 
	} else {
		if (_trename(partfilename,newname))
		{         	
			// Purity - Sending message for erroneous file
			CString MessageText;
			MessageText.Format(GetResString(IDS_ERR_COMPLETIONFAILED) + _T(" ") + GetFileName());
			theApp.smtpconnection->SendMail(MessageText, theApp.glob_prefs->GetNotifierPopOnImportantError(), theApp.glob_prefs->IsSMTPWarningEnabled());
			theApp.emuledlg->ShowNotifier(MessageText, TBN_IMPORTANTEVENT, false, theApp.glob_prefs->GetNotifierPopOnImportantError());
            AddLogLine(true,IDS_ERR_COMPLETIONFAILED,GetFileName()); 
            paused = true; 
            SetStatus(PS_ERROR); 
            return FALSE; 
        }
	} 
    // Lord KiRon - end
    
	// bluecow: to have the accurate date stored in known.met we have to update the 'date' of a just completed file.
	// if we don't update the file date here (after commiting the file and before adding the record to known.met), 
	// that file will be rehashed at next startup and there would also be a duplicate entry (hash+size) in known.met because of different file date!
	ASSERT( m_hpartfile.m_hFile == INVALID_HANDLE_VALUE ); // the file must be closed/commited!
	struct _stat st;
	if (_tstat(newname, &st) == 0)
		date = st.st_mtime;
	// bluecow [SyruS]

    if (_tremove(fullname)) 
    	AddLogLine(true,IDS_ERR_DELETEFAILED,fullname); 

	// START - enkeyDEV(Ottavio84) -ChangeDir-
	CString dirFile = CString(fullname) + CString(".dir") ;
	if (_tremove(dirFile)) if (errno != ENOENT)
		AddLogLine(true, IDS_ERROR_DELETE, dirFile);

	// netwolf - remove .txtscr file
	m_sourcesaver.DeleteFile(this); //<<-- enkeyDEV(Ottavio84) -New SLS-
	
	// Tarod - erase .backup file
 	CString srcName=_T("");
	srcName.Format(_T("%s\\%s.bak"), theApp.glob_prefs->GetTempDir(), GetPartMetFileName());
	if (_tremove(srcName))
		if (errno != ENOENT)
			AddLogLine(true, IDS_FAILDELETE,srcName);

	// DonGato - remove .movie file
	srcName.Format(_T("%s\\%s.movie"), theApp.glob_prefs->GetTempDir(), GetPartMetFileName());
	if (_tremove(srcName))
		if (errno != ENOENT)
			AddLogLine(true, IDS_FAILDELETE,srcName);

	//--- xrmb:keepPartFileStats ---
	//--- remove the stats file ---
	RemovePartFileStats();
	//--- :xrmb ---
    
    fullname = newname; 
    
	// START - enkeyDEV(Ottavio84) -ChangeDir-
	directory = GetOutputDir();

    SetStatus(PS_COMPLETE);
    paused = false;
	UpdateDisplayedInfo(true);	//Cax2 - file completed bugfix
    AddLogLine(true, IDS_DOWNLOADDONE, GetFileName()); 
	// Purity - Sending message for completed file
	CString MessageText;
	MessageText.Format(GetResString(IDS_TBN_DOWNLOADDONE) + _T(" ") + GetFileName());
	theApp.smtpconnection->SendMail(MessageText, theApp.glob_prefs->GetUseDownloadNotifier(), theApp.glob_prefs->IsSMTPInfoEnabled());
    theApp.emuledlg->ShowNotifier(MessageText, TBN_DLOAD, false, theApp.glob_prefs->GetUseDownloadNotifier()); 
    if (renamed) 
    	AddLogLine(true, IDS_DOWNLOADRENAMED, _tcsrchr(newname, '\\') + 1); 
    theApp.knownfiles->SafeAddKFile(this); 

    // Added to change UAP to Normal after finished download of file by Tarod 
    if (CKnownFile::IsAutoPrioritized())
    { 
    	CKnownFile::SetAutoPriority(false); 
        CKnownFile::SetPriority(PR_NORMAL); 
    } 
    // End Tarod 

    theApp.downloadqueue->RemoveFile(this); 
	UpdateDisplayedInfo();

#ifdef OLD_SOCKETS_ENABLED
#ifdef MOBILE_MULE
	// mobilemule
	theApp.mmserver->AddFinishedFile(this);
	// end mobilemule
#endif //MOBILE_MULE
#endif //OLD_SOCKETS_ENABLED

    // Added by Tarod (starts next paused file) 
    if (theApp.glob_prefs->DownloadPausedOnComplete())
    {
    	theApp.downloadqueue->StartPausedDownload(cat); 
    }
	// Barry - Just in case
	// -khaos--+++> Extended Statistics Added 2-10-03
	theApp.glob_prefs->Add2DownCompletedFiles();		// Increments cumDownCompletedFiles in prefs struct
	theApp.glob_prefs->Add2DownSessionCompletedFiles(); // Increments sesDownCompletedFiles in prefs struct
	theApp.glob_prefs->SaveCompletedDownloadsStat();	// Saves cumDownCompletedFiles to INI
	// <-----khaos- End Statistics Modifications

//	transfered = filesize;
	
	if (directory != theApp.glob_prefs->GetIncomingDir())
	{
		for (POSITION pos = theApp.glob_prefs->shareddir_list.GetHeadPosition();pos != 0;theApp.glob_prefs->shareddir_list.GetNext(pos))
			if (theApp.glob_prefs->shareddir_list.GetAt(pos) == (directory + "\\"))
				return TRUE;
		theApp.glob_prefs->shareddir_list.AddTail(directory + "\\");
		theApp.sharedfiles->Reload();
	}

   	return TRUE; 

	EMULE_CATCH

   	return FALSE; 
} 


void  CPartFile::RemoveAllSources(bool bTryToSwap)
{
	EMULE_TRY

	//TODO transfer sources to other downloading files if possible
	POSITION pos1,pos2;
	CUpDownClient* cur_source;
	
	for (int sl=0;sl<SOURCESSLOTS;sl++)
		while (!srclists[sl].IsEmpty())
		{
			cur_source = srclists[sl].GetHead();
			if (bTryToSwap)
			{
				if (!cur_source->SwapToAnotherFile(NULL))	//eklmn: bugfix(08): A4AF switching
					theApp.downloadqueue->RemoveSource(cur_source);
				else	
					//eklmn: if switching was succesfull we need to remove the entry from graphical list
					theApp.emuledlg->transferwnd.m_wndDownloadList.RemoveSource(cur_source,this);
			}
			else
				theApp.downloadqueue->RemoveSource(cur_source);		
		}

	//eklmn: remove A4AF entry from graphical list
	if (!A4AF_srclist.IsEmpty()) 
	{
		for (pos1 = A4AF_srclist.GetHeadPosition(); (pos2 = pos1 ) != NULL;)
		{
			A4AF_srclist.GetNext(pos1);
			cur_source = A4AF_srclist.GetAt(pos2);
			theApp.emuledlg->transferwnd.m_wndDownloadList.RemoveSource(cur_source,this);
		}
	}

	EMULE_CATCH
}

void CPartFile::DeleteFile()
{
	EMULE_TRY

	ASSERT ( !m_bPreviewing );

	StopFile();		//Cax2 bugfix: hanging sources when deleting file

	if (m_hpartfile.m_hFile != INVALID_HANDLE_VALUE) m_hpartfile.Close(); // m_hpartfile isn't the handle to the file...

	if (_tremove(fullname))
		AddLogLine(true, IDS_ERR_DELETE, fullname);
	CString partfilename = fullname.Left(fullname.GetLength() - 4);	
	if (_tremove(partfilename))
		AddLogLine(true, IDS_ERR_DELETE, partfilename);

	// START - enkeyDEV(Ottavio84) -ChangeDir-
	CString dirFile = CString(fullname) + CString(".dir") ;
	if (_tremove(dirFile) && errno != ENOENT)
		AddLogLine(true, IDS_FAILDELETE, dirFile);
	// END - enkeyDEV(Ottavio84) -ChangeDir-

	// remove *.src file (thx to brotbuexe)
	CString srcName=_T("");
	/*
	srcName.Format(_T("%s\\%s.stsrc"),tempdir,partmetfilename);
	if (_tremove(srcName) && errno != ENOENT) 
			AddLogLine(true, IDS_FAILDELETE, srcName);
	*/

	// Tarod - erase .backup file
	srcName.Format(_T("%s\\%s.bak"), theApp.glob_prefs->GetTempDir(), partmetfilename);
	if (_tremove(srcName) && errno != ENOENT)
			AddLogLine(true, IDS_FAILDELETE, srcName);

	// DonGato - remove .movie file
	srcName.Format(_T("%s\\%s.movie"), theApp.glob_prefs->GetTempDir(), partmetfilename);
	if (_tremove(srcName) && errno != ENOENT)
			AddLogLine(true, IDS_FAILDELETE, srcName);

	// netwolf - remove .txtscr file
	m_sourcesaver.DeleteFile(this); //<<-- enkeyDEV(Ottavio84) -New SLS-

	//--- xrmb:keepPartFileStats ---
	//--- remove the stats file ---
	RemovePartFileStats();
	//--- :xrmb ---	
	//Cax2 - file properties get deleted inside one of these calls now...
	theApp.sharedfiles->RemoveFile(this);
	theApp.downloadqueue->RemoveFile(this);
	theApp.emuledlg->transferwnd.m_wndDownloadList.RemoveFile(this);
	
	delete this;

	EMULE_CATCH
}

bool CPartFile::HashSinglePart(uint16 partnumber)
{
	EMULE_TRY

	if ((GetHashCount() <= partnumber) && (GetPartCount() > 1)){
		AddLogLine(true, IDS_ERR_HASHERRORWARNING, GetFileName());
		this->hashsetneeded = true;
		return true;
	}
	else if(!GetPartHash(partnumber) && GetPartCount() != 1){
		AddLogLine(true, IDS_ERR_INCOMPLETEHASH, GetFileName());
		this->hashsetneeded = true;
		return true;		
	}
	else{
		uchar hashresult[16];
		m_hpartfile.Seek((LONGLONG)PARTSIZE*partnumber,0);
		uint32 length = PARTSIZE;
		if ((ULONGLONG)PARTSIZE*(partnumber+1) > m_hpartfile.GetLength())	// netwolf: download bugfix for files > 2 GB (bluecow)
			length = (m_hpartfile.GetLength()- ((ULONGLONG)PARTSIZE*partnumber));
			ASSERT( length <= PARTSIZE );
		CreateHashFromFile(&m_hpartfile,length,hashresult);

		if (GetPartCount() > 1 || GetFileSize()==PARTSIZE){	// netwolf: special case if filesize == partsize
			if (md4cmp(hashresult,GetPartHash(partnumber)))
				return false;
			else
				return true;
		}
		else{
			if (md4cmp(hashresult,filehash))
					return false;
				else
					return true;
		}
	}

	EMULE_CATCH

	return false;
}

bool CPartFile::IsCorruptedPart(uint16 partnumber)
{
	EMULE_TRY

	return corrupted_list.Find(partnumber);

	EMULE_CATCH

	return true;
}

bool CPartFile::IsMovie()
{
	//Cax2 - simplified things a bit
	return (IsAviMovie()||IsMpgMovie());
}

bool CPartFile::IsAviMovie()
{
	EMULE_TRY

	// DonGato, updated by Vorlost
	CString fname = GetFileName();
	fname.MakeLower();

	return (_tcsstr(fname,_T(".avi")) || _tcsstr(fname,_T(".ogm")));

	EMULE_CATCH

	return false;
}

bool CPartFile::IsMpgMovie()
{
	EMULE_TRY

	// DonGato, updated by Vorlost
	CString fname = GetFileName();
	fname.MakeLower();

	if(IsVLCInstalled())
	{
		return (_tcsstr(fname,_T(".mpg")) || _tcsstr(fname,_T(".mpe")) || _tcsstr(fname,_T(".dat")) || _tcsstr(fname,_T(".bin")));
	} else {
		return (_tcsstr(fname,_T(".mpg")) || _tcsstr(fname,_T(".mpe")) || _tcsstr(fname,_T(".dat")));
	}

	EMULE_CATCH

	return false;
}

bool CPartFile::IsMovieChunk(uint8 mode, uint16 chunk)
{
	EMULE_TRY

	uint32 lastblock = ceil((float)PARTSIZE/EMBLOCKSIZE),end;
	if (chunk == GetPartCount()-2)
		end = (PARTSIZE*chunk) + (lastblock*EMBLOCKSIZE)-1; //Endadress from 1chunk before the last chunk

	switch (mode)
	{
 		case 0: //Mode 0: No Preview
			return false;
			break;
 		case 1: //Mode 1: 1F,1L(2L) AVI
  			if (chunk == 0)
				return true; //1F
  			if (chunk == GetPartCount()-1)
				return true; //1L is a moviechunk ?
  			if (chunk == (GetPartCount()-2) && (((GetFileSize()-1) - MINLASTMOVIESIZE ) <= end))
				return true; //2L is a Part of 1/8L at the end of the file
  			break;
 		case 2: //Mode 2: 1F,2F,1L(2L) Long AVI
  			if (chunk == 0)
				return true; //1F
  			if (chunk == 1)
				return true; //2F
  			if (chunk == GetPartCount()-1)
				return true; //1L is a moviechunk ?
  			if (chunk == (GetPartCount()-2) && (((GetFileSize()-1) - MINLASTMOVIESIZE ) <= end))
				return true; //2L is a Part of 1/8L at the end of the file
  				break;
 		case 3: //Mode 3: 1F,2F,3F,1L(2L) Longest AVI
  			if (chunk == 0)
				return true; //1F
  			if (chunk == 1)
				return true; //2F
  			if (chunk == 2)
				return true; //3F
  			if (chunk == GetPartCount()-1)
				return true; //1L is a moviechunk
  			if (chunk == (GetPartCount()-2) && (((GetFileSize()-1) - MINLASTMOVIESIZE ) <= end))
				return true; //2L is a Part of 1/8L at the end of the file
  				break;
 		case 4: //Mode 4: 1F MPG
  			if (chunk == 0)
				return true; //1F
  			break;
 		case 5: //Mode 5: 1F,2F Long MPG
  			if (chunk == 0)
				return true; //1F
  			if (chunk == 1)
				return true; //2F
  			break;
 		case 6: //Mode 6: 1F,2F,3F Longest MPG
  			if (chunk == 0)
				return true; //1F
  			if (chunk == 1)
				return true; //2F
  			if (chunk == 2)
				return true; //3F
  			break;
 		case 7: //Mode 7: 1F,2F,1L,2L AVI/MPG normal
  			if (chunk == 0)
				return true; //1F
  			if (chunk == 1)
				return true; //2F
  			if (chunk == GetPartCount()-1) 
				return true; //1L is a moviechunk ?
  			if (chunk == GetPartCount()-2)
				return true; //2L is a moviechunk ?
  			break;
 		default:
  			return false;
			break;
	}

	EMULE_CATCH

	return false;
}

bool CPartFile::IsBetterMovieChunk(uint16 oldchunk,uint16 newchunk)
{
	EMULE_TRY

	//The routine is based on that both old&new-chunk are moviechunks
	if (oldchunk == 0) //Chunk = 1F
	{
		// 1/8F & !1/8L & (newchunk=1L | 2L)
 		if (IsComplete(0,(MINFIRSTMOVIESIZE)-1) //first 1/8 chunk present
  			&& !IsComplete( (GetFileSize()-1) - (MINLASTMOVIESIZE) ,GetFileSize()-1) //last 1/8 chunk not present
  			&& (newchunk == GetPartCount()-1  //better chunk could be 1L
   			|| (newchunk == GetPartCount()-2))) //better chunk could be 2L
				return true;
	}
	else if (oldchunk == GetPartCount()-1 || oldchunk == GetPartCount()-2) //Chunk = 1L | 2L
	{
 		// (1/8L & !1/8F) | (1/8L & 1/8F)
 		if (((!IsComplete(0,(MINFIRSTMOVIESIZE)-1) //first 1/8 chunk not present
    		&& IsComplete( (GetFileSize()-1) - (MINLASTMOVIESIZE) ,GetFileSize()-1)) //last 1/8 chunk present
   			||(IsComplete(0,(MINFIRSTMOVIESIZE)-1) //first 1/8 chunk present
        	&& IsComplete( (GetFileSize()-1) - (MINLASTMOVIESIZE) ,GetFileSize()-1)) //last 1/8 chunk present
            && newchunk == 0))  //better chunk could be 1F
				return true;
	}
	else if (oldchunk == 1) //Chunk = 2F
	{
 		// 1F | 1L
 		if (newchunk == 0 || newchunk == GetPartCount()-1)
			return true;
	}
	else if (oldchunk == 2) //Chunk = 3F
	{
 		// 1F | 1L otherwise 2F
 		if (newchunk == 0 || newchunk == GetPartCount()-1)
			return true;
 		else if (newchunk == 1)
			return true;
	}

	EMULE_CATCH

	return false;
}

//Movie1 disables (movie --> normal) & (movie --> worst movie)
bool CPartFile::Movie1(uint8 mode,uint16 oldchunk,uint16 newchunk)
{
	EMULE_TRY

	if (IsMovie())
	{
 		if (!IsMovieChunk(mode,newchunk) && IsMovieChunk(mode,oldchunk))
  			return false;
 		if (IsMovieChunk(mode,newchunk) && IsMovieChunk(mode,oldchunk))
		{
  			if (!IsBetterMovieChunk(oldchunk,newchunk))
   				return false;
		}
	}
	return true;

	EMULE_CATCH

	return false;
}

//Movie2 enables (normal --> movie) & (movie --> better movie)
bool CPartFile::Movie2(uint8 mode,uint8 raremovie,bool forcepreview,uint16 oldchunk,uint16 newchunk)
{
	EMULE_TRY

	if (IsMovie() && ((m_SrcpartFrequency[oldchunk] > raremovie
    	&& m_SrcpartFrequency[newchunk] > raremovie)      //old & new chunk are nonrare
    	|| (m_SrcpartFrequency[oldchunk] == m_SrcpartFrequency[newchunk]) //old & new chunk are same rare
    	|| forcepreview))            //Load preview chunks first no matter of rareness
	{
			//This Line just for debugging:
			//AddLogLine(true,false,CString("Movie2::Mode:%d old:%d new:%d rarem:%d - The new & old ones are nonrare,samerare or forced, now we decide if new is a better movie or could change to movie..."),mode,oldchunk,newchunk,raremovie);
 			if (!IsMovieChunk(mode,oldchunk) && IsMovieChunk(mode,newchunk))
			{
  				if (forcepreview)
					return true;
 			}
 			if (IsMovieChunk(mode,oldchunk) && IsMovieChunk(mode,newchunk))
			{
  				if (IsBetterMovieChunk(oldchunk,newchunk))
				{
   					if (forcepreview)
	 					return true;
  				}
 			}
	}

	EMULE_CATCH

	return false;
}

uint8 CPartFile::PrepareComparePart(uint16 chunk)
{
	EMULE_TRY

	uint8 addchunk = 0;

	if (chunk == GetPartCount()-1) //Is this the last chunk ?
	{
 		uint32 lastblock = ceil((float)PARTSIZE/EMBLOCKSIZE);
 		uint32 lastblockinpart = lastblock;
 		uint32 start;
    	//Get last possible blocknumber
 		for (lastblockinpart = 0; lastblockinpart != lastblock;lastblockinpart++)
		{
  			start = (PARTSIZE*chunk) + lastblockinpart*EMBLOCKSIZE;
  			if (start >= GetFileSize())
   				break;
 		}
 		addchunk = lastblock - lastblockinpart;
	}

	return GetRemainingBlocksInPart(chunk) + addchunk;

	EMULE_CATCH

	return 0;
}
//Vorlost for LoadRarestFinishedChunks Patch2 end

// Barry - Also want to preview zip/rar files
bool CPartFile::IsArchive()
{
	CString extension = CString(GetFileName()).Right(4);
	return ((extension.CompareNoCase(_T(".zip")) == 0) || (extension.CompareNoCase(_T(".rar")) == 0));
}

void CPartFile::SetPriority(uint8 np)
{
	EMULE_TRY

	priority = np;
	theApp.downloadqueue->SortByPriority();
	//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
	UpdateDisplayedInfo(true);
	SavePartFile();

	EMULE_CATCH
}

void CPartFile::StopFile()
{
	EMULE_TRY

	if (status==PS_COMPLETE || status==PS_COMPLETING)
		return;

	//eklmn: set status to PS_STOPPED in order to prevent any action from other threads
	//SetStatus(PS_STOPPED); <- DonGato: this is wrong, we break the sharing of incomplete files and make a bogus tag

	// Barry - Need to tell any connected clients to stop sending the file
	// eklmn: send a downloading client a cancel
	POSITION pos1,pos2;
	CUpDownClient* cur_src;
	Packet* packet = new Packet(OP_CANCELTRANSFER,0);
	for (pos1 = m_downloadingSourceList.GetHeadPosition();( pos2 = pos1 ) != NULL;)
	{	
		m_downloadingSourceList.GetNext(pos1);
		cur_src = m_downloadingSourceList.GetAt(pos2);;
		theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
#ifdef OLD_SOCKETS_ENABLED
		cur_src->socket->SendPacket(packet,false,true);
#endif //OLD_SOCKETS_ENABLED
		cur_src->SetDownloadState(DS_ONQUEUE);		
	}
	delete packet;

	//eklmn: reset A4AF auto file
	if (theApp.downloadqueue->GetA4AFAutoFile() == this)
		theApp.downloadqueue->SetA4AFAutoFile(NULL);
	
	// it is poitless to hide sources if they will be deleted
	//theApp.emuledlg->transferwnd.m_wndDownloadList.HideSources(this); // cax2 closefile bugfix
	RemoveAllSources(true);
	paused = true;
	stopped = true;
	datarate = 0;
	// reset a statistical variables
	m_iSrcNNP =	0;
	m_iSrcOnQueue =	0;
	m_iSrcHighQR = 0;
	m_iSrcConnecting = 0;
	m_iSrcTooManyConns = 0;
	m_iSrcConnected = 0;
	m_iSrcTimeout = 0;
	m_iSrcConnViaServer = 0;
	m_iSrcUnknown =	0;
	m_iSrcRecHash = 0;
	m_iSrcLowToLow = 0;
	m_iSrcQueueFull = 0;
	m_iSrcNoUpload = 0;
	m_iSrcBanned = 0;
	m_iSrcError = 0;
	FlushBuffer();
	//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
	UpdateDisplayedInfo(true);
	SavePartFile();

	EMULE_CATCH
}

// v- eklmn: bugfix(02): file were paused not properly.... (who forgot the brackets?! :O)
void CPartFile::PauseFile()
{
	EMULE_TRY

	if (status==PS_COMPLETE || status==PS_COMPLETING)
		return;

	//eklmn: set status to PS_PAUSED in order to prevent any action from other threads
	//SetStatus(PS_PAUSED); <- DonGato: this is wrong, we break the sharing of incomplete files and make a bogus tag
	
	Packet* packet = new Packet(OP_CANCELTRANSFER,0);
	CUpDownClient* cur_src;
	POSITION pos1,pos2;

	//eklmn: send a downloading client a cancel
	for (pos1 = m_downloadingSourceList.GetHeadPosition();( pos2 = pos1 ) != NULL;)
	{	
		m_downloadingSourceList.GetNext(pos1);
		cur_src = m_downloadingSourceList.GetAt(pos2);;
		theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
#ifdef OLD_SOCKETS_ENABLED
		cur_src->socket->SendPacket(packet,false,true);
#endif //OLD_SOCKETS_ENABLED
		cur_src->SetDownloadState(DS_ONQUEUE);		
	}
	//eklmn: at this point nobody is downloading, and we can try to swap sources
	for (int sl=0;sl<SOURCESSLOTS;sl++) {
		if (!srclists[sl].IsEmpty()) {
			for( pos1 = srclists[sl].GetHeadPosition(); ( pos2 = pos1 ) != NULL; ) {
				srclists[sl].GetNext(pos1);
				CUpDownClient* cur_src = srclists[sl].GetAt(pos2);
				// eklmn: swap source to any other file
				if (cur_src->GetDownloadState() == DS_ONQUEUE)
					cur_src->SwapToAnotherFile(NULL);
			}
		}
	}
	delete packet;
	paused = true;
	datarate = 0;
	UpdateDisplayedInfo(true);
	SavePartFile();

	EMULE_CATCH
}
// ^- eklmn: bugfix(02): file were paused not properly.... (who forgot the brackets?! :O)

void CPartFile::ResumeFile()
{
	EMULE_TRY

	if (status==PS_COMPLETE || status==PS_COMPLETING) return;
	//eklmn: check if chunks are availibla & set status
	SetStatus(PS_EMPTY);
	for (int i = 0; i < GetPartCount(); i++) 
	{
		uint32 uStart = i* PARTSIZE;
		uint32 uEnd   = ((filesize - 1) < (uStart + PARTSIZE - 1)) ? (filesize - 1) : (uStart + PARTSIZE - 1);
		if (IsComplete(uStart,uEnd) == true)
		{
			SetStatus(PS_READY);
			break;
		}
	}
	
	if (GetStartTimeReset())
	{	//SyruS needed for files which are paused after adding or loading
		m_SessionStartTime = CTime::GetCurrentTime();
		SetStartTimeReset(false);
	}
	
	paused = false;
	stopped = false;
	lastsearchtime = 0;
	SavePartFile();
	//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
	UpdateDisplayedInfo(true);

	EMULE_CATCH
}

// Added by Vorlost to load initial sources from Server after connect 
void CPartFile::GetSourcesAfterServerConnect()
{
	EMULE_TRY

	if (lastsearchtime>0){ 
		//AddLogLine(true,CString("Resetting LastSearchTime:%d in Filename:%s"),lastsearchtime,GetFileName());
		lastsearchtime = 0; 
	} 

	EMULE_CATCH
}

CString CPartFile::GetPartfileStatus()
{
	EMULE_TRY

	CString mybuffer=_T(""); 
	if (GetTransferingSrcCount()>0) mybuffer=GetResString(IDS_DOWNLOADING); else mybuffer=GetResString(IDS_WAITING); 
	switch (GetStatus()) {
		case PS_HASHING: 
		case PS_WAITINGFORHASH:
			mybuffer=GetResString(IDS_HASHING);
			break; 
		case PS_COMPLETING:
			mybuffer=GetResString(IDS_COMPLETING);
			break; 
		case PS_COMPLETE:
			mybuffer=GetResString(IDS_COMPLETE);
			break;
		case PS_PAUSED:
			mybuffer=GetResString(IDS_PAUSED);
			break; 
		case PS_ERROR:
			mybuffer=GetResString(IDS_ERRORLIKE);
			break;
		case PS_STOPPED:
			mybuffer=GetResString(IDS_STOPPED);
			break;
	} 
	return mybuffer; 

	EMULE_CATCH

	return _T("");
} 

int CPartFile::getPartfileStatusRang()
{
	EMULE_TRY 

	// Sort order is Complete - Completing - Downloading - Waiting - Paused - Errors

	int status=2;
	if (GetTransferingSrcCount()==0)
		status=3;
	switch (GetStatus()) {
		case PS_HASHING: 
		case PS_WAITINGFORHASH:
			status=5;
			break; 
		case PS_COMPLETING:
			status=1;
			break; 
		case PS_COMPLETE:
			status=0;
			break; 
		case PS_PAUSED:
			status=4;
			break; 
		case PS_STOPPED:
			status=6;
			break;
		case PS_ERROR:
			status=7;
			break;
	} 
	return status;

	EMULE_CATCH

	return 1;
} 

sint32 CPartFile::GetTimeRemaining(bool bAvgerage /*=false*/)
{
	EMULE_TRY

	uint32 DataRate = 0;
	sint32 timeRemaining = -1;
	if (bAvgerage)
		DataRate = GetAvgDatarate();
	else
		DataRate = GetDatarate();
	timeRemaining = (DataRate > 0 ? (GetFileSize()-GetCompletedSize()) / DataRate : -1);
	return ((timeRemaining > 0 && timeRemaining <= 8640000) ? timeRemaining : -1);

	EMULE_CATCH

	return -1;
} 

//bond006 (smoothed "remaining time" calculation) //SyruS
uint32 CPartFile::GetAvgDatarate(bool bUpdated /*=false*/) {
	EMULE_TRY

	if (bUpdated)
	{
		CTimeSpan timespan;
		//calc on last FlushBuffer() to keep sessionstats for completed files
		if (status == PS_COMPLETING || status == PS_COMPLETE)
			timespan = GetFlushTimeSpan();
		else
			timespan = GetSessionTimeSpan();
		//delay 1st calculation a bit & dont use datarate for balancing (peak-stopper?)
		uint32 seconds = timespan.GetTotalSeconds();
		if (seconds > 10)
			m_AvgDataRate = ( (GetSessionTransfered() /*+ GetDatarate()*/) / (seconds /*+ 1*/) );
		else
			m_AvgDataRate = 0;
	}
	
	EMULE_CATCH
	return m_AvgDataRate;
}

void CPartFile::PreviewFile()
{
	EMULE_TRY

	if (IsArchive())
	{
		if ((!m_bRecoveringArchive) && (!m_bPreviewing))
			CArchiveRecovery::recover(this, true);
		return;
	}

	if (!PreviewAvailable())
	{
		ASSERT( false );
	 	return;
	}

	if (theApp.glob_prefs->BackupPreview()) {
		m_bPreviewing = true;
		CPreviewThread* pThread = (CPreviewThread*) AfxBeginThread(RUNTIME_CLASS(CPreviewThread), THREAD_PRIORITY_NORMAL+ theApp.glob_prefs->GetMainProcessPriority(),0, CREATE_SUSPENDED);
		pThread->SetValues(this);
		pThread->ResumeThread();
	} else {
		CString strLine = this->GetFullName();
		CString player=theApp.glob_prefs->GetVideoPlayer();
		if(player.IsEmpty())
			ShellExecute(NULL, _T("open"), strLine, NULL,NULL, SW_SHOWNORMAL);
		else
		{
			if (player.Find(_T("vlc.exe")))
				strLine.Replace(_T(".met"),_T(""));
			TCHAR shortPath[512]; //Cax2 short path for vlc
			GetShortPathName(strLine,shortPath,512);
			CString runDir=player;
			int i=runDir.ReverseFind('\\');
			runDir=(i>=0)?runDir.Left(i+1):_T("");
			ShellExecute(NULL, _T("open"),player, shortPath, runDir, SW_SHOWNORMAL);
		}
	}

	EMULE_CATCH
}

bool CPartFile::FirstLastLoaded()
{
	bool first = (IsComplete(0,(MINFIRSTMOVIESIZE)-1));
	//Cax2 - we "have" the last chunk (i.e. we don't care about it) if we're previewing using vlc, or if we have an mpeg
	bool last = IsVLCInstalled() || IsMpgMovie() || (IsComplete((GetFileSize()-1) - (MINLASTMOVIESIZE) ,GetFileSize()-1 ));
	return (first && last);
}

bool CPartFile::PreviewAvailable()
{
	EMULE_TRY
	//Cax2 - this would stop people from previewing almost complete movies
	//if vlan is previewing the file, the file *will not* be transfered 
	//to the incoming directory on completion, and emule will display an error message.
	//commented it out, to let our users do as they please!!!
	/*
	if ( gaplist.GetCount() <= 1 )
	{
		return false;
	}
	*/
	uint64 buf1,buf2,space;
	space = 0;
	GetDiskFreeSpaceEx(tempdir,(PULARGE_INTEGER)&space,(PULARGE_INTEGER)&buf1,(PULARGE_INTEGER)&buf2);
	// Barry - Allow preview of archives of any length > 1k

	if (GetStatus() == PS_COMPLETING || GetStatus() == PS_COMPLETE || GetStatus() == PS_HASHING)		// no preview! :)
		return false;

	if (IsArchive()) //before checking IsMovie!
		return (GetFileSize() > 1024 && !m_bRecoveringArchive && (space + 100000000) > (2*GetFileSize()));

	if (!IsMovie())
		return false;

	//Vorlost
	char buffer[200] = "";	//used for feedback in debug log
	bool first = (IsComplete(0,(MINFIRSTMOVIESIZE)-1));
	//Cax2 - we "have" the last chunk (i.e. we don't care about it) if we're previewing using vlc, or if we have an mpeg
	bool last = IsVLCInstalled() || IsMpgMovie() || (IsComplete((GetFileSize()-1) - (MINLASTMOVIESIZE) ,GetFileSize()-1 ));

	//let's build that string...
 	if (!first)
	{
   		strcat(buffer,"first ");
   		if (!last)
			strcat(buffer,"& last ");
  	}
  	else if (!last)
		strcat(buffer," last ");

	if (!first || !last) //they sould "be" both there for a preview
	{
		strcat(buffer,"movie block(s) not downloaded yet");
		AddDebugLogLine(false,_T("Movie preview: %s"),buffer);
		return false; 
	}

	if (space < (GetFileSize() + 100000000)  && theApp.glob_prefs->BackupPreview())		//Cax2 - now the logic makes sense: allow for an extra 100mb on top of the film preview.
	{
		AddDebugLogLine(false,_T("Movie preview: too little space on disk to enable preview."));
		return false;
	}

	if ( m_bPreviewing)
	{
		AddDebugLogLine(false,_T("Movie preview: only one preview at a time!"));
		return false;
	}
	return true;
	//End of Patch by Vorlost 

	EMULE_CATCH

	return false;
}

/*
void CPartFile::UpdateAvailablePartsCount(){
	EMULE_TRY

	uint8 availablecounter = 0;
	bool breakflag = false;
	uint16 iPartCount = GetPartCount();
	for (uint32 ixPart = 0; ixPart < iPartCount; ixPart++){
		breakflag = false;
		for (uint32 sl = 0; sl < SOURCESSLOTS && !breakflag; sl++){
			if (!srclists[sl].IsEmpty()){
				for(POSITION pos = srclists[sl].GetHeadPosition(); pos && !breakflag; ){
					if (srclists[sl].GetNext(pos)->IsPartAvailable(ixPart)){ 
						availablecounter++; 
						breakflag = true;
					}
				}
			}
		}
	}
	if (iPartCount == availablecounter && availablePartsCount < iPartCount)
		lastseencomplete = CTime::GetCurrentTime();
	availablePartsCount = availablecounter;
	m_bUpdateCompleteSrcCount= true;	// netwolf: zegzav:completesrc

	EMULE_CATCH
}
*/

Packet*	CPartFile::CreateSrcInfoPacket(CUpDownClient* forClient)
{
	EMULE_TRY

	if (forClient->reqfile != this)
		return NULL;

	if(GetSourceCount() == 0) return NULL;		//netwolf 07.05.03

	CMemFile data;
	uint16 nCount = 0;
	CUpDownClient* cur_src;
	bool bNeeded;
	POSITION pos1, pos2;

	data.Write(filehash,16);
	data.Write(&nCount,2);
	
	for (int sl=0;sl<SOURCESSLOTS;sl++) 
		if (!srclists[sl].IsEmpty())
			for (pos1 = srclists[sl].GetHeadPosition();( pos2 = pos1 ) != NULL;)
			{
				bNeeded = false;
				srclists[sl].GetNext(pos1);
				cur_src = srclists[sl].GetAt(pos2);

				// Modified by Tarod -sends sources with LowID that are in same sever
				// that client is requesting sources
				if (cur_src->HasLowID() 
					&& (cur_src->GetServerIP() != forClient->GetServerIP() 
					|| cur_src->GetServerPort() != forClient->GetServerPort()))
				{
					continue;
				}

				// LANCAST (moosetea) - We don't share Lan clients sources as they are private ips/userids
				if (cur_src->isOnLAN()) continue;

				// Added by Tarod, do not send incomplete established conns. to avoid sending
				// no valid sources
				switch (cur_src->GetDownloadState()) 
				{
					case DS_NONE:
					case DS_CONNECTING:
					case DS_ERROR:
					case DS_TOOMANYCONNS:
						continue;
						break;
				}

				// only send source which have needed parts for this client if possible
				if (forClient->reqfile == this && forClient->GetPartStatus() && cur_src->GetPartStatus())
				{
					uint8* reqstatus = forClient->GetPartStatus();
					uint8* srcstatus = cur_src->GetPartStatus();
					for (int x = 0; x != GetPartCount(); x++)
					{
						if (srcstatus[x] && !reqstatus[x])
						{
							bNeeded = true;
							break;
						}
					}
				}
				else if (cur_src->GetPartStatus())	//netwolf 04.05.03
				{
					uint8* srcstatus = cur_src->GetPartStatus();
					for (int x = 0; x < GetPartCount(); x++)
					{
						if (srcstatus[x])
						{
							bNeeded = true;
							break;
						}
					}
				}								// netwolf end

				if( bNeeded )
				{
					nCount++;
					uint32 dwID = cur_src->GetUserID();
					uint16 nPort = cur_src->GetUserPort();
					uint32 dwServerIP = cur_src->GetServerIP();
					uint16 nServerPort = cur_src->GetServerPort();
					data.Write(&dwID,4);
					data.Write(&nPort,2);
					data.Write(&dwServerIP,4);
					data.Write(&nServerPort,2);
					if (nCount > 500)
						break;
				}
			}
			
	if (!nCount)
		return 0;
	data.Seek(16,0);
	data.Write(&nCount,2);

	Packet* result = new Packet(&data,OP_EMULEPROT);
	result->opcode = OP_ANSWERSOURCES;
	if (nCount > 28)
		result->PackPacket();
	//AddDebugLogLine(false, "Send:Source User(%s) File(%s) Count(%i)", forClient->GetUserName(), GetFileName(), nCount);
	return result;

	EMULE_CATCH

	return NULL;
}


//v- eklmn: support Source Exchange v2
void CPartFile::AddClientSources(CMemFile* sources, uint8 sourceexchangeversion)
{
	EMULE_TRY

	//if this part file is paused or complete/completing no sources needed any more (Cax2 + SyruS check)
	if (GetStatus() == PS_PAUSED 
		|| GetStatus() == PS_STOPPED
		|| GetStatus() == PS_COMPLETE
		|| GetStatus() == PS_COMPLETING) 
		return;
	
	uint16 nCount;
	sources->Read(&nCount,2);

/*	if (theApp.glob_prefs->IsRemoveNNSEnabled())  //Vorlost added prefs question
	{
	    // Added by Tarod - calculate if we need more space than what is disponible
	    // if so remove No Needed Sources
	    int source_count_plus_new = GetSourceCount() + nCount;
	    if (source_count_plus_new > theApp.glob_prefs->GetMaxSourcePerFile()
		    && ((lastpurgetime == 0) || (::GetTickCount() - lastpurgetime) > PURGE_TIME)) {
    
		    RemoveNoNeededSources(source_count_plus_new - theApp.glob_prefs->GetMaxSourcePerFile());
	    }
	}
*/
	// Check if the data size matches the 'nCount' for v1 or v2 and eventually correct the source
	// exchange version while reading the packet data. Otherwise we could experience a higher
	// chance in dealing with wrong source data, userhashs and finally duplicate sources.
	UINT uDataSize = sources->GetLength() - sources->GetPosition();
	if ((UINT)nCount*(4+2+4+2) == uDataSize)
		sourceexchangeversion = 1;
	else if ((UINT)nCount*(4+2+4+2+16) == uDataSize)
		sourceexchangeversion = 2;
	else{
		// If v3 inserts additional data (like v2), the above code will correctly filter those packets.
		// If v3 appends additional data after <count>(<Sources>)[count], we are in trouble with the 
		// above code. Though a client which does not understand v3+ should never receive such a packet.
		theApp.emuledlg->AddDebugLogLine(false, _T("Received invalid source exchange packet (v%u) of data size %u for %s\n"), sourceexchangeversion, uDataSize, GetFileName());
		return;
	}

	for (int i = 0;i != nCount;i++){
		CClientSource source;
		sources->Read(&source.sourceID,4);
		sources->Read(&source.sourcePort,2);
		sources->Read(&source.serverIP,4);
		sources->Read(&source.serverPort,2);		
		if (sourceexchangeversion > 1)
			sources->Read(&source.achUserHash,16);
		AddClientSource(&source, i,true,sourceexchangeversion);
	}

	EMULE_CATCH
}
//^- eklmn: support Source Exchange v2

void CPartFile::GetSizeToTransferAndNeededSpace(uint32& pui32SizeToTransfer, uint32& pui32NeededSpace) 
{
	EMULE_TRY

	for (POSITION pos = gaplist.GetHeadPosition();pos != 0;gaplist.GetNext(pos)){
		Gap_Struct* cur_gap = gaplist.GetAt(pos);
		pui32SizeToTransfer += cur_gap->end - cur_gap->start;
		if(cur_gap->end == GetFileSize()-1)
			pui32NeededSpace = cur_gap->end - cur_gap->start;
	}

	EMULE_CATCH
}

// Updates priority of file if autopriority is activated
void CPartFile::UpdateDownloadAutoPriority(void)
{
	EMULE_TRY

	if (IsAutoPrioritized()) {
		CPartFile* partfile = theApp.downloadqueue->GetFileByID(GetFileHash()) ;
		
		if (partfile != NULL && partfile->IsPartFile()) {
			int valid_sources = partfile->GetSourceCount() - partfile->GetNotCurrentSourcesCount();
			if (valid_sources < theApp.glob_prefs->PriorityHigh() && priority != PR_HIGH)
				SetPriority(PR_HIGH) ;
			else if (valid_sources >= theApp.glob_prefs->PriorityHigh() && valid_sources < theApp.glob_prefs->PriorityLow() && priority != PR_NORMAL)
				SetPriority(PR_NORMAL) ;
			else if (valid_sources >= theApp.glob_prefs->PriorityLow() && priority != PR_LOW)
				SetPriority(PR_LOW) ;
		} 
	}

	EMULE_CATCH
}

// making this function return a higher when more sources have the extended
// protocol will force you to ask a larger variety of people for sources
int CPartFile::GetCommonFilePenalty() {
	//TODO: implement, but never return less than MINCOMMONPENALTY!
	return MINCOMMONPENALTY;
}

CString CPartFile::GetStatsFullPath() const
{	
	return RemoveFileExtension(fullname) + _T(".stats");	
}

//--- xrmb:keepPartFileStats ---
bool CPartFile::LoadPartFileStats()
{
	EMULE_TRY
	
	FILE *fh = _tfopen(GetStatsFullPath(), _T("rb"));
	
	//--- if we cant open this file... you lose the stats ---
	if(fh==NULL)
		return false;
	
	//--- get version ---
	uint8	version=0;
	if(fread(&version, sizeof(version), 1, fh)!=1) { fclose(fh); return false; }

	uint32 alltimerequested=0;
	uint32 alltimeaccepted=0;
	uint64 alltimetransfered=0;

	if(fread(&alltimerequested,		sizeof(alltimerequested),	1, fh)!=1) { fclose(fh); return false; }
	if(fread(&alltimeaccepted,		sizeof(alltimeaccepted),	1, fh)!=1) { fclose(fh); return false; }
	if(fread(&alltimetransfered,	sizeof(alltimetransfered),	1, fh)!=1) { fclose(fh); return false; }

	statistic.AddAll(alltimerequested, alltimeaccepted, alltimetransfered);

	//--- this is for the parttraffic ---
	//--- i thought about explicit adding the traffic like for the other stats above ---
	//--- but it doesnt make sense, because you'll no transfer if you reach this ---
	if(LoadFromFileTraffic(fh, version)==false) { fclose(fh); return false;	}

	fclose(fh);

	return true;

	EMULE_CATCH

	return false;
}

bool CPartFile::SavePartFileStats()
{
	EMULE_TRY
	
	if (!fullname || IsBadStringPtr(fullname,MAX_PATH))
	{
		return false;
	}	
	FILE *fh = _tfopen(GetStatsFullPath(), _T("wb"));
	
	//--- if we cant open this file... you lose the stats ---
	if(fh==NULL)
		return false;
	
	//--- set version ---
	uint8	version=2;
	if(fwrite(&version, sizeof(version), 1, fh)!=1) { fclose(fh); return false; }

	uint32 alltimerequested=statistic.GetAllTimeRequests();
	uint32 alltimeaccepted=statistic.GetAllTimeAccepts();
	uint64 alltimetransfered=statistic.GetAllTimeTransfered();

	if(fwrite(&alltimerequested,	sizeof(alltimerequested),	1, fh)!=1) { fclose(fh); return false; }
	if(fwrite(&alltimeaccepted,		sizeof(alltimeaccepted),	1, fh)!=1) { fclose(fh); return false; }
	if(fwrite(&alltimetransfered,	sizeof(alltimetransfered),	1, fh)!=1) { fclose(fh); return false; }

	//--- this is for the parttraffic ---
	if(SaveToFileTraffic(fh)==false) { fclose(fh); return false; }

	fclose(fh);

	return true;

	EMULE_CATCH

	return false;
}

int CPartFile::RemovePartFileStats()
{
	EMULE_TRY	
	return _tremove(GetStatsFullPath());
	EMULE_CATCH

	return 0;
}
//--- :xrmb ---

// Used as separate thread to complete file [Lord KiRon]
UINT CPartFile::CompleteThreadProc(CPartFile* pFile) 
{ 
	EMULE_TRY

	if ( theApp.glob_prefs->IsNTBased() )
	{
		SetThreadLocale(theApp.glob_prefs->GetLanguageID());
	}

	if ( !pFile ) 
   	{ 
    	return -1; 
   	} 
	CSingleLock Lock(&s_ComplitingMutex,TRUE); // Prevent of compliting two files in same time
   	pFile->PerformFileComplete(); 
   	return 0; 

	EMULE_CATCH

   	return -1; 
}

bool CPartFile::IsVLCInstalled() 
{ 
	EMULE_TRY
	CString player=theApp.glob_prefs->GetVideoPlayer();
	if (player.Find(_T("vlc.exe"))>=0)
	{
		CFileFind ff;
		return ff.FindFile(player,0);
	}

	EMULE_CATCH

	return false;
}	

CString CPartFile::DtLocale(CTime dt){

	SYSTEMTIME st;
	dt.GetAsSystemTime(st); 

	int nDateSize = GetDateFormat(LOCALE_USER_DEFAULT, 0, &st, NULL, NULL, 0);
	int nTimeSize = GetTimeFormat(LOCALE_USER_DEFAULT, 0, &st, NULL, NULL, 0);

	CString strDate;
	GetDateFormat(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &st, NULL, strDate.GetBuffer(nDateSize), nDateSize);
	strDate.ReleaseBuffer();

	CString strTime; 
	GetTimeFormat(LOCALE_USER_DEFAULT, TIME_NOSECONDS, &st, NULL, strTime.GetBuffer(nTimeSize), nTimeSize); 
	strTime.ReleaseBuffer();

	CString strDay;
    switch ( dt.GetDayOfWeek() )
      {
         case 1:
			 strDay = GetResString(IDS_SCH_EXCEPT_SUN);
            break;
         case 2:
			 strDay = GetResString(IDS_SCH_EXCEPT_MON);
            break;
		case 3:
			strDay = GetResString(IDS_SCH_EXCEPT_TUE);
            break;
		case 4:
			strDay = GetResString(IDS_SCH_EXCEPT_WED);
            break;
		case 5:
			strDay = GetResString(IDS_SCH_EXCEPT_THU);
            break;
		case 6:
			strDay = GetResString(IDS_SCH_EXCEPT_FRI);
            break;
		case 7:
			strDay = GetResString(IDS_SCH_EXCEPT_SAT);
            break;
      }

	return 	strDay + _T(", ") + strDate + _T(", ") +  strTime; 

}

// DonGato - Movie Preview Mode
void CPartFile::LoadMovieMode()
{ 
	EMULE_TRY

   	CIni pvmoviefile(CString(fullname) + CString(".movie"));
	pvmoviefile.SetDefaultCategory(_T("MoviePreview"));
   	uint8 Mode = pvmoviefile.GetInt(_T("Mode"), 0); 
	CKnownFile::SetMovieMode(Mode);

	pvmoviefile.SetDefaultCategory(_T("DownloadSettings"));
    m_bDiscardSuperCompressed = pvmoviefile.GetBool(_T("DiscardSuperCompressed"),false);

	pvmoviefile.CloseWithoutSave();

	EMULE_CATCH
}

void CPartFile::SaveMovieMode()
{
	EMULE_TRY

   	CIni pvmoviefile(CString(fullname) + CString(".movie"));
	pvmoviefile.SetDefaultCategory(_T("MoviePreview"));
	pvmoviefile.SetInt(_T("Mode"), ((CKnownFile*)this)->GetMovieMode()); 

	pvmoviefile.SetDefaultCategory(_T("DownloadSettings"));
    pvmoviefile.SetBool(_T("DiscardSuperCompressed"),m_bDiscardSuperCompressed);

	pvmoviefile.SaveAndClose();

	EMULE_CATCH
}
// DonGato - Moview Preview Mode end

/* Barry - Replaces BlockReceived() 

           Originally this only wrote to disk when a full 180k block 
           had been received from a client, and only asked for data in 
		   180k blocks.

		   This meant that on average 90k was lost for every connection
		   to a client data source. That is a lot of wasted data.

		   To reduce the lost data, packets are now written to a buffer
		   and flushed to disk regularly regardless of size downloaded.
		   This includes compressed packets.

		   Data is also requested only where gaps are, not in 180k blocks.
		   The requests will still not exceed 180k, but may be smaller to
		   fill a gap.
*/
uint32 CPartFile::WriteToBuffer(uint32 transize, BYTE *data, uint32 start, uint32 end, Requested_Block_Struct *block)
{
	EMULE_TRY

	// Increment transfered bytes counter for this file
	transfered += transize;

	// This is needed a few times
	uint32 lenData = end - start + 1;

	if( lenData > transize )
		m_iGainDueToCompression += lenData-transize;

	// Occasionally packets are duplicated, no point writing it twice
	if (IsComplete(start, end))
	{
		AddDebugLogLine(false, _T("File '%s' has already been written from %ld to %ld"), GetFileName(), start, end);
		return 0;
	}

	// Create copy of data as new buffer
	BYTE *buffer = new BYTE[lenData];
	#ifndef AMD
	memcpy(buffer, data, lenData);
	#else
	memcpy_amd(buffer, data, lenData);
	#endif

	// Create a new buffered queue entry
	PartFileBufferedData *item = new PartFileBufferedData;
	item->data = buffer;
	item->start = start;
	item->end = end;
	item->block = block;

	// Add to the queue in the correct position (most likely the end)
	PartFileBufferedData *queueItem;
	bool added = false;
	POSITION pos = m_BufferedData_list.GetTailPosition();
	while (pos != NULL)
	{	
		queueItem = m_BufferedData_list.GetPrev(pos);
		if (item->end > queueItem->end)
		{
			added = true;
			m_BufferedData_list.InsertAfter(pos, item);
			break;
		}
	}
	if (!added)
		m_BufferedData_list.AddHead(item);

	// Increment buffer size marker
	m_nTotalBufferData += lenData;

	// Mark this small section of the file as filled
	FillGap(item->start, item->end);

	// Update the flushed mark on the requested block 
	// The loop here is unfortunate but necessary to detect deleted blocks.
	pos = requestedblocks_list.GetHeadPosition();
	while (pos != NULL)
	{	
		if (requestedblocks_list.GetNext(pos) == item->block)
			item->block->transferred += lenData;
	}

	// v- eklmn: bugfix(20): check last part in WriteToBuffer correct way
	// if (gaplist.IsEmpty()) FlushBuffer();  //netwolf: another approach to avoid sharing of corrupted parts (trockenbeerenauslese)
	uint16 partnumber = floor((float)start/PARTSIZE);
	const uint32 uStart = partnumber * PARTSIZE;
	const uint32 uEnd   = ((filesize- 1) < (uStart + PARTSIZE - 1)) ? (filesize- 1) : (uStart + PARTSIZE - 1);
	if (IsComplete(uStart, uEnd))
	{
		FlushBuffer(); //netwolf end
		m_ReqParts[partnumber] = PARTSIZE;
	}
	// ^- eklmn: bugfix(20): check last part in WriteToBuffer correct way

	// Return the length of data written to the buffer
	return lenData;

	EMULE_CATCH

	return 0;
}

void CPartFile::FlushBuffer(void)
{
	EMULE_TRY

	m_nLastBufferFlushTime = GetTickCount();  // obaldin
	if (m_BufferedData_list.IsEmpty())
		return;

	m_LastDownTransfer = CTime::GetCurrentTime();
	GetAvgDatarate(true);	//SyruS do update on flushing
	// there are a couple of ways of doing this, this is one... 
	uint32 partCount = (filesize/PARTSIZE)+((filesize%PARTSIZE)? 1 : 0);
	bool *changedPart = new bool[partCount];

//	AddDebugLogLine(false, "Flushing file %s - buffer size = %ld bytes (%ld queued items) transfered = %ld [time = %ld]\n", GetFileName(), m_nTotalBufferData, m_BufferedData_list.GetCount(), transfered, m_nLastBufferFlushTime);

	try
	{
		// Remember which parts need to be checked at the end of the flush
		for (int partNumber=0; (uint32)partNumber<partCount; partNumber++)
		{
			changedPart[partNumber] = false;
		}

		// Ensure file is big enough to write data to (the last item will be the furthest from the start)
		PartFileBufferedData *item = m_BufferedData_list.GetTail();
		if (m_hpartfile.GetLength() <= item->end)
			m_hpartfile.SetLength(item->end + 1);	// netwolf: file should not be too large (taab)
		if (m_hpartfile.GetLength()>filesize)
			m_hpartfile.SetLength(filesize);
		// Loop through queue
		for (int i = m_BufferedData_list.GetCount(); i>0; i--)
		{
			// Get top item
			item = m_BufferedData_list.GetHead();

			// This is needed a few times
			uint32 lenData = item->end - item->start + 1;

			//SyruS (0.29a) SLUGFILLER: SafeHash - could be more than one part
			/*int curpart = item->start/PARTSIZE;
			changedPart[curpart] = true;*/
			for (uint32 curpart = item->start/PARTSIZE; curpart <= item->end/PARTSIZE; curpart++)
			changedPart[curpart] = true;
			// SLUGFILLER: SafeHash

			// Go to the correct position in file and write block of data
			m_hpartfile.Seek(item->start, CFile::begin);
			m_hpartfile.Write(item->data, lenData);

			// Remove item from queue
			m_BufferedData_list.RemoveHead();

			// Decrease buffer size
			m_nTotalBufferData -= lenData;

			// Release memory used by this item
			delete [] item->data;
			delete item;
		}

		// Flush to disk
		m_hpartfile.Flush();

		// Check each part of the file
		uint32 partRange = (filesize % PARTSIZE) - 1;	//netwolf 03.05.03
		for (int partNumber = partCount-1; partNumber >= 0; partNumber--)
		{
			if (changedPart[partNumber] == false)
			{
				// Any parts other than last must be full size
				partRange = PARTSIZE - 1;
				continue;
			}

			// Is this 9MB part complete
			if ( IsComplete(PARTSIZE * partNumber, (PARTSIZE * (partNumber + 1)) - 1 ) )
			{
				// Is part corrupt
				if (!HashSinglePart(partNumber))
				{
					AddLogLine(true, IDS_ERR_PARTCORRUPT, partNumber, GetFileName());
					AddGap(PARTSIZE*partNumber, (PARTSIZE*partNumber + partRange));
					corrupted_list.AddTail(partNumber);
					// Reduce transfered amount by corrupt amount
					this->m_iLostDueToCorruption += (partRange + 1);
				}
				else
				{
					// Successfully completed part, make it available for sharing
					if (status == PS_EMPTY)
					{
						SetStatus(PS_READY);
						theApp.sharedfiles->SafeAddKFile(this);
					}
				}
			}
			else if ( IsCorruptedPart(partNumber))
			{
				// Try to recover with minimal loss
				if (HashSinglePart(partNumber))
				{
					m_iTotalPacketsSavedDueToICH++;
					FillGap(PARTSIZE*partNumber,(PARTSIZE*partNumber+partRange));
					RemoveBlockFromList(PARTSIZE*partNumber,(PARTSIZE*partNumber + partRange));
					AddLogLine(true, IDS_ICHWORKED, partNumber,GetFileName());
					// BavarianSnail - add info to stats
					theApp.glob_prefs->Add2SessionPartsSavedByICH();
				}
			}
			// Any parts other than last must be full size
			partRange = PARTSIZE - 1;
		}

		// Update met file
		SavePartFile();

		// Is this file finished?
		if (gaplist.IsEmpty() && !hashsetneeded) // SLUGFILLER: hashlistRecheck - can't complete without a hashset...
			CompleteFile(false);
	}
	catch (CFileException* error)
	{
		OUTPUT_DEBUG_TRACE();

		if (theApp.glob_prefs->IsErrorBeepEnabled()) Beep(800,200);

		if (error->m_cause == CFileException::diskFull) 
		{
			AddLogLine(true, IDS_ERR_OUTOFSPACE, GetFileName());
			// Purity - Sending message for insufficient diskspace
			CString MessageText;
			MessageText.Format(GetResString(IDS_ERR_OUTOFSPACE) + _T(" ") + GetFileName());
			theApp.smtpconnection->SendMail(MessageText, theApp.glob_prefs->GetNotifierPopOnImportantError(), theApp.glob_prefs->IsSMTPWarningEnabled());
			theApp.emuledlg->ShowNotifier(MessageText, TBN_IMPORTANTEVENT, false, theApp.glob_prefs->GetNotifierPopOnImportantError());
		}
		else
		{
			AddLogLine(true, IDS_ERR_WRITEERROR, GetFileName(), GetErrorMessage(error));
			SetStatus(PS_ERROR);
		}
        /*
		paused = true;
		datarate = 0;
		transferingsrc = 0;
	
		//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
		UpdateDisplayedInfo();
        */
        PauseFile();
		error->Delete();
	}
	catch(...)
	{		
		AddLogLine(true, IDS_ERR_WRITEERROR, GetFileName(), GetResString(IDS_UNKNOWN));
		SetStatus(PS_ERROR);
		PauseFile();
	}
	delete[] changedPart;

	EMULE_CATCH
}

// Barry - This will invert the gap list, up to caller to delete gaps when done
// 'Gaps' returned are really the filled areas, and guaranteed to be in order
void CPartFile::GetFilledList(CTypedPtrList<CPtrList, Gap_Struct*> *filled)
{
	EMULE_TRY

	Gap_Struct *gap = NULL;
	Gap_Struct *best=NULL;
	POSITION pos;
	uint32 start = 0;
	uint32 bestEnd = 0;

	// Loop until done
	bool finished = false;
	while (!finished)
	{
		finished = true;
		// Find first gap after current start pos
		bestEnd = filesize;
		pos = gaplist.GetHeadPosition();
		best = gaplist.GetHead();	//Cax2 - empty file bugfix
		while (pos != NULL)
		{
			gap = gaplist.GetNext(pos);
			if ((gap->start > start) && (gap->end < bestEnd))
			{
				best = gap;
				bestEnd = best->end;
				finished = false;
			}
		}

		if (!finished)
		{
			// Invert this gap
			gap = new Gap_Struct;
			gap->start = start;
			gap->end = best->start - 1;
			start = best->end + 1;
			filled->AddTail(gap);
		}
		else if (best->end < filesize)
		{
			gap = new Gap_Struct;
			gap->start = best->end + 1;
			gap->end = filesize;
			filled->AddTail(gap);
		}
	}
	EMULE_CATCH
}

void CPartFile::UpdateFileRatingCommentAvail() {
	EMULE_TRY

	if (!this) return;

	bool prev=(hasComment || hasRating);

	hasComment=false;
	hasRating=false;

	POSITION pos1,pos2;
	for (int sl=0;sl<SOURCESSLOTS;sl++) 
		if (!srclists[sl].IsEmpty())
			for (pos1 = srclists[sl].GetHeadPosition();( pos2 = pos1 ) != NULL;)
			{
				srclists[sl].GetNext(pos1);
				CUpDownClient* cur_src = srclists[sl].GetAt(pos2);
				if (cur_src->GetFileComment().GetLength()>0) 
					hasComment=true;
				if (cur_src->GetFileRate()>0) 
					hasRating=true;
				if (hasComment && hasRating) 
					break;
			}

	// #zegzav:pastcomment (BEGIN)
	if ((!hasComment) || (!hasRating))
	{
		// past comments
		for (pos1= m_PastComment.GetHeadPosition(); pos1 != 0;)
		{
			CPastComment &pc= m_PastComment.GetNext(pos1);
			if (pc.GetComment().GetLength() > 0)
				hasComment=true;
			if (pc.GetRate() > 0)
				hasRating=true;
			if (hasComment && hasRating)
				break;
		}
	}
	// #zegzav:pastcomment (END)

	if (prev != (hasComment || hasRating)) 
		UpdateDisplayedInfo();//theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);

	EMULE_CATCH
}

uint16 CPartFile::GetSourceCount() {
	EMULE_TRY

	uint16 count=0;
	for (int i=0;i<SOURCESSLOTS;i++) count+=srclists[i].GetCount();
	return count;

	EMULE_CATCH

	return 0;
}

int CPartFile::GetRating(){  
	EMULE_TRY

     if (!hasRating) return 0;  
     int num,tot,fRate;  
     num=tot=0;  
     POSITION pos1,pos2;  
     for (int sl=0;sl<SOURCESSLOTS;sl++) if (!srclists[sl].IsEmpty())  
     for (pos1 = srclists[sl].GetHeadPosition();( pos2 = pos1 ) != NULL;){  
          srclists[sl].GetNext(pos1);  
          fRate =((CUpDownClient*) srclists[sl].GetAt(pos2))->GetFileRate();  
          if (fRate>0)   
          {  
               num++;
			   //Cax2 - bugfix: for some $%#ing reason  fair=4 & good=3, breaking the progression from fake(1) to excellent(5)
			   if (fRate==3 || fRate==4) fRate=(fRate==3)?4:3;
               tot+=fRate;  
          }  
     }  

     // #zegzav:pastcomment (BEGIN)
     for (pos1= m_PastComment.GetHeadPosition(); pos1 != 0;)	
     {
          fRate= m_PastComment.GetNext(pos1).GetRate();
          if (fRate>0)   
          {  
               num++;
               //Cax2 - bugfix: for some $%#ing reason  fair=4 & good=3, breaking the progression from fake(1) to excellent(5)
               if (fRate==3 || fRate==4) fRate=(fRate==3)?4:3;
               tot+=fRate;  
          }  
     }
	// #zegzav:pastcomment (END)

	 if (num>0)
	 {
		 num=(float)tot/num+.5;	//Cax2 - get the average of all the ratings
	 	//Cax2 - bugfix: for some $%#ing reason good=3 & fair=4, breaking the progression from fake(1) to excellent(5)
		if (num==3 || num==4) num=(num==3)?4:3;
	 }
     return num;	//Cax2 - if no ratings found, will return 0!

	 EMULE_CATCH

	 return 0;
}

void CPartFile::UpdateDisplayedInfo(boolean force) {
	EMULE_TRY

    DWORD curTick = ::GetTickCount();

    if(force || curTick-m_lastRefreshedDLDisplay > MINWAIT_BEFORE_DLDISPLAY_WINDOWUPDATE+(uint32)(rand()*1000/RAND_MAX)) {
		if (theApp.emuledlg != NULL) {
			theApp.emuledlg->transferwnd.m_wndDownloadList.UpdateItem(this);
			m_lastRefreshedDLDisplay = curTick;
		}
    }

	EMULE_CATCH
}

// START enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
//v- eklmn: support Source Exchange v2
void CPartFile::AddClientSource(CClientSource* source, int i_Source, bool exchanged, uint8 sourceexchangeversion) 
{
	EMULE_TRY

	// check first if we are this source
#ifdef OLD_SOCKETS_ENABLED
	if (theApp.serverconnect->GetClientID() < 16777216 && theApp.serverconnect->IsConnected()) {
		if ((theApp.serverconnect->GetClientID() == source->sourceID) && theApp.serverconnect->GetCurrentServer()->GetIP() == source->serverIP)
			return;
		// although we are currently having a LowID, we could have had a HighID before, which
		// that client is sending us now! seems unlikely ... it happend!
		if (source->sourceID == theApp.serverconnect->GetLocalIP())
			return;
	}
	else if (theApp.serverconnect->GetClientID() == source->sourceID)
		return;
	// Modified by Tarod -use LowID source only if source is in our same server
	else if (source->sourceID < 16777216 && !theApp.serverconnect->IsLocalServer(source->serverIP, source->serverPort))
		return;
#endif //OLD_SOCKETS_ENABLED
	
	// eklmn: optimization: create source only if we need to do it
	if( theApp.glob_prefs->GetMaxSourcePerFile() > GetSourceCount()
		&& i_Source < (int)theApp.glob_prefs->GetMaxSourcePerFileSoft()) // DonGato - Using Max Sources per File (soft)	//Cax2 - used new call
	{ 
		CUpDownClient* newsource = new CUpDownClient(source->sourcePort,source->sourceID,source->serverIP,source->serverPort,this);
		if (exchanged) {
			//Cax2 - exchanged sources are named now
			newsource->SetUserName(GetResString(IDS_EXCHANGEDSOURCE));
			// check exchange version
			if (sourceexchangeversion > 1)
				newsource->SetUserHash(source->achUserHash);
		}
		theApp.downloadqueue->CheckAndAddSource(this,newsource);
	} 

	EMULE_CATCH
}
//^- eklmn: support Source Exchange v2

void CPartFile::AddClientSources(CTypedPtrList<CPtrList, CClientSource*>* sources){
	EMULE_TRY

	POSITION pos;
	pos=sources->GetHeadPosition();
	while (pos!=NULL) {
		CClientSource* source=(CClientSource*)sources->GetAt(pos);  

		if (source->sourceType != ED2KLINK_SOURCE_IP) {
			sources->GetNext(pos);
			continue;
		}

		POSITION removePos=pos;
		sources->GetNext(pos);
		sources->RemoveAt(removePos);

		AddClientSource(source);
		delete source;
	}
	theApp.downloadqueue->AddClientHostnameToResolve(sources);

	EMULE_CATCH
}
// END enkeyDEV(kei-kun) -hostname sources in ED2K superLink-

// START - enkeyDEV(Ottavio84) -Create source ed2k from downloads-
CString CPartFile::CreateED2KSourceLink(uint32 expirein, int sourcenum)
{
	EMULE_TRY

	CTypedPtrList<CPtrList, CUpDownClient*> srcstoadd;
	POSITION pos1,pos2;
	for (int sl=0;sl<SOURCESSLOTS;sl++) if (!srclists[sl].IsEmpty())
		for (pos1 = srclists[sl].GetHeadPosition();( pos2 = pos1 ) != NULL;){
			srclists[sl].GetNext(pos1);
			CUpDownClient* cur_src = srclists[sl].GetAt(pos2);
		if (cur_src->HasLowID())
			continue;
		if (srcstoadd.IsEmpty()) {
			srcstoadd.AddHead(cur_src);
			continue;
		}
		if ((sourcenum == 0) || (srcstoadd.GetCount() < sourcenum) || (cur_src->GetAvailablePartCount() > srcstoadd.GetTail()->GetAvailablePartCount())) {
			if ((sourcenum != 0) && (srcstoadd.GetCount() == sourcenum))
				srcstoadd.RemoveTail();
			ASSERT(srcstoadd.GetCount() < sourcenum);
			bool bInserted = false;
			for (POSITION pos2 = srcstoadd.GetTailPosition();pos2 != 0;srcstoadd.GetPrev(pos2)){
				CUpDownClient* cur_srctoadd = srcstoadd.GetAt(pos2);
				if (cur_srctoadd->GetAvailablePartCount() > cur_src->GetAvailablePartCount()) {
					srcstoadd.InsertAfter(pos2, cur_src);
					bInserted = true;
					break;
				}
			}
			if (!bInserted)
				srcstoadd.AddHead(cur_src);
		}
	}
#ifdef OLD_SOCKETS_ENABLED
	if (srcstoadd.IsEmpty() && (!theApp.serverconnect->IsConnected() || theApp.serverconnect->IsLowID())){
		AddLogLine(true, IDS_SOURCELINKFAILED);
		return CString("");
	}
#endif //OLD_SOCKETS_ENABLED
	CString strExpiration;
	if (expirein == 0)
		strExpiration = "";
	else {
		COleDateTime expirationDate = COleDateTime::GetCurrentTime();
		COleDateTimeSpan timediff;
		timediff.SetDateTimeSpan(expirein, 0, 0, 0);
		expirationDate += timediff;
		strExpiration.Format(_T("@%02i%02i%02i"), (expirationDate.GetYear() % 100), expirationDate.GetMonth(), expirationDate.GetDay());
	}  
	uint32 dwID;
#ifdef OLD_SOCKETS_ENABLED
	dwID = theApp.serverconnect->GetClientID();
#endif //OLD_SOCKETS_ENABLED
	CString strLink;
	strLink.Format(_T("ed2k://|file|%s|%d|%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x|/|sources%s"),
		theApp.StripInvalidFilenameChars(GetFileName(), false), // spaces to dots
		GetFileSize(),
		GetFileHash()[0],GetFileHash()[1],GetFileHash()[2],GetFileHash()[3],GetFileHash()[4],GetFileHash()[5],GetFileHash()[6],GetFileHash()[7],
		GetFileHash()[8],GetFileHash()[9],GetFileHash()[10],GetFileHash()[11],GetFileHash()[12],GetFileHash()[13],GetFileHash()[14],GetFileHash()[15],
		strExpiration);
	CString newsource;
	if (dwID >= 16777216) {
		newsource.Format(_T(",%i.%i.%i.%i:%i"),(uint8)dwID,(uint8)(dwID>>8),(uint8)(dwID>>16),(uint8)(dwID>>24), theApp.glob_prefs->GetPort());
		strLink.Append(newsource);
	}
	AddDebugLogLine(false, _T("Sources: %i"), (int)srcstoadd.GetCount());
	while (!srcstoadd.IsEmpty()) {
		CUpDownClient* cur_src = srcstoadd.RemoveHead();
		AddDebugLogLine(false, _T("Available part count: %i"), (int)cur_src->GetAvailablePartCount());
		newsource.Format(_T(",%s:%i"),cur_src->GetFullIP(), cur_src->GetUserPort());
		strLink.Append(newsource);
	}
	strLink.Append(_T("|/"));
	return strLink;

	EMULE_CATCH

	return _T("");
}
// END - enkeyDEV(Ottavio84) -Create source ed2k from downloads-

// START - enkeyDEV(Ottavio84) -ChangeDir-
void CPartFile::SetAlternativeOutputDir(CString* path) {
	EMULE_TRY

	path->TrimRight(_T("\\"));
	m_alternativepath = *path;

	EMULE_CATCH
}

CString CPartFile::GetOutputDir() {
	EMULE_TRY

	CString retpath = theApp.glob_prefs->GetIncomingDir();
	CString catpath = theApp.glob_prefs->GetCategory(GetCategory())->incomingpath;

	// If no alternative path defined
	if ( m_alternativepath.IsEmpty() )
	{
		if(catpath != "")
			return catpath; // Return the category dir
		else
			return retpath; // Return the incoming dir
	}
	// we going to use desktop folder "static" functions , we need an object
	// of IShellFolder type and DeskTop always exists
	IShellFolder *pDesktop	=	NULL;
	ITEMIDLIST *pList	=	NULL;
	CString dir;
	// Try to create DeskTop folder object
	if ( SHGetDesktopFolder(&pDesktop) == NOERROR )
	{
		IShellFolder *pFolder = NULL;
		ULONG	uAttrib	=	SFGAO_FOLDER ; //we going to look for "folder" attribute
		dir = m_alternativepath;
		// "ParseDisplayName" uses UNICODE so BSTR will go 
		// We have not to forget to release it
		BSTR pData	=	dir.AllocSysString();
		if ( pData )
		{
			// The "m_alternativepath" always created without "\" but this was the way 
			// in original code , may be it was for a reason :) , so I am going to leave 
			// first check without addin "\" as well ...

			// So here we convert "display name" (path string) into ID list used by Shell
			pDesktop->ParseDisplayName(NULL,NULL,pData,NULL,&pList,&uAttrib);
			// Release the BSTR
			::SysFreeString(pData);
			// If list we got is null then no such object exists
			if ( !pList )
			{
				// Add "\" to get folder name
				dir	 +=  _T("\\");
				// allocate new search BSTR
				pData	=	dir.AllocSysString();
				if ( pData )
				{
					//we going to look for "folder" attribute
					uAttrib	=	SFGAO_FOLDER ;
					// we convert "display name" (path string) into ID list used by Shell
					pDesktop->ParseDisplayName(NULL,NULL,pData,NULL,&pList,&uAttrib);
					// Release the BSTR
					::SysFreeString(pData);
				}
			}
			// If we found the object and it's "folder" type
			if ( pList  && (uAttrib	& SFGAO_FOLDER) )
			{
				LPMALLOC ppMalloc;
				if(SHGetMalloc(&ppMalloc) == NOERROR)
				{
					ppMalloc->Free(pList);
					ppMalloc->Release();
				}

				for (POSITION pos = theApp.glob_prefs->shareddir_list.GetHeadPosition();pos != 0;theApp.glob_prefs->shareddir_list.GetNext(pos))
					if (theApp.glob_prefs->shareddir_list.GetAt(pos) == dir)
						return m_alternativepath;
				theApp.glob_prefs->shareddir_list.AddTail(dir);
				theApp.sharedfiles->Reload();
				return m_alternativepath;
			}
			// Just in case the object is not folder we have to release memory
			else if (pList)
			{
				LPMALLOC ppMalloc;
				if(SHGetMalloc(&ppMalloc) == NOERROR)
				{
					ppMalloc->Free(pList);
					ppMalloc->Release();
				}
			}
		}
		pDesktop->Release();
	}

	// default return 

	retpath.TrimRight(_T("\\"));
	return retpath;

	EMULE_CATCH

	return _T("");
}
// END - enkeyDEV(Ottavio84) -ChangeDir-

CString CPartFile::GetDownloadFileInfo()
{
	CString compl;
	compl.Format(_T("%s/%s"),CastItoXBytes(GetCompletedSize()),CastItoXBytes(GetFileSize()));

	CString lsc;
	if (lastseencomplete==NULL)
		lsc = GetResString(IDS_NEVER);
	else
		lsc = localelastseencomplete();	// netwolf 13.05.03 use local date/time format

	CString lastprogr;
	if (GetTransfered()==0 /*SyruS won't work: GetFileDate()==NULL*/)
		lastprogr = GetResString(IDS_NEVER);
	else
		lastprogr = localelastdowntransfer();	// netwolf 13.05.03 use local date/time format

	float availability = 0;
	if(GetPartCount() != 0) {
		availability = GetAvailablePartCount() * 100 / GetPartCount();
	}

	CString sRet;
	CString strHash = EncodeBase16(GetFileHash(), 16);
	
	sRet.Format(GetResString(IDS_DL_FILENAME)+_T(": %s (%s)\n\n%s\n\n")
		+GetResString(IDS_FD_HASH) +_T(" %s\n")
		+GetResString(IDS_PARTINFOS)+
		GetResString(IDS_PARTINFOS2)+_T("\n%s\n%s"),
		GetFileName(), CastItoXBytes(GetFileSize()),
				GetResString(IDS_STATUS)+_T(": ")+GetPartfileStatus(),
				strHash,
				GetPartMetFileName(), GetPartCount(),GetResString(IDS_AVAIL),GetAvailablePartCount(),availability,
				(int)GetPercentCompleted(), compl, GetTransferingSrcCount(),
				GetResString(IDS_LASTSEENCOMPL)+_T(" ")+lsc,
                GetResString(IDS_FD_LASTCHANGE) +_T(" ")+lastprogr);

	return sRet;
}

//BEGIN Improved Tooltips
CString CPartFile::GetDownloadFileInfo4Tooltips()
{
	CString compl;
	compl.Format(_T("%s/%s"),CastItoXBytes(GetCompletedSize()),CastItoXBytes(GetFileSize()));

	CString lsc;
	if (lastseencomplete==NULL)
		lsc = GetResString(IDS_NEVER);
	else
		lsc = localelastseencomplete();	// netwolf 13.05.03 use local date/time format

	CString lastprogr;
	if (GetTransfered()==0)
		lastprogr = GetResString(IDS_NEVER);
	else
		lastprogr = localelastdowntransfer();	// netwolf 13.05.03 use local date/time format

	float availability = 0;
	if(GetPartCount() != 0) {
		availability = GetAvailablePartCount() * 100 / GetPartCount();
	}
	
	CString sTransfSrcCount;
	if(GetTransferingSrcCount() > 0)
		sTransfSrcCount.Format(GetResString(IDS_TT_PARTINFOS3),GetTransferingSrcCount());
	
	CString strHash = EncodeBase16(GetFileHash(), 16);;
	CTime t(GetFileDate());

	CString sRet;
	sRet.Format(GetResString(IDS_TT_FILENAME)
			+GetResString(IDS_TT_PARTINFOS2)
			+GetResString(IDS_TT_STATUS)
//			+GetResString(IDS_TT_DATE)
			+GetResString(IDS_TT_HASH)
			+GetResString(IDS_TT_PARTINFOS)
			+GetResString(IDS_TT_PARTINFOS4)
			+GetResString(IDS_TT_LASTSEENCOMPL)
			+GetResString(IDS_TT_LASTCHANGE),
			GetFileName(),CastItoXBytes(GetFileSize()),GetFileSize(),GetResString(IDS_BYTES),
			(int)GetPercentCompleted(),	compl, sTransfSrcCount,
			GetPartfileStatus(),
//			t.Format("%A, %B %d, %Y"),
			strHash,
			GetPartMetFileName(),GetPartCount(),GetResString(IDS_AVAIL),GetAvailablePartCount(),availability,
			lsc,lastprogr);

	return sRet;
}
//END Improved Tooltips

// Ornis: Creating progressive presentation of the partfilestatuses - for webdisplay
CString CPartFile::GetProgressString(uint16 size){
	EMULE_TRY

	char crProgress = '0';//green
	char crHave = '1';	// black
	char crPending='2';	// yellow
	//Ju1i3n Start - merged with eMulePlus by janes bong
	char crMissing='3';  // red
	char crWaiting[6]; // differents blue
	crWaiting[0]='4'; // blue few source
	crWaiting[1]='5';
	crWaiting[2]='6';
	crWaiting[3]='7';
	crWaiting[4]='8';
	crWaiting[5]='9';
	//Ju1i3n End

	CString my_ChunkBar=_T("");
	for (uint16 i=0;i<=size+1;i++) my_ChunkBar.AppendChar(crHave);	// one more for safety

	uint32 m_nFileSize=GetFileSize();
	float unit= (float)size/(float)m_nFileSize;
	uint32 allgaps = 0;

	if(GetStatus() == PS_COMPLETE || GetStatus() == PS_COMPLETING) {
		CharFillRange(&my_ChunkBar,0,(float)m_nFileSize*unit, crProgress);
	} else

	// red gaps
	for (POSITION pos = gaplist.GetHeadPosition();pos !=  0;gaplist.GetNext(pos)){
		Gap_Struct* cur_gap = gaplist.GetAt(pos);
		allgaps += cur_gap->end - cur_gap->start;
		bool gapdone = false;
		uint32 gapstart = cur_gap->start;
		uint32 gapend = cur_gap->end;
		for (uint32 i = 0; i != GetPartCount(); i++){
			if (gapstart >= i*PARTSIZE && gapstart <=  (i+1)*PARTSIZE){ // is in this part?
				if (gapend <= (i+1)*PARTSIZE)
					gapdone = true;
				else{
					gapend = (i+1)*PARTSIZE; // and next part
				}
				// paint
				uint8 color;
				if (m_SrcpartFrequency.GetCount() >= (INT_PTR)i && m_SrcpartFrequency[i])  // frequency?
					color = m_SrcpartFrequency[i] <  10 ? crWaiting[m_SrcpartFrequency[i]/2]:crWaiting[5];	//Ju1i3n - merged with eMulePlus by janes bong
				else
					color = crMissing;

				CharFillRange(&my_ChunkBar,(float)gapstart*unit, (float)gapend*unit + 1,  color);

				if (gapdone) // finished?
					break;
				else{
					gapstart = gapend;
					gapend = cur_gap->end;
				}
			}
		}
	}

	// yellow pending parts
	for (POSITION pos = requestedblocks_list.GetHeadPosition();pos !=  0;requestedblocks_list.GetNext(pos))
	{
		Requested_Block_Struct* block =  requestedblocks_list.GetAt(pos);
		CharFillRange(&my_ChunkBar, (float)(block->StartOffset + block->transferred)*unit, (float)block->EndOffset*unit,  crPending);
	}

	return my_ChunkBar;

	EMULE_CATCH

	return _T("");
}

void CPartFile::CharFillRange(CString* buffer,float start, float end, char color) {
	EMULE_TRY
	for (uint32 i=start;i<=end;i++)
		buffer->SetAt(i,color);
	EMULE_CATCH
}

//vcf:- eklmn: A4AF management
void CPartFile::DownloadAllA4AF()
{
	EMULE_TRY

	//AddDebugLogLine(false, _T("Total %u A4AF sources before swap"), A4AF_srclist.GetCount());

	// Make a list of sources
	CArray<CUpDownClient*, CUpDownClient*> SourcesList;
	for (POSITION pos =  A4AF_srclist.GetHeadPosition(); pos != 0; A4AF_srclist.GetNext(pos)) {
		SourcesList.Add(A4AF_srclist.GetAt(pos));
	}

	// Actually handover
	for(int i = 0; i < SourcesList.GetCount(); i++)
		SourcesList[i]->SwapToAnotherFile(this);	

	EMULE_CATCH
}

//v- eklmn: download fix
void CPartFile::AddDownloadingSource(CUpDownClient* client){
	EMULE_TRY
	POSITION pos = m_downloadingSourceList.Find(client); // to be sure
	if (pos == NULL){
		m_downloadingSourceList.AddTail(client);
	}
	EMULE_CATCH
}

void CPartFile::RemoveDownloadingSource(CUpDownClient* client){
	EMULE_TRY
	POSITION pos = m_downloadingSourceList.Find(client); // to be sure
	if(pos != NULL){
		m_downloadingSourceList.RemoveAt(pos);
	}
	EMULE_CATCH
}
//^- eklmn: download fix

uint8* CPartFile::MMCreatePartStatus(){
	EMULE_TRY
	// create partstatus + info in mobilemule protocol specs
	// result needs to be deleted[] | slow, but not timecritical
	uint8* result = new uint8[GetPartCount()+1];
	for (int i = 0; i != GetPartCount(); i++){
		result[i] = 0;
		if (IsComplete(i*PARTSIZE,((i+1)*PARTSIZE)-1)){
			result[i] = 1;
			continue;
		}
		else{
			if (IsComplete(i*PARTSIZE + (0*(PARTSIZE/3)), ((i*PARTSIZE)+(1*(PARTSIZE/3)))-1))
				result[i] += 2;
			if (IsComplete(i*PARTSIZE+ (1*(PARTSIZE/3)), ((i*PARTSIZE)+(2*(PARTSIZE/3)))-1))
				result[i] += 4;
			if (IsComplete(i*PARTSIZE+ (2*(PARTSIZE/3)), ((i*PARTSIZE)+(3*(PARTSIZE/3)))-1))
				result[i] += 8;
			uint8 freq = (uint8)m_SrcpartFrequency[i];
			if (freq > 44)
				freq = 44;
			freq = ceilf((float)freq/3);
			freq <<= 4;
			result[i] += freq;
		}

	}
	return result;
	EMULE_CATCH
	return NULL;
};

// #zegzav:pastcomment (BEGIN)
void CPartFile::AddPastComment(CUpDownClient *pClient)
{
	EMULE_TRY
	if ((!pClient) || (pClient->reqfile != this))
		return;

	if ((pClient->GetFileComment().IsEmpty()) && (pClient->GetFileRate() == 0))
		return;

	m_PastComment.Add(pClient->GetUserHash(), pClient->GetUserName(), GetFileName(), pClient->GetFileComment(), pClient->GetFileRate());
	EMULE_CATCH
}

void CPartFile::RemovePastComment(CUpDownClient *pClient, bool bRestore)
{
	EMULE_TRY
	if (!pClient)
		return;

	if (bRestore)
	{
		CString strPastComment;
		int8 nPastRate;
		if ((m_PastComment.GetCommentRate(pClient->GetUserHash(), &strPastComment, &nPastRate)) &&
			(pClient->GetFileComment().IsEmpty()) && (pClient->GetFileRate() == 0))
		{
			// restore past comment
			pClient->SetFileComment(strPastComment);
			pClient->SetFileRate(nPastRate);
		}
	}

	m_PastComment.Remove(pClient->GetUserHash());
	EMULE_CATCH
}
// #zegzav:pastcomment (END)

int CPartFile::GetCategory()
{
	if(m_iCategory > theApp.glob_prefs->GetCatCount()-1)
		m_iCategory = 0;

	return m_iCategory;
}

void CPartFile::SetCategory(int iCategory)
{
	m_iCategory = iCategory;
	
	// set new prio
	if (IsPartFile())
	{
		switch (theApp.glob_prefs->GetCategory(GetCategory())->prio)
		{
		case 0:
			break;
		case 1:
			SetAutoPriority(false);
			SetPriority(PR_LOW);
			break;
		case 2:
			SetAutoPriority(false);
			SetPriority(PR_NORMAL);
			break;
		case 3:
			SetAutoPriority(false);
			SetPriority(PR_HIGH);
			break;
		case 4:
			SetAutoPriority(true);
			SetPriority(PR_HIGH);
			break;
		}

		SavePartFile();
	}
}

void CPartFile::SetStatus(uint8 in)
{
	status=in;

		if (theApp.emuledlg->transferwnd.m_wndDownloadList.m_nCurTab==0)
			theApp.emuledlg->transferwnd.m_wndDownloadList.ChangeCategory(0); 
		else
			UpdateDisplayedInfo(true);
		
		if (theApp.glob_prefs->ShowCatTabInfos())
			theApp.emuledlg->transferwnd.UpdateCatTabTitles();
}
