//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "serverlist.h"
#include "emule.h"
#include "HttpDownloadDlg.h"
#include "safefile.h"
#include "memcpy_amd.h"
#include <io.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CServerList::CServerList(CPreferences* in_prefs){
	servercount = 0;
	version = 0;
	serverpos = 0;
	searchserverpos = 0;
	statserverpos = 0;
	app_prefs = in_prefs;
	udp_timer = 0;
	delservercount = 0;
	m_nLastSaved = ::GetTickCount();
}

void CServerList::AutoUpdate(){
	theApp.glob_prefs->LoadServerlistAddresses() ; // Added by Tarod
	if (app_prefs->adresses_list.IsEmpty()){
		MessageBox(0,GetResString(IDS_ERR_EMPTYADRESSESDAT),GetResString(IDS_ERR_EMPTYADRESSESDAT_TITLE),64);
		return;
	}
	bool bDownloaded=false;
	CString servermet;
	CString oldservermet;
	CString strURLToDownload; 
	CSingleLock AccessLock(&m_ServerMetMutex,TRUE);
	servermet.Format(_T("%sserver.met"),app_prefs->GetConfigDir());
	oldservermet.Format(_T("%sserver_met.old"),app_prefs->GetConfigDir());
	_tremove(oldservermet);
	_trename(servermet,oldservermet);
	
	POSITION Pos = app_prefs->adresses_list.GetHeadPosition(); 
	while ((!bDownloaded) && (Pos != NULL)){
		CHttpDownloadDlg dlgDownload;
		strURLToDownload = app_prefs->adresses_list.GetNext(Pos); 
		dlgDownload.m_sURLToDownload = strURLToDownload.GetBuffer(); 
		dlgDownload.m_sFileToDownloadInto = servermet;
		if (dlgDownload.DoModal() == IDOK){
			_tremove(oldservermet);
			bDownloaded=true;
		}
		else{
			AddLogLine(true, IDS_ERR_FAILEDDOWNLOADMET, strURLToDownload.GetBuffer());
		}
	}
	if (!bDownloaded)
		_trename(oldservermet,servermet);	
}

bool CServerList::Init(){
	// auto update the list by using an url
	if (app_prefs->AutoServerlist())
		AutoUpdate();
	// Load Metfile
	CString strPath = app_prefs->GetConfigDir() + CString(_T("server.met"));
	CSingleLock AccessLock(&m_ServerMetMutex,TRUE);
	bool bRes = AddServermetToList(strPath, false);	// this deletes all existing servers
	// insert static servers from textfile
	strPath = app_prefs->GetConfigDir() + CString(_T("staticservers.dat"));
	AddServersFromTextFile(strPath);
	return bRes;
}

bool CServerList::AddServermetToList(const CString& strFile, bool merge){
	if (!merge)
	{
		theApp.emuledlg->serverwnd.serverlistctrl.DeleteAllItems();
		RemoveAllServers();
	}
	CSafeFile servermet;
	try{
		if (!servermet.Open(strFile,CFile::modeRead|CFile::osSequentialScan)){
			if(!merge)
				AddLogLine(false, IDS_ERR_LOADSERVERMET);
			return false;
		}
		servermet.Read(&version,1);
		if (version != 0xE0 && version != MET_HEADER){
			servermet.Close();
			AddLogLine(false, IDS_ERR_BADSERVERMETVERSION, version);
			return false;
		}
		theApp.emuledlg->serverwnd.serverlistctrl.Hide();
		uint32 fservercount;
		servermet.Read(&fservercount,4);
		
		ServerMet_Struct sbuffer;
		uint32 iAddCount = 0;
		for (uint32 j = 0;j != fservercount;j++){
			// get server
			servermet.Read(&sbuffer,sizeof(ServerMet_Struct));
			CServer* newserver = new CServer(&sbuffer);
			//add tags
			for (uint32 i=0;i != sbuffer.tagcount;i++)
				newserver->AddTagFromFile(servermet);
			// set listname for server
			if (!newserver->GetListName()){
				CString listname;
				listname.Format(_T("Server %s"),newserver->GetAddress());				
				newserver->SetListName(listname);				
			}
			if (!theApp.emuledlg->serverwnd.serverlistctrl.AddServer(newserver,true, true) ){
				CServer* update = theApp.serverlist->GetServerByAddress(newserver->GetAddress(), newserver->GetPort());
				if(update){
					update->SetListName( newserver->GetListName());
					if( newserver->GetDescription() )
						update->SetDescription( newserver->GetDescription());
					theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*update);
				}
				delete newserver;
			}
			else
				iAddCount++;
		}

		if (!merge)
			AddLogLine(true, IDS_SERVERSFOUND,fservercount);
		else
			AddLogLine(true, IDS_SERVERSADDED, iAddCount, fservercount-iAddCount);
		servermet.Close();
	}
	catch(CFileException* error){
		OUTPUT_DEBUG_TRACE();
		if (error->m_cause == CFileException::endOfFile)
			AddDebugLogLine(true,IDS_ERR_BADSERVERLIST);
		else{			
			AddDebugLogLine(true,IDS_ERR_FILEERROR_SERVERMET, GetErrorMessage(error));
		}
		error->Delete();	//memleak fix
	}
	theApp.emuledlg->serverwnd.serverlistctrl.Visable();
	theApp.emuledlg->serverwnd.serverlistctrl.ShowFilesCount();
	return true;
}

bool CServerList::AddServer(CServer* in_server)
{
	if (in_server == NULL) return false;	//eklmn: check NULL case at server addition
	
	if (theApp.glob_prefs->FilterBadIPs()) 
		if ( !IsGoodServerIP( in_server ))
			return false;
	CServer* test_server = FindServerByAddress(*in_server);
	if (test_server){
		test_server->ResetFailedCount();
		theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer( *test_server );		
		return false;
	}
	list.AddTail(in_server);
	return true;
}

void CServerList::ServerStats() {
#ifdef OLD_SOCKETS_ENABLED
	if( theApp.serverconnect->IsConnected() && list.GetCount() > 0 )
	{
		CServer* ping_server = GetNextStatServer();
		CServer* test = ping_server;
		if( !ping_server )
			return;
		while (ping_server->GetLastPingedTime() != 0 && ((uint32)time(NULL) - ping_server->GetLastPingedTime()) < UDPSERVSTATREASKTIME)
        	{ 
			ping_server = GetNextStatServer();
			if( ping_server == test )
				return;
		}
		if( ping_server->GetFailedCount() >= theApp.glob_prefs->GetDeadserverRetries() && 
			theApp.glob_prefs->DeadServer() &&
			!ping_server->IsStaticMember()){
			theApp.emuledlg->serverwnd.serverlistctrl.RemoveServer(ping_server);
			return;
		}
		Packet* packet = new Packet( OP_GLOBSERVSTATREQ, 4 );
		srand((unsigned)time(NULL));
		uint32 c = 0x55AA0000 + (uint16)rand();
		ping_server->SetChallenge(c);
		#ifndef AMD
		memcpy( packet->pBuffer, &c, 4 );
		#else
		memcpy_amd( packet->pBuffer, &c, 4 );
		#endif
		ping_server->SetLastPinged(::GetTickCount());
		ping_server->SetLastPingedTime((uint32)time(NULL));
		ping_server->AddFailedCount();
		theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*ping_server );		
		theApp.uploadqueue->AddUpDataOverheadServer(packet->size);
		theApp.serverconnect->SendUDPPacket( packet, ping_server, true );
		ping_server->SetLastDescPingedCount(false);
		if (ping_server->GetLastDescPingedCount() < 2)
		{
			packet = new Packet( OP_SERVER_DESC_REQ,0);
			theApp.uploadqueue->AddUpDataOverheadServer(packet->size);
			theApp.serverconnect->SendUDPPacket( packet, ping_server, true );
		}
		else
		{
			ping_server->SetLastDescPingedCount(true);
		}
	}
#endif //OLD_SOCKETS_ENABLED
}

/*filter Servers with invalid IP's / Port
0.*
10.*
172.16.0.0 - 172.31.255.255
192.168.0.0 - 192.168.255.255
127.*
*/
bool CServerList::IsGoodServerIP(CServer* in_server)
{
	if (!in_server->HasPublicAddress() && !in_server->HasDynIP())
		return false;

	// --- no4one : using filters from ipfilter.dat for server filtering ---
	// netwolf: option for filtering servers listed in ipfilter.dat and DynIP server
	if (theApp.glob_prefs->FilterServersByIP())	
		if (in_server->HasDynIP() || theApp.ipfilter->IsFiltered(in_server->GetIP()))
		{
			AddDebugLogLine(false,_T("Filtered Server: %s %s "),in_server->GetFullIP(),theApp.ipfilter->GetLastHit());

			if(!theApp.glob_prefs->GetAddServersFromConn()) {  //netwolf: remove filtered servers only if 'update from servers/clients' disabled (otherwise receive-remove loop)
				RemoveServer(in_server); }
			return false;
		}
		// --- no4one end ---

	return true; 
}

void CServerList::RemoveServer(CServer* out_server)
{
	POSITION remove_pos = list.Find(out_server);
	if (remove_pos)
	{
		delete list.GetAt(remove_pos);
		list.RemoveAt(remove_pos);
		delservercount++;	//statistic value
		// reset UDP server
		if (theApp.downloadqueue->cur_udpserver == out_server)
			theApp.downloadqueue->cur_udpserver = 0;
	}
}

void CServerList::RemoveAllServers()
{
	POSITION pos1,pos2;
	for ( pos1 = list.GetHeadPosition(); ( pos2 = pos1 ) != NULL;)
	{
		list.GetNext(pos1);
		delete list.GetAt(pos2);
		list.RemoveAt(pos2);
		delservercount++;	//statistic value
	}
}

void CServerList::GetStatus (uint32 &total, uint32 &failed, uint32 &user, uint32 &file, uint32 &tuser, uint32 &tfile,float &occ)
{
	total = list.GetCount();
	failed = 0;
	user = 0;
	file = 0;
	tuser = 0;
	tfile = 0;
	occ=0;
	uint32 maxusers = 0;
	uint32 tuserk = 0;

	CServer* curr;
	for (POSITION pos = list.GetHeadPosition(); pos !=0;list.GetNext(pos))
	{
		curr = (CServer*)list.GetAt(pos);
		if( curr->GetFailedCount() ){
			failed++;
		}
		else{
			user += curr->GetUsers();
			file += curr->GetFiles();
		}
		tuser += curr->GetUsers();
		tfile += curr->GetFiles();
		
		if (curr->GetMaxUsers()) {
			tuserk += curr->GetUsers(); // total users on servers with known maximum
			maxusers+=curr->GetMaxUsers();
		}
	}
	if (maxusers>0) occ=(float)(tuserk*100)/maxusers;
}

void CServerList::GetUserFileStatus(uint32 &user, uint32 &file){
	user = 0;
	file = 0;
	CServer* curr;
	for (POSITION pos = list.GetHeadPosition(); pos !=0; list.GetNext(pos))
	{
		curr = (CServer*)list.GetAt(pos);
		if( !curr->GetFailedCount() )
		{
			user += curr->GetUsers();
			file += curr->GetFiles();
		}
	}
}

CServerList::~CServerList(){
	SaveServermetToFile();
	POSITION pos1,pos2;
	for ( pos1 = list.GetHeadPosition(); ( pos2 = pos1 ) != NULL;)
	{
		list.GetNext(pos1);
		delete list.GetAt(pos2);
		list.RemoveAt(pos2);
	}
	if (udp_timer)
		KillTimer(0,udp_timer);
}

void CServerList::MoveServerDown(CServer* aServer){
   POSITION pos1, pos2;
   uint16 i = 0;
   for( pos1 = list.GetHeadPosition(); ( pos2 = pos1 ) != NULL; )
   {
	   list.GetNext(pos1);
	   CServer* cur_server = list.GetAt(pos2);
	   if (cur_server==aServer){
		   list.AddTail(cur_server);
		   list.RemoveAt(pos2);
		   return;
	   }
	   i++;
	   if (i == list.GetCount())
		   break;
   }
}

void CServerList::Sort(){
   POSITION pos1, pos2;
   uint16 i = 0;
   for( pos1 = list.GetHeadPosition(); ( pos2 = pos1 ) != NULL; )
   {
	   list.GetNext(pos1);
	   CServer* cur_server = list.GetAt(pos2);
	   if (cur_server->GetPreferences()== CServer::ePR_HIGH)
	   {
		   list.AddHead(cur_server);
		   list.RemoveAt(pos2);
	   }
	   else if (cur_server->GetPreferences() == CServer::ePR_LOW)
	   {
		   list.AddTail(cur_server);
		   list.RemoveAt(pos2);
	   }
	   i++;
	   if (i == list.GetCount())
		   break;
   }
}

CServer* CServerList::GetNextServer(){
	CServer* nextserver = NULL;
	uint32 i = 0;
	if (serverpos>=((uint32)list.GetCount()))
		return NULL;
	while (!nextserver && i != list.GetCount()){
		POSITION posIndex = list.FindIndex(serverpos);
		if (posIndex == NULL) {	// check if search position is still valid (could be corrupted by server delete operation)
			posIndex = list.GetHeadPosition();
			serverpos= 0;       //<<--9/27/02 zg
		}

		nextserver = list.GetAt(posIndex);
		serverpos++;
		i++;
		// TODO: Add more option to filter bad server like min ping, min users etc
		//if (nextserver->preferences = ?)
		//	nextserver = 0;
		if (serverpos == list.GetCount()) 	
			serverpos = 0;
	}
	return nextserver;
}

CServer* CServerList::GetNextSearchServer(){
	CServer* nextserver = NULL;
	uint32 i = 0;
	while (!nextserver && i != list.GetCount())
	{
		POSITION posIndex = list.FindIndex(searchserverpos);
		if (posIndex == NULL) {	// check if search position is still valid (could be corrupted by server delete operation)
			posIndex = list.GetHeadPosition();
			searchserverpos=0;
		}
		nextserver = list.GetAt(posIndex);
		searchserverpos++;
		i++;
		if (searchserverpos == list.GetCount())
			searchserverpos = 0;
	}
	return nextserver;
}

CServer* CServerList::GetNextStatServer(){
	CServer* nextserver = NULL;
	uint32 i = 0;
	while (!nextserver && i != list.GetCount())
	{
		POSITION posIndex = list.FindIndex(statserverpos);
		if (posIndex == NULL)	// check if search position is still valid (could be corrupted by server delete operation)
		{	
			posIndex = list.GetHeadPosition();
			statserverpos=0;
		}
		nextserver = list.GetAt(posIndex);
		statserverpos++;
		i++;
		if (statserverpos == list.GetCount())
			statserverpos = 0;
	}
	return nextserver;
}

bool CServerList::BroadCastPacket(Packet* packet){ // unused atm . but might be useful later
	if (udp_timer)
		return false;
	udp_timer = SetTimer(0,4322,UDPSEARCHSPEED,CServerList::UDPTimerProc);
	broadcastpos = list.GetHeadPosition();
	broadcastpacket = packet;
	return true;
}


void CServerList::SendNextPacket(){
#ifdef OLD_SOCKETS_ENABLED
	if (theApp.listensocket->TooManySockets()){
		KillTimer(0,udp_timer);
		udp_timer = 0;
		delete broadcastpacket;
		return;
	}
#endif //OLD_SOCKETS_ENABLED

	if (broadcastpos)
	{
		CServer* cur_server = list.GetAt(broadcastpos);
#ifdef OLD_SOCKETS_ENABLED
		if (cur_server != theApp.serverconnect->GetCurrentServer())
			theApp.serverconnect->SendUDPPacket(broadcastpacket,cur_server,false);
#endif //OLD_SOCKETS_ENABLED
		list.GetNext(broadcastpos);
	}
	else{
		KillTimer(0,udp_timer);
		udp_timer = 0;
		delete broadcastpacket;

	}
}

void CServerList::CancelUDPBroadcast(){
	if (udp_timer)
	{
		KillTimer(0,udp_timer);
		udp_timer = 0;
		delete broadcastpacket;
	}
}

void CALLBACK CServerList::UDPTimerProc(HWND hwnd, UINT uMsg,UINT_PTR idEvent,DWORD dwTime)
{
	EMULE_TRY
	theApp.serverlist->SendNextPacket();
	EMULE_CATCH
}

CServer* CServerList::GetNextServer(CServer* lastserver)
{
	POSITION pos;
	if (list.IsEmpty()) return 0;
	if (!lastserver) 
		return list.GetHead();
	else
		pos = list.Find(lastserver);
		if (!pos)
		{
			TRACE("Error: CServerList::GetNextServer");
			return list.GetHead();
		}
	list.GetNext(pos);
	if (!pos)
		return NULL;
	else
		return list.GetAt(pos);
}

CServer* CServerList::FindServerByAddress(const CServer& in_server){
	for (POSITION pos = list.GetHeadPosition();pos != 0;list.GetNext(pos))
	{
		CServer *s = list.GetAt(pos); // i_a: small speed optimization
		if (s->HasSameAddress(in_server)) 
			return s; 
	}
	return NULL;
}

CServer* CServerList::GetServerByAddress(const CString& address, uint16 port){
	for (POSITION pos = list.GetHeadPosition();pos != 0;list.GetNext(pos))
	{
		CServer *s = list.GetAt(pos); // i_a: small speed optimization
		if (( port == s->GetPort() || port==0) && s->GetAddress() == address) 
			return s; 
	}
	return NULL;
}

CServer* CServerList::GetServerByIP(uint32 nIP, uint16 port){
	for (POSITION pos = list.GetHeadPosition();pos != 0;){
        CServer* s = list.GetNext(pos);
		if (s->GetIP() == nIP && (port == 0 || s->GetPort() == port))
			return s; 
	}
	return NULL;
}

bool CServerList::SaveServermetToFile(){
	CSingleLock AccessLock(&m_ServerMetMutex,TRUE);
	CString newservermet;
	newservermet.Format(_T("%sserver.met.new"),app_prefs->GetConfigDir());
//	AddLogLine( false, "%s", newservermet );
	try{
	FILE* servermet = _tfopen(newservermet, _T("wb"));
	if (!servermet){
		AddLogLine(false, IDS_ERR_SAVESERVERMET);		
		return false;
	}
	version = 0xE0;

	fwrite( &version, 1, 1, servermet);
	uint32 fservercount = list.GetCount();
	fwrite(&fservercount,4,1,servermet);
	ServerMet_Struct sbuffer;
	CServer* nextserver;

	for (uint32 j = 0;j != fservercount;j++){
		nextserver = GetServerAt(j);
		sbuffer.ip = nextserver->GetIP();
		sbuffer.port = nextserver->GetPort();
		uint16 tagcount = 9;
		if (!nextserver->GetListName().IsEmpty())
			tagcount++;
		if (!nextserver->GetDynIP().IsEmpty())
			tagcount++;
		if (!nextserver->GetDescription().IsEmpty())
			tagcount++;
		if (!nextserver->GetVersion().IsEmpty())
			tagcount++;
		sbuffer.tagcount = tagcount;
		fwrite( &sbuffer, 1, sizeof(ServerMet_Struct),servermet);
		
		if( !nextserver->GetListName().IsEmpty() ){
			CTag servername( ST_SERVERNAME, nextserver->GetListName() );
			servername.WriteTagToFile(servermet);
		}
		if( !nextserver->GetDynIP().IsEmpty() ){
			CTag serverdynip( ST_DYNIP, nextserver->GetDynIP() );
			serverdynip.WriteTagToFile(servermet);
		}
		if( !nextserver->GetDescription().IsEmpty() ){
			CTag serverdesc( ST_DESCRIPTION, nextserver->GetDescription() );
			serverdesc.WriteTagToFile(servermet);
		}
		CTag serverfail(ST_FAIL, nextserver->GetFailedCount() );
		serverfail.WriteTagToFile(servermet);
		// Modified by Tarod [txh Hein] - save server preferences using eDonkey correct values
		CTag serverpref( ST_PREFERENCE, nextserver->eMule2ed2k(nextserver->GetPreferences()));
		serverpref.WriteTagToFile(servermet);
		CTag serveruser(_T("users"), nextserver->GetUsers() );
		serveruser.WriteTagToFile(servermet);
		CTag serverfiles(_T("files"), nextserver->GetFiles() );
		serverfiles.WriteTagToFile(servermet);
		CTag serverping(ST_PING, nextserver->GetPing() );
		serverping.WriteTagToFile(servermet);
		CTag serverlastp(ST_LASTPING, nextserver->GetLastPingedTime() );
		serverlastp.WriteTagToFile(servermet);
		CTag servermaxusers(ST_MAXUSERS, nextserver->GetMaxUsers() );
		servermaxusers.WriteTagToFile(servermet);
		CTag serversoftfiles(ST_SOFTFILES, nextserver->GetSoftFiles() );
		serversoftfiles.WriteTagToFile(servermet);
		CTag serverhardfiles(ST_HARDFILES, nextserver->GetHardFiles() );
		serverhardfiles.WriteTagToFile(servermet);
		if( !nextserver->GetVersion().IsEmpty() ){
			CTag serverversion( ST_VERSION, nextserver->GetVersion() );
			serverversion.WriteTagToFile(servermet);
		}
	}
	// v1:- eklmn: bugfix(19): by shutdown force OS flush data direct to disk [SlugFiller]
	if ( !theApp.emuledlg->IsRunning() ) _commit(_fileno(servermet));
	fclose(servermet);
	CString curservermet;
	CString oldservermet;
	curservermet.Format(_T("%sserver.met"),app_prefs->GetConfigDir());
	oldservermet.Format(_T("%sserver_met.old"),app_prefs->GetConfigDir());
	_tremove(oldservermet);
	_trename(curservermet,oldservermet);
	_trename(newservermet,curservermet);	
	}
	catch(CFileException* error) {
		OUTPUT_DEBUG_TRACE();
		AddLogLine(false, IDS_ERR_SAVESERVERMET2);		
		error->Delete();
		return false;
	}	
	m_nLastSaved = ::GetTickCount(); 
	return true;
}

void CServerList::AddServersFromTextFile(CString strFilename) {
	CString strLine;
	CStdioFile f;
	if (!f.Open(strFilename, CFile::modeRead | CFile::typeText))
		return;
	while(f.ReadString(strLine)) {
		// format is host:port,priority,Name
		if (strLine.GetLength() < 5)
			continue;
		if (strLine.GetAt(0) == '#' || strLine.GetAt(0) == '/')
			continue;

		// fetch host
		int pos = strLine.Find(':');
		if (pos == -1)
			continue;
		CString strHost = strLine.Left(pos);
		strLine = strLine.Mid(pos+1);

		// fetch  port
		pos = strLine.Find(',');
		if (pos == -1)
			continue;
		CString strPort = strLine.Left(pos);
		strLine = strLine.Mid(pos+1);

		// Barry - fetch priority
		pos = strLine.Find(',');
		int priority = PR_HIGH;
		if (pos == 1)
		{
			CString strPriority = strLine.Left(pos);
			try
			{
				priority = _tstoi(strPriority.GetBuffer(0));
				if ((priority < 0) || (priority > 2))
					priority = PR_HIGH;
			} catch (...) {}
			strLine = strLine.Mid(pos+1);
		}
	
		// fetch name
		CString strName = strLine;
		strName.Replace(_T("\r"), _T(""));
		strName.Replace(_T("\n"), _T(""));

		// create server object and add it to the list
		CServer* nsrv = new CServer(_tstoi(strPort), strHost.GetBuffer());
		nsrv->SetListName(strName.GetBuffer());
		nsrv->SetIsStaticMember(true);
		// Barry - Was always high
		nsrv->SetPreference((CServer::eServerPriority)priority); 
		if (!theApp.emuledlg->serverwnd.serverlistctrl.AddServer(nsrv, true))
		{
			delete nsrv;
			CServer* srvexisting = GetServerByAddress(strHost.GetBuffer(), _tstoi(strPort));
			if (srvexisting) {
				srvexisting->SetListName(strName.GetBuffer());
				srvexisting->SetIsStaticMember(true);
				// Barry - Was always high
				srvexisting->SetPreference((CServer::eServerPriority)priority); 
				if (theApp.emuledlg->serverwnd) theApp.emuledlg->serverwnd.serverlistctrl.RefreshServer(*srvexisting);
			}

		}
	}

	f.Close();
}
void CServerList::Process()
{
	if (::GetTickCount() - m_nLastSaved > 180000)
		this->SaveServermetToFile();
}
