// Tasks.cpp: implementation of the CTasks class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#ifdef NEW_SOCKETS

#include "Tasks.h"
#include "EmEngine.h"
#include "../emule.h"
#include "../EngineData.h"
#include "../server.h"
#include "TaskProcessor_Main.h"

CTask::~CTask()
{
}

bool CTask::OnException(CEmEngine&)
{
	// delete this buggy task
	return true;
}

void CTask::ProcessForUI(CEmEngine&)
{
}

bool CTask_Tcp_Err::Post(CEmEngine& stEngine, SOCKET hSocket, int nErrorCode)
{
	CTask_Tcp_Err* pTask = new CTask_Tcp_Err;
	if (pTask)
	{
		pTask->m_hSocket = hSocket;
		pTask->m_nError = nErrorCode;
		stEngine.m_stMain.Push(pTask);
	} 
	else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "no memory");
	}

	return NULL != pTask;
}

bool CTask_Tcp_Err::Process(CEmEngine& stEngine)
{
	// check if we still have this client
	CEmClient* pClient = stEngine.m_stMain.Lookup(m_hSocket);
	if (pClient)
	{
		ASSERT(pClient->m_hSocket == m_hSocket);
		// here we can write to the log that this client has disconnected from us

		stEngine.m_stMain.KillClient(pClient);
	}

	return true;
}

bool CTask_Tcp_Accepted::Process(CEmEngine& stEngine)
{
	CEmClient* pClient = stEngine.m_stMain.AllocClient(m_eType);
	ASSERT(pClient);
	if (pClient)
	{
		pClient->m_hSocket = m_hSocket;
		pClient->m_bIsConnected = true;
		stEngine.m_stMain.m_mapClients.SetAt(m_hSocket, pClient);

		pClient->OnAccepted(stEngine);

	} else
	{
		ASSERT(FALSE);
		AddDebugLogLine(false, "no memory");
		if (closesocket(m_hSocket))
			AddDebugLogLine(false, "closesocket failed");
	}

	return true;
}

bool CTask_Tcp_Connected::Process(CEmEngine& stEngine)
{
	CEmClient* pClient = stEngine.m_stMain.Lookup(m_hSocket);
	if (pClient)
	{
		ASSERT(!pClient->m_bIsConnected);
		pClient->m_bIsConnected = true;

		pClient->OnConnected(stEngine);
	}
	return true;
}

bool CTask_Tcp_Web::Process(CEmEngine& stEngine)
{
	CEmClient_Web* pClient = (CEmClient_Web*) stEngine.m_stMain.Lookup(m_hSocket);
	if (pClient)
	{
		ASSERT(T_CLIENT_WEB == pClient->GetType());

		  CString strTxt;
		  strTxt.Format("Currently I have %d clients", stEngine.m_stMain.m_mapClients.GetCount());
		  stEngine.m_stTcp.AllocSend(m_hSocket, (LPCTSTR) strTxt, strTxt.GetLength());
		  stEngine.m_stTcp.AllocDisconnect(m_hSocket);
	}

	return true;
}

CTask_Connect::CTask_Connect(ULONG nAddr, USHORT nPort, T_CLIENT_TYPE eType)
	: m_nAddr(nAddr), m_nPort(nPort), m_eType(eType)
{
}

CTask_Connect::CTask_Connect(LPCTSTR sAddr, USHORT nPort, T_CLIENT_TYPE eType) 
	: m_nAddr(inet_addr(sAddr)), m_nPort(nPort), m_eType(eType)
{
}

CTask_Connect::CTask_Connect(CServer* pServer /* = NULL*/)
{
	EMULE_TRY

	if(pServer)
	{
		AddLogLine(false, IDS_CONNECTINGTO, pServer->GetListName(), pServer->GetAddress(), pServer->GetPort());

		m_nAddr = inet_addr(pServer->GetAddress());
		m_nPort = pServer->GetPort();
	}
	else
		m_nAddr = m_nPort = NULL;

	m_eType = T_CLIENT_SERVER;

	EMULE_CATCH2
}

bool CTask_Connect::Process(CEmEngine& stEngine)
{
	if(m_nAddr && m_nPort)
		stEngine.m_stMain.AllocTcpConnect(m_nAddr, m_nPort, m_eType);
	else
	{
		// Connect to any server
	}

	return true;
}

CTask_KillClient::CTask_KillClient(CEmClient* pClient)
	: m_pClient(pClient)
{
}

bool CTask_KillClient::Process(CEmEngine& stEngine)
{
	stEngine.m_stMain.KillClient(m_pClient);

	return true;
}

CTask_ServerState::CTask_ServerState(long nConnectionState, long nClientID, CServer* pServer, CServer* pNewServer)
	: m_nConnectionState(nConnectionState), m_nClientID(nClientID), m_pCurrentServer(pServer), m_pNewServer(pNewServer)
{
}

bool CTask_ServerState::Process(CEmEngine& stEngine)
{
	stEngine.PushToUI(this);

	return false;
}

void CTask_ServerState::ProcessForUI(CEmEngine&)
{
	if(theApp.emuledlg->m_pEngineData)
		theApp.emuledlg->m_pEngineData->ImportServerState(this);
}

#endif