//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "zlib/zlib.h"
#include "UpDownClient.h"
#include "opcodes.h"
#include "packets.h"
#include "emule.h"
#include "uploadqueue.h"
#include "otherstructs.h"
#include "memcpy_amd.h"
#include "math.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//	members of CUpDownClient
//	which are mainly used for uploading functions 

void CUpDownClient::DrawUpStatusBar(CDC* dc, RECT* rect, bool onlygreyrect, bool  bFlat){ 
	EMULE_TRY

	const COLORREF crBoth=(bFlat) ? RGB(0, 150, 0) : RGB(0, 192, 0);	//Cax2 - same green used for files & clients 
	const COLORREF crNeither=(bFlat) ? RGB(224, 224, 224) : RGB(240, 240, 240); 
    const COLORREF crBeingSent = RGB(255,208,0);

	uint32 filesize =(m_nUpPartCount) ? PARTSIZE*(m_nUpPartCount):1; // might be a little off.. better than nothing

	CBarShader statusBar(rect->bottom - rect->top, rect->right - rect->left); //Maella - Not a static object anymore
	statusBar.SetFileSize(filesize); 
	statusBar.Fill(crNeither); 

	if (filesize !=1 && !onlygreyrect && m_abyUpPartStatus)	//Cax2 - no more black rectangle
	{ 
		for (uint32 i = 0;i != m_nUpPartCount;i++)
		{ 
            /* no need for that now, cause we don't use the precise filesize
			if (PARTSIZE*(i+1) > filesize) 
				uEnd = filesize; 
			else 
				uEnd = PARTSIZE*(i+1); 
            */
		    uint32 uEnd = PARTSIZE*(i+1);
			
			if (m_abyUpPartStatus[i]){ //they have this part
                statusBar.FillRange(PARTSIZE*i, uEnd, crBoth); // will do for now
			}
			else //they don't have a part.
                statusBar.FillRange(PARTSIZE*i, uEnd, crNeither);
                uint32 partno = GetCurrentlyUploadingPart();
                if(i==partno) // won't be equal if it is 0xFFFF
                    statusBar.FillRange(PARTSIZE*i,  uEnd, crBeingSent);
		} 
   	} 
   	statusBar.Draw(dc, rect->left, rect->top, bFlat); 

	EMULE_CATCH
} 

// [i_a]
bool CUpDownClient::IsCommunity()        
{
	EMULE_TRY

	if (!community_cached)
	{
		// [i_a] DonGato (Community Sharing)
		//eklmn: to prevent a leeching the community string must have a same format
		CString username = m_pszUsername;
		CString community = theApp.glob_prefs->CommunityString();
		CString communityok;
		
		//community.Remove('['); community.Remove(']');
		communityok.Format("[%s]",community);
		iscommunity = (username.Find(communityok) >= 0);
		community_cached = true;

		communityok.Empty();
		community.Empty();
		username.Empty();
	}
	return iscommunity;
	
	EMULE_CATCH
	return false;
}

void CUpDownClient::ResetValueCache()
{
	EMULE_TRY
    score_base_cached = false;
    community_cached = false;
	EMULE_CATCH
}
// [/i_a]

uint32 CUpDownClient::GetScore(bool sysvalue, bool isdownloading, bool onlybasevalue)
{
	EMULE_TRY
	//TODO: complete this (friends, uploadspeed, emuleuser etc etc)
	if (m_pszUsername.IsEmpty()) return 0;

	//SyruS moved this up (from end) to skip the whole useless calculation
	//Start milobac: Anti-leechermods
	if (m_bBanned) 
		return 0;
	else	
		if (IsStolenName() || IsLeecherType() /*&& theApp.glob_prefs->GetMlDonkeyCM()*/)
		{
			if (!m_modVerString.IsEmpty())
				AddDebugLogLine(false,_T("Anti-leechermods: Client %s (%s) using the mod %s has been banned"),GetUserName(),GetClientNameAndVersionString(this),GetModVersion());
			else 
				AddDebugLogLine(false,_T("Anti-leechermods: Client %s (%s) has been banned"),GetUserName(),GetClientNameAndVersionString(this));
			//katsyonak: Send leecher ban message 15.06.03
			Ban(_T("AUTOMATED MESSAGE: You're being banned because your client is leeching!"), GetResString(IDS_BAN_LEECHERADV));
			return 0;
		}
	//End milobac: Anti-leechermods

#ifdef OLD_SOCKETS_ENABLED
	if (!isdownloading && !onlybasevalue)
	{
		if (HasLowID() && !(socket && socket->IsConnected()))
	{ 
			if (!theApp.serverconnect->IsConnected() || theApp.serverconnect->IsLowID())
				return 0;
		}
	}
#endif //OLD_SOCKETS_ENABLED

	CKnownFile* req_file = theApp.sharedfiles->GetFileByID(reqfileid);
	if (req_file == NULL) return 0;

		// TODO coded by tecxx & herbert, one yet unsolved problem here:
		// sometimes a client asks for 2 files and there is no way to decide, which file the 
		// client finally gets. so it could happen that he is queued first because of a 
		// high prio file, but then asks for something completely different.
		
		// isfriend = 	theApp.emuledlg->chatwnd.IsFriend(CString(m_pszUsername));
		// isemuleclient = false; // i_a: unused

	//eklmn: we cann't cache a value here, cause otherwise our chages of priority will 
	//           be not applied until we restart eMule.
        ASSERT(reqfileid != NULL); // i_a: reqfileid is an array so the if() was superfluous 
        { 
		switch(req_file->GetPriority())
		{ 
			case PR_VERYHIGH: // Modified by Tarod for UAP
				if (req_file->IsPartFile())
					filepriority = 50;
				else
					filepriority = 100;
				break;
					
			case PR_HIGH:
				filepriority = 20;
				break;
					
			case PR_LOW:
				filepriority = 5;
				break;

			case PR_VERYLOW:
				filepriority = 2;
				break;
					
			case PR_NORMAL:
			default:
				filepriority = 10;	// standard
				break;
		} 
	}

	// calculate score, based on waitingtime and other factors
	float fBaseValue;
	if (onlybasevalue)
		fBaseValue = 100.0f;
	else if (!isdownloading)
		fBaseValue = (float)(GetTickCount()-m_dwWaitTime)/1000.0f;
	else
	{
		// we dont want one client to download forever
		// the first 15 min downloadtime counts as 15 min waitingtime and you get a 15 min bonus while you are in the first 15 min :)
		// (to avoid 20 sec downloads) after this the score won't raise anymore 
		fBaseValue = (float)(m_dwUploadTime-m_dwWaitTime);
		ASSERT ( m_dwUploadTime-m_dwWaitTime >= 0 ); //oct 28, 02: changed this from "> 0" to ">= 0"
		fBaseValue += (float)(GetTickCount() - m_dwUploadTime > 900000)? 900000.0f:1800000.0f;
		fBaseValue /= 1000.0f;
	}	
	if (GetDownloadState() == DS_DOWNLOADING && GetDatarate() > 500) // obaldin: prevent 0.1 upload abuse
		fBaseValue *= 1.5f; // since up/down credits are in, this isn't that improtant anymore
	if (IsFriend())
		fBaseValue *= 2.0f;
	// DonGato (Community Sharing)
	if (theApp.glob_prefs->CommunityEnabled() && IsCommunity())		
		fBaseValue *= 2.0f;
	
	//Fix by Amdribant [loadedsrc]
	//ASSERT(credits != NULL); 
	if(credits != (CClientCredits*)NULL)
	{ 
		fBaseValue *= credits->GetScoreRatio(this->GetIP());
	} 

	// Added by Tarod (herbert code)
	fBaseValue *= GetSmallFilePushRatio();
	fBaseValue *= GetRareFilePushRatio();

	fBaseValue *= ((float)(filepriority))/10.0f; 

	return (uint32)fBaseValue;

	EMULE_CATCH
	return 1;
}

// Added by Tarod based in herbert code and modifications by MatzeHH
float CUpDownClient::GetRareFilePushRatio()
{
	EMULE_TRY
	//if(!theApp.glob_prefs->FairShareRarePush())
	//	return 1.0f;
	CKnownFile* srcfile = theApp.sharedfiles->GetFileByID((uchar*)reqfileid);
	if (srcfile == NULL)
		return 1.0f;
	
	// keep the FileRatio
	float ratio = 0+srcfile->GetFileRatio();
	return (ratio < 1.0f ? 1.0f :((ratio>100.0f)?100.0f: ratio)); //Cax2 bugfix

	EMULE_CATCH
	return 1.0f;
}

// Checks if it is next requested block from another chunk of the actual file or from another file 
// 
// [Returns] 
//   true : Next requested block is from another different chunk or file than last downloaded block 
//   false: Next requested block is from same chunk that last downloaded block 
bool CUpDownClient::IsDifferentPartBlock(bool silent) // [Tarod 12/22/2002] Cax2 - added silent option
{ 
	Requested_Block_Struct* last_done_block;
	Requested_Block_Struct* next_requested_block;
	uint32 last_done_part = 0xffffffff;
	uint32 next_requested_part = 0xffffffff;
	
	bool different_part = false;
	
	try {
		// Check if we have good lists and proceed to check for different chunks
		if (!m_BlockRequests_queue.IsEmpty() && !m_DoneBlocks_list.IsEmpty())
		{
			// Get last block and next pending
			last_done_block = (Requested_Block_Struct*)m_DoneBlocks_list.GetHead();
			next_requested_block = (Requested_Block_Struct*)m_BlockRequests_queue.GetHead(); 
			
			// Calculate corresponding parts to blocks
			last_done_part = last_done_block->StartOffset / PARTSIZE;
			next_requested_part = next_requested_block->StartOffset / PARTSIZE; 
             
			// Test is we are asking same file and same part
			/*
			if ( last_done_part != next_requested_part)
			{ 
				different_part = true;
				//AddDebugLogLine(false, "Session for user %s (%s) ended due to new chunk, sent %u KB", GetUserName(), GetClientNameAndVersionString(this), GetSessionUp() / 1024); 
			}
			if (md4cmp (last_done_block->FileID, next_requested_block->FileID) != 0)
			{ 
				different_part = true;
                //AddDebugLogLine(false, "Session for user %s(%s) ended due to different file, sent %u KB", GetUserName(), GetClientNameAndVersionString(this), GetSessionUp() / 1024); 
			}
			*/
			if ( last_done_part != next_requested_part || md4cmp(last_done_block->FileID, next_requested_block->FileID) != 0)
				different_part = true;
		} 
   	}
   	catch(...)
   	{ 
      		different_part = true; 
   	}

	if(different_part) {
		//AddDebugLogLine(false, "Debug: User %s, last_done_part (%u) %s (%u) next_requested_part, sent %u KB", GetUserName(), last_done_part, different_part? "!=": "==", next_requested_part, this->GetTransferedUp() / 1024); 

		// obaldin: this is for old clients, that rotate the chunk,
		// so that they'll be able to download at least _something_
		// But probably even after the majority switches we'd
		// better check for 0 here
		if(GetSessionUp() < 1024*1024) {
			different_part = false;
			if (!silent)
				AddDebugLogLine(false, _T("Client %s continues downloading (should be back on queue) because of a low transferred size"),GetUserName()); 
		}
	}

	return different_part; 
}

#if JUMPSTART
bool CUpDownClient::IsForbiddenByJumpstart()
{ 
	EMULE_TRY
		
	Requested_Block_Struct* next_requested_block;
	uint32 next_requested_part = 0xffffffff;

	if(m_BlockRequests_queue.IsEmpty()) {
		return false;
	} 
	else {
		next_requested_block = (Requested_Block_Struct*)m_BlockRequests_queue.GetHead();
		next_requested_part = next_requested_block->StartOffset / PARTSIZE;
		CKnownFile* kf = theApp.sharedfiles->GetFileByID(next_requested_block->FileID);
		if ((kf != NULL) && !kf->AllowChunkForClient(next_requested_part, this)) {
			return true;
		}
	}

	EMULE_CATCH

	return false;
}
#endif // JUMPSTART


float CUpDownClient::GetSmallFilePushRatio(){
	EMULE_TRY
	//if(!theApp.glob_prefs->FairShareRarePush())
	//	return 1.0f;
	CKnownFile* srcfile = theApp.sharedfiles->GetFileByID((uchar*)reqfileid);
	if (srcfile == NULL)
		return 1.0f;

	//Cax2 faster code ....
	long filesize = srcfile->GetFileSize() ;

	//if (filesizeinmb > 0.0f && filesizeinmb < 9.5f) {
	if (filesize > 0.0f && filesize < 9961472) {

		// Modded by Tarod to: Min(100, Max(1, 9.5 / FS)) [9.5 Mbs. == Chunk Size]
		float mul = 9961472/ filesize ;		// VQB formula + Cax2 overflow bugfix...
		if (mul > 100.0f) mul = 100.0f ;		// VQB formula 
		return mul ;
	}

	EMULE_CATCH
	return 1.0f;
}
// End added by Tarod

//SyruS (0.29c)
class CSyncHelper
{
public:

	CSyncHelper()
	{
		m_pObject = NULL;
	}

	~CSyncHelper()
	{
		if (m_pObject)
			m_pObject->Unlock();
	}

	CSyncObject* m_pObject;
};

bool CUpDownClient::CreateNextBlockPackage(){
	// check if we should kick this client (time critical !!!)
	// Never kick friends
	// Always kick if transfer is longer than 80 minutes
	// Never allow more than the size of a chunk (+500 KBytes for small resends of invalid data) to be transfered
	// VQB to provide full chunk transfers (modified by Tarod)
	// Note: Full chunk transfers really should be enforced, not an option
	EMULE_TRY
	// LANCAST (moosetea) We never kick a lancast user
	bool AllowedMore = isOnLAN();	
	// v- eklmn: bugfix(04): conflict between upload logic & LANcast
	bool diff_part = IsDifferentPartBlock(true);	//Cax2 - don't write anyting to the log yet!
#if JUMPSTART
	bool JS_blocked = IsForbiddenByJumpstart();
#endif
	bool shouldGo=((GetUpStartTimeDelay()>= 80*60*1000) || diff_part);

	// if user is from Internet && full chunk (or timeout)
	if (!AllowedMore && shouldGo)
	{
#if JUMPSTART
		if (IsFriend() && !theApp.uploadqueue->CheckForTimeOver(this) && !JS_blocked)
#else
		if (IsFriend() && !theApp.uploadqueue->CheckForTimeOver(this))
#endif
		{
			AllowedMore = true;
			AddDebugLogLine(false, _T("Client %s downloaded %u KB so far. Allowed to continue downloading (should go back on queue) because it's a friend."), GetUserName(), this->GetSessionUp() / 1024); 
		}
		else 
		{
			SetWaitStartTime();
			if (diff_part)
			{
				theApp.uploadqueue->RemoveFromUploadQueue(this,ETS_BLOCKED_CHUNK);
				AddDebugLogLine(false, _T("Client %s (%s)  back on queue. Asked for a new chunk/file, after receiving %u KB."), GetUserName(), GetClientNameAndVersionString(this), GetSessionUp() / 1024);
			}
#if JUMPSTART
			else if (JS_blocked)
			{
				theApp.uploadqueue->RemoveFromUploadQueue(this,ETS_BLOCKED_CHUNK);
				AddDebugLogLine(false, _T("Client %s (%s)  back on queue. Asked for a chunk forbidden by Jumpstart, after receiving %u KB."), GetUserName(), GetClientNameAndVersionString(this), GetSessionUp() / 1024);
			}
#endif
			else
			{
				theApp.uploadqueue->RemoveFromUploadQueue(this,ETS_TIMEOUT);
				AddDebugLogLine(false, _T("Client %s (%s)  back on queue. Upload timeout, after receiving %u KB"), GetUserName(), GetClientNameAndVersionString(this), GetSessionUp() / 1024);
			}
			theApp.uploadqueue->AddClientToQueue(this,true);
			return false;
		}
	}
	// ^- eklmn: bugfix(04): conflict between upload logic & LANcast
	
	if (m_BlockRequests_queue.IsEmpty()) {
		// There are no new blocks requested, put we will happily transfer more
		// if the client just requests something
		// AddDebugLogLine(false, "No more data to send to client %s(%s), should be back on queue", GetUserName(), GetClientNameAndVersionString(this)); 
		return false; //eklmn: bugfix(03): keeping user in queue if he doesn't need anything
	}

	CFile file;
	CKnownFile* srcfile;
	byte* filedata = 0;
	bool bFromPF = true;
	CString fullname;
	CSyncHelper lockFile;
	try
	{
		// repeat as long as next requested block is in the same chunk as previous blocks
		// and as long as there is a next requested block
		//added: if a friend or lancast user, keep going![Cax2]
#if JUMPSTART
		while (!m_BlockRequests_queue.IsEmpty() && (AllowedMore || !IsDifferentPartBlock()) && !IsForbiddenByJumpstart() )
#else
		while (!m_BlockRequests_queue.IsEmpty() && (AllowedMore || !IsDifferentPartBlock()) )		//Cax2 - if a friend or lancast user, keep going!
#endif
		{
			Requested_Block_Struct* currentblock = m_BlockRequests_queue.GetHead();
			srcfile = theApp.sharedfiles->GetFileByID(currentblock->FileID);
			if (!srcfile)
			{
				theApp.uploadqueue->RemoveFromUploadQueue(this,ETS_FILE_ERROR);	// eklmn: ???
				throw GetResString(IDS_ERR_REQ_FNF);
			}

			if (srcfile->IsPartFile() && ((CPartFile*)srcfile)->GetStatus() != PS_COMPLETE)
			{
				// SyruS (0.29c) Do not access a part file, if it is currently moved into the incoming directory.
				// Because the moving of part file into the incoming directory may take a noticable 
				// amount of time, we can not wait for 'm_FileCompleteMutex' and block the main thread.
				if (!((CPartFile*)srcfile)->m_FileCompleteMutex.Lock(0)) // just do a quick test of the mutex's state and return if it's locked.
					return false;
				lockFile.m_pObject = &((CPartFile*)srcfile)->m_FileCompleteMutex;
				// If it's a part file which we are uploading the file remains locked until we've read the
				// current block. This way the file completion thread can not (try to) "move" the file into
				// the incoming directory.
				fullname = ((CPartFile *)srcfile)->GetFullName();
				fullname.Truncate(fullname.GetLength() - 4);			
			}
			else
			{				
				fullname.Format(_T("%s\\%s"),srcfile->GetPath(),srcfile->GetFileName());
			}

			//v- eklmn: bugfix(18): conflict between JUMSTART & PartTraffic)
#if JUMPSTART
			if (!srcfile->GetJumpstartEnabled())
#else
			if (true) 
#endif
			{
				//--- xrmb:partprio ---
				//--- accessing a blocked part??? ---
				uint8 part_status = srcfile->GetPartStatus(currentblock->StartOffset / PARTSIZE);
				CString s;
				switch (part_status) {
					case PR_PART_OFF:
						AddLogLine(false, IDS_PARTPRIO_DENIED, currentblock->StartOffset / PARTSIZE, srcfile->GetFileName());
						s.Format(GetResString(IDS_PARTPRIO_DENIED), currentblock->StartOffset / PARTSIZE, srcfile->GetFileName());
						theApp.uploadqueue->RemoveFromUploadQueue(this,ETS_BLOCKED_CHUNK);
						throw s;
						break;
					case PR_PART_HIDDEN:
						AddLogLine(false, IDS_PARTPRIO_HIDDEN, GetUserName(), srcfile->GetFileName(), currentblock->StartOffset / PARTSIZE);
				}
				//--- :xrmb ---
			}
			//^- eklmn: bugfix(18): conflict between JUMSTART & PartTraffic)
			
			uint32 togo;
			if (AllowedMore) {	// Friends & LAN  are not limited by chunks boundaries.
				// Is this a wraparound request?
				if (currentblock->StartOffset > currentblock->EndOffset-1){
					// If StartOffset is later in the file than EndOffset, it means
					// that the request is a wraparound request. It reads to the end of
					// the file, then it wraps around and continues in the beginning of the file.

					// Calculate how much the client is requesting, knowing that this is a wraparound request
					togo = currentblock->EndOffset + (srcfile->GetFileSize() - currentblock->StartOffset);
				} 
				else {
					// not a wraparound request. This makes it easier to calculate how many bytes is
					// being requested.
					togo = currentblock->EndOffset - currentblock->StartOffset;
				}
			} 
			else {
				// Everyone else are limited to a single chunk
				// Check that StartOffset and EndOffset is in the same chunk
				if(currentblock->StartOffset/PARTSIZE != (currentblock->EndOffset - 1)/PARTSIZE) {
					// The EndOffset goes into the next chunk. Set it to the end of the chunk
					// that StartOffset is in.
					currentblock->EndOffset = ((uint32)(currentblock->StartOffset/PARTSIZE)+1)*PARTSIZE;
				}

				// Make sure we don't pass end of file
				if (currentblock->StartOffset > currentblock->EndOffset) {
					currentblock->EndOffset = srcfile->GetFileSize();
				}

				// This can't be a wrapped around request, since it has been limited to
				// a single chunk.
				togo = currentblock->EndOffset - currentblock->StartOffset;
			}

			if (srcfile->IsPartFile()) { // proper check
				uint32 parts_start = ((currentblock->StartOffset)/PARTSIZE)*PARTSIZE;
				uint32 parts_end = ((currentblock->EndOffset-1)/PARTSIZE + 1)*PARTSIZE -1;
				if(!((CPartFile*)srcfile)->IsComplete(parts_start,parts_end))
				{
					theApp.uploadqueue->RemoveFromUploadQueue(this,ETS_BLOCKED_CHUNK);
					throw GetResString(IDS_ERR_INCOMPLETEBLOCK);
				}
			}
			
			if (!srcfile->IsPartFile())
			{
				bFromPF = false;	// <-khaos--+++> This is not a part file...
				if (!file.Open(fullname,CFile::modeRead|CFile::osSequentialScan|CFile::shareDenyNone))
				{
					theApp.uploadqueue->RemoveFromUploadQueue(this,ETS_FILE_ERROR);
					throw GetResString(IDS_ERR_OPEN);				
				}
				file.Seek(currentblock->StartOffset,0);
				
				filedata = new byte[togo+500];
				if (uint32 done = file.Read(filedata,togo) != togo){
					file.SeekToBegin();
					file.Read(filedata + done,togo-done);
				}
				file.Close();
			}
			else
			{
				CPartFile* partfile = (CPartFile*)srcfile;				
				partfile->m_hpartfile.Seek(currentblock->StartOffset,0);
				
				filedata = new byte[togo+500];
				if (uint32 done = partfile->m_hpartfile.Read(filedata,togo) != togo){
					partfile->m_hpartfile.SeekToBegin();
					partfile->m_hpartfile.Read(filedata + done,togo-done);
				}
			}
			if (lockFile.m_pObject)
				lockFile.m_pObject->Unlock(); //SyruS (0.29c) Unlock the (part) file as soon as we are done with accessing it.

			#ifndef AMD
			md4cpy(reqfileid,currentblock->FileID);
			#else
			memcpy_amd(reqfileid,currentblock->FileID,16);
			#endif
			CString ext = srcfile->GetFileName().Right(4);
			// -khaos--+++> We're going to add bFromPF as a parameter to the calls to create packets...
			if (m_byDataCompVer == 1 && ext != _T(".zip") && ext != _T(".rar") && ext != _T(".ace"))
				CreatePackedPackets(filedata,togo,currentblock,bFromPF);
			else
				CreateStandartPackets(filedata,togo,currentblock,bFromPF);
			// <-----khaos-
			
			// file statistic
			srcfile->statistic.AddTransferd(togo);

			//--- xrmb:parttraffic ---
			srcfile->statistic.AddTraffic(currentblock->StartOffset, togo);
			//--- :xrmb ---

#if JUMPSTART
			if (!srcfile->IsPartFile()){
				if (srcfile->GetJumpstartEnabled()){
					srcfile->AddSentBlock(this, currentblock->StartOffset, togo);
				}
			}
#endif // JUMPSTART

			m_DoneBlocks_list.AddHead(m_BlockRequests_queue.RemoveHead());
			delete[] filedata;
			filedata = 0;
		}
	}
	catch(CString &error){
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(false,IDS_ERR_CLIENTERRORED,GetUserName(),error.GetBuffer());
		if (filedata)
			delete filedata;
		return false;
	}
	//v- eklmn: bugfix(13): memleak in UploadClient due CFile (based on 0.29a)
	catch(CFileException* error){
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(false,_T("Failed to create upload package for %s - %s"),GetUserName(),GetErrorMessage(error));
		theApp.uploadqueue->RemoveFromUploadQueue(this,ETS_FILE_ERROR);
		if (filedata)
			delete filedata;
		error->Delete();
		return false;
	}
	//^- eklmn: bugfix(13): memleak in UploadClient due CFile (based on 0.29a)
	//	AddDebugLogLine(false,"Debug: Packet done. Size: %i",blockpack->GetLength());
	return true;
	
	EMULE_CATCH
	return false;
}

void CUpDownClient::CheckForUpPartsInfo(){
	EMULE_TRY
    if(m_nPartCount==0)
        return;
    if(!reqfile)
        return;
    if(memcmp(reqfileid,reqfile->GetFileHash(),16))
        return;
    if(!m_abyPartStatus)
        return;

    if (m_abyUpPartStatus) {
		delete[] m_abyUpPartStatus;
		m_abyUpPartStatus = NULL;
    }
	m_nUpPartCount = m_nPartCount;
    m_nAvailUpPartCount = 0;
    m_abyUpPartStatus = new uint8[m_nUpPartCount];
    for(int i=0; i<m_nUpPartCount; i++) {
        m_abyUpPartStatus[i] = m_abyPartStatus[i];
        if(m_abyUpPartStatus[i]) 
            m_nAvailUpPartCount++;
    }
	EMULE_CATCH
}

void CUpDownClient::ProcessUpFileStatus(char* packet,uint32 size){
    EMULE_TRY

    if (m_abyUpPartStatus) {
		delete[] m_abyUpPartStatus;
		m_abyUpPartStatus = NULL;	// added by jicxicmic
    }
	m_nUpPartCount = 0;
    m_nAvailUpPartCount = 0;
	m_nUpCompleteSourcesCount= 0;	// netwolf: complete sources (zegzav )
    if( size == 16 ) {
        CheckForUpPartsInfo(); // obaldin
		return;
    }
	CSafeMemFile data((BYTE*)packet,size);
	uchar cfilehash[16];
	data.Read(cfilehash,16);
	CKnownFile* tempreqfile = theApp.sharedfiles->GetFileByID(cfilehash);
	data.Read(&m_nUpPartCount,2);
	if (!m_nUpPartCount){
		m_nUpPartCount = tempreqfile->GetPartCount();
		m_abyUpPartStatus = new uint8[m_nUpPartCount];
		memset(m_abyUpPartStatus,0,m_nUpPartCount);
	}
	else{
		if (tempreqfile->GetPartCount() != m_nUpPartCount){
			m_nUpPartCount = 0;
			return;
		}
		m_abyUpPartStatus = new uint8[m_nUpPartCount];
		uint16 done = 0;
		while (done != m_nUpPartCount){
			uint8 toread;
			data.Read(&toread,1);
			for (sint32 i = 0;i != 8;i++){
				m_abyUpPartStatus[done] = ((toread>>i)&1)? 1:0;
                if(m_abyUpPartStatus[done])
                    m_nAvailUpPartCount++;
//				We may want to use this for another feature..
//				if (m_abyUpPartStatus[done] && !tempreqfile->IsComplete(done*PARTSIZE,((done+1)*PARTSIZE)-1))
//					bPartsNeeded = true;
				done++;
				if (done == m_nUpPartCount)
					break;
			}
		}

		// netwolf: complete sources (zegzav ) - BEGIN
		if ((GetExtendedRequestsVersion() > 1) && (size >= 20 + (((uint32)m_nUpPartCount + 7) >> 3)))
		{
			uint16 nCount;
			data.Read(&nCount,2);
			SetUpCompleteSourcesCount(nCount);
		}
		// netwolf: complete sources (zegzav ) - END
	}
	tempreqfile->CalculateCompleteSources(); // netwolf complete sources (zegzav) // tempreqfile->NewAvailPartsInfo()
	theApp.emuledlg->transferwnd.m_wndQueueList.RefreshClient(this);

    EMULE_CATCH
}


void CUpDownClient::CreateStandartPackets(byte* data,uint32 togo, Requested_Block_Struct* currentblock, bool bFromPF)
{
	EMULE_TRY
	uint32 nPacketSize;
	CMemFile memfile((BYTE*)data,togo);
	if (togo > 10240) 
		nPacketSize = togo/(uint32)(togo/10240);
	else
		nPacketSize = togo;
	while (togo){
		if (togo < nPacketSize*2)
			nPacketSize = togo;
		togo -= nPacketSize;
		Packet* packet = new Packet(OP_SENDINGPART,nPacketSize+24,OP_EDONKEYPROT,bFromPF);	// -khaos: Create the packet with the new boolean.
		md4cpy(&packet->pBuffer[0],reqfileid);
		uint32 statpos = (currentblock->EndOffset - togo) - nPacketSize;
		uint32 endpos = (currentblock->EndOffset - togo);
		#ifndef AMD
		memcpy(&packet->pBuffer[16],&statpos,4);
		memcpy(&packet->pBuffer[20],&endpos,4);
		#else
		memcpy_amd(&packet->pBuffer[16],&statpos,4);
		memcpy_amd(&packet->pBuffer[20],&endpos,4);
		#endif
		memfile.Read(&packet->pBuffer[24],nPacketSize);

        	splitAndAddPacketsToSendQueue(packet, bFromPF);
	}
	EMULE_CATCH
}

void CUpDownClient::CreatePackedPackets(byte* data,uint32 togo, Requested_Block_Struct* currentblock, bool bFromPF){
	EMULE_TRY
	BYTE* output = new BYTE[togo+300];
	uLongf newsize = togo+300;
	uint16 result = compress2(output,&newsize,data,togo,9);
	if (result != Z_OK || togo <= newsize){
		delete[] output;
		CreateStandartPackets(data,togo,currentblock, bFromPF);		// -khaos: Create the packet with the new boolean.
		return;
	}
	m_bUsedComprUp = true;
	compressiongain += (togo-newsize); // Add show compression
	notcompressed += togo;// Add show compression
	CMemFile memfile(output,newsize);
	togo = newsize;
	uint32 nPacketSize;
	if (togo > 10240) 
		nPacketSize = togo/(uint32)(togo/10240);
	else
		nPacketSize = togo;
	while (togo){
		if (togo < nPacketSize*2)
			nPacketSize = togo;
		togo -= nPacketSize;
		Packet* packet = new Packet(OP_COMPRESSEDPART,nPacketSize+24,OP_EMULEPROT, bFromPF);	// -khaos: Create the packet with the new boolean.
		md4cpy(&packet->pBuffer[0],reqfileid);
		uint32 statpos = currentblock->StartOffset;
		#ifndef AMD
		memcpy(&packet->pBuffer[16],&statpos,4);
		memcpy(&packet->pBuffer[20],&newsize,4);
		#else
		memcpy_amd(&packet->pBuffer[16],&statpos,4);
		memcpy_amd(&packet->pBuffer[20],&newsize,4);
		#endif
		memfile.Read(&packet->pBuffer[24],nPacketSize);

        	splitAndAddPacketsToSendQueue(packet, bFromPF);
	}
	delete[] output;
	EMULE_CATCH
}

/**
 * A packet should never be larger than MAXFRAGSIZE. This method splits up the
 * given packets into several packets with the size MAXFRAGSIZE, if necessary.
 */
void CUpDownClient::splitAndAddPacketsToSendQueue(Packet* packet, bool bFromPF)
{
	EMULE_TRY

	if (packet->GetRealPacketSize() > 0 && packet->GetRealPacketSize() > MAXFRAGSIZE*2) 
	// splitting packets
	{
		uint32 nSize = packet->GetRealPacketSize();
		char* pBuffer = packet->DetachPacket();
		delete packet;
        	
		uint32 nPos = 0;
		bool bLast = true;
		while (nPos < nSize)
		{
			uint32 nNewSize = (nSize-nPos < MAXFRAGSIZE) ? nSize-nPos : MAXFRAGSIZE;
			char* pBuffer2 = new char[nNewSize];
			#ifndef AMD
			memcpy(pBuffer2,pBuffer+nPos,nNewSize);
			#else
			memcpy_amd(pBuffer2,pBuffer+nPos,nNewSize);
			#endif
			nPos += nNewSize;
			m_BlockSend_queue.AddTail(new Packet(pBuffer2,nNewSize,(nNewSize < MAXFRAGSIZE), bFromPF));
			bLast = false;
		}
		delete[] pBuffer;
	} 
	else 
	{
		m_BlockSend_queue.AddTail(packet);
	}
	
	EMULE_CATCH
}

void CUpDownClient::SetUploadFileID(uchar* tempreqfileid){
	EMULE_TRY
	if(tempreqfileid)
		md4cpy(reqfileid,tempreqfileid);
	else
		for( int i = 0; i <16; i++)
			reqfileid[i] = 0;
	EMULE_CATCH
}

void CUpDownClient::AddReqBlock(Requested_Block_Struct* reqblock){
	EMULE_TRY
	for (POSITION pos = m_DoneBlocks_list.GetHeadPosition();pos != 0;m_DoneBlocks_list.GetNext(pos)){
		if (reqblock->StartOffset == m_DoneBlocks_list.GetAt(pos)->StartOffset && reqblock->EndOffset == m_DoneBlocks_list.GetAt(pos)->EndOffset){
			delete reqblock;
			return;
		}
	}
	for (POSITION pos = m_BlockRequests_queue.GetHeadPosition();pos != 0;m_BlockRequests_queue.GetNext(pos)){
		if (reqblock->StartOffset == m_BlockRequests_queue.GetAt(pos)->StartOffset && reqblock->EndOffset == m_BlockRequests_queue.GetAt(pos)->EndOffset){
			delete reqblock;
			return;
		}
	}
	m_BlockRequests_queue.AddTail(reqblock);

	EMULE_CATCH
}

void CUpDownClient::SetUpStartTime(uint32 dwTime){
	EMULE_TRY
	if (dwTime)
		m_dwUploadTime = dwTime;
	else
		m_dwUploadTime = ::GetTickCount();
	EMULE_CATCH
}

void CUpDownClient::SetWaitStartTime(uint32 dwTime){
	EMULE_TRY
	if (dwTime)
		m_dwWaitTime = dwTime;
	else
		m_dwWaitTime = ::GetTickCount();
	EMULE_CATCH
}

uint32 CUpDownClient::SendBlockData(uint32 nMaxAmmount, bool& wantsMoreBandwidth)
{
	EMULE_TRY
    DWORD curTick = GetTickCount();
    // this will be set to true if/when the slot is actually fed some data
    wantsMoreBandwidth = false;
    uint32 amounttransfered = 0;

    // check for 'stuck' clients
    //	if ((GetSessionUp()==0) && (GetUpStartTimeDelay()>2*60*1000)) {
    if ((curTick-GetLastGotULData() > 2*60*1000)  && (GetUpStartTimeDelay()>2*60*1000)) {
		// were unable to transfer for 2 minutes, put the client back on queue
		AddDebugLogLine(false, _T("Client %s(%s) appears to be stuck, putting back on queue"), GetUserName(), GetClientNameAndVersionString(this)); 
		SetWaitStartTime();
		theApp.uploadqueue->RemoveFromUploadQueue(this,ETS_TIMEOUT);
		theApp.uploadqueue->AddClientToQueue(this,true);
		return 0;
	}

#ifdef OLD_SOCKETS_ENABLED
	// Only feed this slot new data if it has been able to empty its
	// buffer since we last offered it data
	if (socket && !socket->IsBusy() && GetUploadState() == US_UPLOADING) {
		// v- eklmn: bugfix(03): keeping user in queue if he doesn't need anything (v2)
		if (m_BlockSend_queue.IsEmpty()) {	
			 // no valid blocks to transfer. Put the client back on queue
			if ( !CreateNextBlockPackage())  return 0;
		}
		// ^- eklmn: bugfix(03): keeping user in queue if he doesn't need anything (v2)
		
		// Feed data to this slot, one block at a time, as long as there's
		// enough bandwidth left for the entire next block.
		// There must also be blocks to send (not empty request queue)
		while (!m_BlockSend_queue.IsEmpty() && m_BlockSend_queue.GetHead()->GetRealPacketSize()+amounttransfered <= nMaxAmmount) 
		{
			// Get next block and give it to the socket for transfer
			Packet* tosend = m_BlockSend_queue.RemoveHead();
			uint32 nBlockSize = tosend->GetRealPacketSize();
			// -khaos--+++>
			// Extended statistics information based on which client software and which port we sent this data to...
			// This also updates the grand total for sent bytes, etc.  And where this data came from.  Yeesh.
			// eklmnf: we have to call function before packet will sended, otherwise packet will be deleted & we have Access violation
			theApp.glob_prefs->Add2SessionTransferData(GetClientSoft(), GetUserPort(), tosend->IsFromPF(), Upload, nBlockSize);
			//theApp.UpdateSentBytes(nBlockSize); 
			socket->SendPacket(tosend,true,false);
			tosend = NULL;	//eklmn: bugfix(17): upload fix(reset tosend in SendBlockData)
			//socket->ResetTimeOutTimer();

			// We've transfered some data. Store this in relevant statistics variables
			m_nTransferedUp += nBlockSize;
			amounttransfered += nBlockSize;
//			theApp.uploadqueue->AddUpDataRateMSOverhead(24, 160);
			credits->AddUploaded(nBlockSize,this->GetIP());

			// If we're out of prepared blocks, create some more
			if (m_BlockSend_queue.IsEmpty()) CreateNextBlockPackage();
		}

		// If this slot wasn't given enough data to empty it's request queue, 
		// it wants more data.

		if(!m_BlockSend_queue.IsEmpty()) {
			wantsMoreBandwidth = true;
		}

		// Remember when this connection was last fed some data, so we
		// can make sure we give it data often enough to not time out.
		if(amounttransfered > 0) {
			SetLastGotULData();
		}
	}
#endif //OLD_SOCKETS_ENABLED

	// Store how much data we've transfered this round,
	// to be able to calculate average speed later
	m_AverageUDR_list.push_front(m_nTransferedUp);
	m_Average_UL_tick_list.push_front(curTick);

	// Don't store more than 40 seconds of statistics m_Average_DL_tick_list.size()
	//while(curTick-m_Average_UL_tick_list.back() > 40000) {
	while(m_Average_UL_tick_list.size() > 1000) 
	{
   		m_AverageUDR_list.pop_back();
		m_Average_UL_tick_list.pop_back();
	}

	//eklmn: since performance is more importatnt, that graphical output this function moved to the end
	// Check if it's time to update the display.
	if (curTick-m_lastRefreshedULDisplay > MINWAIT_BEFORE_ULDISPLAY_WINDOWUPDATE && m_AverageUDR_list.size() >= 2) 
	{
		// Calculate average datarate
		// Present this slot's speed as 0 unless there's a good chunk
		// of stored statistics for it. If there's to little data, the
		// UL speed will be far off, so we might as well hide it in those cases.
		uint32 timePeriod = m_Average_UL_tick_list.front()-m_Average_UL_tick_list.back();
		if (timePeriod > 5000)
			m_nUpDatarate = (uint32)(((float)(m_AverageUDR_list.front()-m_AverageUDR_list.back()))*1000 / timePeriod);
		else
			m_nUpDatarate = -1;

		theApp.emuledlg->transferwnd.m_wndUploadList.RefreshClient(this);
		m_lastRefreshedULDisplay = curTick+(uint32)(rand()/(RAND_MAX/1000));	//SyruS moved rand out of if(...)
	}

	return amounttransfered;
	
	EMULE_CATCH
	return 0;
}

void CUpDownClient::FlushSendBlocks(){ // call this when you stop upload, or the socket might be not able to send
	EMULE_TRY
	bool bBreak = false;
#ifdef OLD_SOCKETS_ENABLED
	while (!m_BlockSend_queue.IsEmpty() && m_BlockSend_queue.GetHead()->IsSplitted() && socket && socket->IsConnected() && !bBreak ){	
		Packet* tosend = m_BlockSend_queue.RemoveHead();
		bool bBreak = tosend->IsLastSplitted();
		theApp.uploadqueue->AddUpDataOverheadOther(tosend->size);
		socket->SendPacket(tosend,true,false);
	}
#endif //OLD_SOCKETS_ENABLED
	EMULE_CATCH
}

void CUpDownClient::SendHashsetPacket(char* forfileid){
	EMULE_TRY
	CKnownFile* file = theApp.sharedfiles->GetFileByID((uchar*)forfileid);
	if (!file)
	{
		AddDebugLogLine(false, GetResString(IDS_ERR_REQ_FNF)+CString(" (SendHashsetPacket)"));
		return;
	}

	CSafeMemFile data(1024);
	data.Write(file->GetFileHash(),16);
	uint16 parts = file->GetHashCount();
	data.Write(&parts,2);
	for (int i = 0; i < parts; i++)
		data.Write(file->GetPartHash(i),16);
	Packet* packet = new Packet(&data);
	packet->opcode = OP_HASHSETANSWER;
	theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
#ifdef OLD_SOCKETS_ENABLED
	socket->SendPacket(packet,true,true);
#endif //OLD_SOCKETS_ENABLED
	EMULE_CATCH
}

float CUpDownClient::GetClientSpeed()
{
	EMULE_TRY
	// Calculates the session speed of the client
	// Providing the other Clients with this info
	// gives then the possibility to select the 
	// sources more intelligently
	if (theApp.stat_transferStarttime==0) return 0;

	DWORD running=(GetTickCount()-theApp.stat_transferStarttime)/1000;
	if (running<5) return 0;

	return (float)(theApp.stat_sessionSentBytes/1024) / running;
	EMULE_CATCH
	return 0;
}

void CUpDownClient::ClearUploadBlockRequests(){
	EMULE_TRY
	FlushSendBlocks();
	for (POSITION pos = m_BlockRequests_queue.GetHeadPosition();pos != 0;m_BlockRequests_queue.GetNext(pos))
		delete m_BlockRequests_queue.GetAt(pos);
	m_BlockRequests_queue.RemoveAll();
	
	for (POSITION pos = m_DoneBlocks_list.GetHeadPosition();pos != 0;m_DoneBlocks_list.GetNext(pos))
		delete m_DoneBlocks_list.GetAt(pos);
	m_DoneBlocks_list.RemoveAll();
	
	for (POSITION pos = m_BlockSend_queue.GetHeadPosition();pos != 0;m_BlockSend_queue.GetNext(pos))
		delete m_BlockSend_queue.GetAt(pos);
	m_BlockSend_queue.RemoveAll();
	EMULE_CATCH
}

void CUpDownClient::SendRankingInfo(){
	EMULE_TRY
	if (!ExtProtocolAvailable())
		return;
	uint16 nRank = theApp.uploadqueue->GetWaitingPosition(this);
    // By Maverick
#ifndef DISABLE_EXTENDED_INFO_EXCHANGE
    #define DISABLE_EXTENDED_INFO_EXCHANGE
    // The extended info exchange is currently disabled since it is possible
    // to ask the client with several userhashes for the file and then use 
    // the one with the best UL/DL ratio
    // It is planned to reactivate this feature, but with a protection against
    // this userhash spoofing.
    // At the last stage this faeture should make it possible to choose the 
    // sources more intelligently by taking UL/DL ratio, queuerank and 
    // speed of the other client into account.
#endif

#ifdef DISABLE_EXTENDED_INFO_EXCHANGE
	if (!nRank)
		return;
#else
	uint16 nBaseModifier = (uint16) GetScore(false,false,true); //16-Bits are enough for base value
	uint32 nScore = GetScore(false,false,false);
	
	uint8 nCommunity = -1;
	if (!theApp.glob_prefs->CommunityEnabled())
		nCommunity = 2;
	else if (IsCommunity())
		nCommunity = 1;
	

	float fBaseValue = 1.0;
	if(credits != (CClientCredits*)NULL) { 
		fBaseValue *= credits->GetScoreRatio(this->GetIP());
	} 
	fBaseValue *= 10;
	uint16 nRatio = (uint16)fBaseValue; 
#endif
	uint16 nClientSpeed = (uint16)((GetClientSpeed() < 65535) ? GetClientSpeed() : 65535); 
										// Providing the other clients with this info 
										// can not be used to leech

	Packet* packet = new Packet(OP_QUEUERANKING,12,OP_EMULEPROT);
	memset(packet->pBuffer,0,12);
	#ifndef AMD
	memcpy(packet->pBuffer+0,&nRank,2);
#ifndef DISABLE_EXTENDED_INFO_EXCHANGE
	memcpy(packet->pBuffer+2,&nBaseModifier,1);		// UINT8 is enough 0-100
	memcpy(packet->pBuffer+3,&nRatio,2);			// UINT16 needed saw about 32000 max (SF * UL/DL * RF) 
	memcpy(packet->pBuffer+5,&nScore,4);			// UINT32 needed
	memcpy(packet->pBuffer+9,&nCommunity,1);		// 2-Bits needed
#endif
	memcpy(packet->pBuffer+10,&nClientSpeed,2);		// UINT16 (I think not many have more than 65 MByte UL)
	#else
	memcpy_amd(packet->pBuffer+0,&nRank,2);
#ifndef DISABLE_EXTENDED_INFO_EXCHANGE
	memcpy_amd(packet->pBuffer+2,&nBaseModifier,1);		// UINT8 is enough 0-100
	memcpy_amd(packet->pBuffer+3,&nRatio,2);			// UINT16 needed saw about 32000 max (SF * UL/DL * RF) 
	memcpy_amd(packet->pBuffer+5,&nScore,4);			// UINT32 needed
	memcpy_amd(packet->pBuffer+9,&nCommunity,1);		// 2-Bits needed
#endif
	memcpy_amd(packet->pBuffer+10,&nClientSpeed,2);		// UINT16 (I think not many have more than 65 MByte UL)
	#endif
	// END Maverick
	theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
#ifdef OLD_SOCKETS_ENABLED
	socket->SendPacket(packet,true,true);
#endif //OLD_SOCKETS_ENABLED
	EMULE_CATCH
}

void CUpDownClient::SendCommentInfo(CKnownFile *file) {
	EMULE_TRY
	if (!m_bCommentDirty || file == NULL || !ExtProtocolAvailable() || m_byAcceptCommentVer < 1)
		return;
	m_bCommentDirty = false;

	int8 rating=file->GetFileRate();
	CString desc=file->GetFileComment();
	if(file->GetFileRate() == 0 && desc.IsEmpty())
		return;

	CSafeMemFile data(256);
	data.Write(&rating,sizeof(rating));
	int length=desc.GetLength();
	if (length>128) 
		length=128;
	data.Write(&length,sizeof(length));
	if (length>0) 
		data.Write(desc.GetBuffer(),length);
	Packet *packet = new Packet(&data,OP_EMULEPROT);
	packet->opcode = OP_FILEDESC;
	theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
#ifdef OLD_SOCKETS_ENABLED
	socket->SendPacket(packet,true);
#endif //OLD_SOCKETS_ENABLED
	EMULE_CATCH
}

void  CUpDownClient::AddRequestCount(uchar* fileid){
	EMULE_TRY
	for (POSITION pos = m_RequestedFiles_list.GetHeadPosition();pos != 0;m_RequestedFiles_list.GetNext(pos)){
		Requested_File_Struct* cur_struct = m_RequestedFiles_list.GetAt(pos);
		if (!md4cmp(cur_struct->fileid,fileid)){
			if (::GetTickCount() - cur_struct->lastasked < MIN_REQUESTTIME){
				if (GetDownloadState() != DS_DOWNLOADING)
					cur_struct->badrequests++;
				if (cur_struct->badrequests == BADCLIENTBAN) //katsyonak 16.06.03
					Ban(_T("AUTOMATED MESSAGE: You're being banned because your client is behaving aggressively!"), GetResString(IDS_BAN_AGGRESSIVECLI));
			}
			else{
				if (cur_struct->badrequests)
					cur_struct->badrequests--;
			}
			cur_struct->lastasked = ::GetTickCount();
			return;
		}
	}
	Requested_File_Struct* new_struct = new Requested_File_Struct;
	memset(new_struct,0,sizeof(Requested_File_Struct));
	md4cpy(new_struct->fileid,fileid);
	new_struct->lastasked = ::GetTickCount();
	m_RequestedFiles_list.AddHead(new_struct);
	EMULE_CATCH
}

void  CUpDownClient::UnBan(){
	EMULE_TRY
	m_bBanned = false;
	m_dwBanTime = 0;
	SetWaitStartTime();
	theApp.uploadqueue->UpdateBanCount();
	theApp.emuledlg->transferwnd.UpdateUploadHeader();
	//--- xrmb:hashthieves1 ---
	offensecounter.RemoveKey(getUID());
	hashbase.RemoveKey(getUID());
	hashthief=false;
	//--- :xrmb ---
	for (POSITION pos = m_RequestedFiles_list.GetHeadPosition();pos != 0;m_RequestedFiles_list.GetNext(pos)){
		Requested_File_Struct* cur_struct = m_RequestedFiles_list.GetAt(pos);
		cur_struct->badrequests = 0;
		cur_struct->lastasked = 0;	
	}
	theApp.emuledlg->transferwnd.m_wndQueueList.RefreshClient(this);
	EMULE_CATCH
}

void CUpDownClient::Ban(LPCTSTR line, const CString& reason, ...){ //katsyonak 16.06.03
	EMULE_TRY

 	if (!theApp.glob_prefs->BanEnabled())
	{ // DbT: if the ban function is disable, no need to ban a client (default is enable!)
		m_bBanned = false;
		m_dwBanTime = 0;
		return;
	}
	if (m_byUploadState == US_UPLOADING)
	{	//SyruS kick already downloading client back to waitingqueue
		AddDebugLogLine(false,_T("Client '%s' banned and kicked out of uploadqueue %s"), GetUserName(), reason);
		theApp.uploadqueue->RemoveFromUploadQueue(this, ETS_BAN, true);
		theApp.uploadqueue->AddClientToQueue(this, true);
	}
	if (m_bBanned)
	{	//SyruS no need to ban a client (and send a message) again except for extending the bantime
		AddDebugLogLine(false,_T("Client '%s' is already banned from the uploadqueue %s"), GetUserName(), reason);
		m_dwBanTime = ::GetTickCount();
		return;
	}
	if(theApp.glob_prefs->CommunityNoBanEnabled() && theApp.glob_prefs->CommunityEnabled() && IsCommunity())
		return;
	m_bBanned = true;
	theApp.uploadqueue->UpdateBanCount();
	theApp.emuledlg->transferwnd.UpdateUploadHeader();
	m_dwBanTime = ::GetTickCount();
	if(theApp.glob_prefs->IsBanMessageEnabled())
	{
		theApp.emuledlg->chatwnd.chatselector.StartSession(this, TRUE);
		theApp.emuledlg->chatwnd.chatselector.SendMessage(line);
		theApp.emuledlg->chatwnd.chatselector.EndSession(this);		// now it automatically closes the session afterwards
		if (!theApp.glob_prefs->GetCMNotLog())
			AddLogLine(false, IDS_SENTBAN, GetUserName(), reason);
	}
	else
		if (!theApp.glob_prefs->GetCMNotLog())
			AddLogLine(false, IDS_CLIENTBLOCKED, GetUserName(), reason);

	theApp.emuledlg->transferwnd.m_wndQueueList.RefreshClient(this);

	EMULE_CATCH
}

void CUpDownClient::UDPFileReasked(){
	EMULE_TRY
	AddAskedCount();
	SetLastUpRequest();
	SetLastL2HACExecution(); //<<-- enkeyDEV(th1) -L2HAC
	uint16 nRank = theApp.uploadqueue->GetWaitingPosition(this);
	Packet* response = new Packet(OP_REASKACK,2,OP_EMULEPROT);
	#ifndef AMD
	memcpy(response->pBuffer,&nRank,2);
	#else
	memcpy(response->pBuffer,&nRank,2);
	#endif

	// By Maverick
	ProxySettings proxy = theApp.glob_prefs->GetProxy();
	if (!proxy.UseProxy){
		theApp.uploadqueue->AddUpDataOverheadFileRequest(response->size);
#ifdef OLD_SOCKETS_ENABLED
		theApp.clientudp->SendPacket(response,GetIP(),GetUDPPort());
#endif //OLD_SOCKETS_ENABLED
	}
	EMULE_CATCH
}

// Lord KiRon
CString CUpDownClient::GetDownloadStateAsString(void) 
{ 
	return GetStatusDLQueueString((eDLQState)GetDownloadState());
} 
// Lord KiRon - end
// By Maverick
uint32	CUpDownClient::GetRemoteScore()
{
	EMULE_TRY
	if (IsRemoteInfoAvaiable()) return m_RemoteScore;
	EMULE_CATCH
	return 0;	// We do not know for how long we are on the queue
				// Furhtermore the we do not know the remote ratio.
}

uint16	CUpDownClient::GetRemoteRatio()
{
	EMULE_TRY
	if (IsRemoteInfoAvaiable()) return m_RemoteRatio;
	// This calculation is not very accurate for several reasons:
	// - We do not know the file priority
	// - We do not save the client info if he has nothing uploaded to us
	// - Other factores like community etc

	/* // Myself Bot user?
	bool bIsBotuser;	//we still assume that the other client 'punishes' us if our name is 'wrong' 	//Cax2 - not anymore
	CString pszUsername = theApp.glob_prefs->GetUserNick();
	if (pszUsername)
		bIsBotuser = (strstr(pszUsername, "edk-files") || strstr(pszUsername, "finder")) && strstr(pszUsername, "com");
	else bIsBotuser = false;
	*/

	float fBaseValue = 100.0f;
	//if (bIsBotuser) fBaseValue *= 0.9f;	//Cax2 - not anymore
#ifdef OLD_SOCKETS_ENABLED
	if (theApp.serverconnect->IsLowID()) fBaseValue *= 0.8f;
#endif //OLD_SOCKETS_ENABLED
	if (GetDownloadState() == DS_DOWNLOADING) fBaseValue *= 1.5f;
	if (IsFriend()) fBaseValue *= 2.0f;
	if (theApp.glob_prefs->CommunityEnabled() && IsCommunity())
		fBaseValue *= 2.0f;
	fBaseValue *= GetRemoteBaseModifier();
	fBaseValue *= GetSmallFilePushRatio() ;
	fBaseValue *= GetRareFilePushRatio() ;
	return (uint32)fBaseValue;
	EMULE_CATCH
	return 100;
}

float	CUpDownClient::GetRemoteBaseModifier()
{
	EMULE_TRY
	if (IsRemoteInfoAvaiable()) return (float)m_RemoteBaseModifier / 100.0;
	// This calculation is not very accurate for some reasons:
	// - Crashes can cause that either client can not write
	//   its client data. This can cause differences of the
	//   client data at the two clients.
	if(credits == (CClientCredits*)-1)
		credits = NULL;
	if(credits != NULL)
		return credits->GetScoreRatio(this->GetIP(),true);
	EMULE_CATCH
	return 1.0;
}

uint32 CUpDownClient::GetCurrentlyUploadingPart() {
	EMULE_TRY
    if(!m_DoneBlocks_list.IsEmpty()) { // do we need to check upload state here?
        Requested_Block_Struct* done_block = (Requested_Block_Struct*)m_DoneBlocks_list.GetHead();
        uint32 partno = done_block->StartOffset / PARTSIZE;
        return partno;
    }
    else
        return 0xFFFF;
	EMULE_CATCH
    return 0xFFFF;
}

