//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"
#include "emuleDlg.h"
#include "knownfilelist.h"
#include "preferences.h"
#ifdef OLD_SOCKETS_ENABLED
#include "sockets.h"
#endif //OLD_SOCKETS_ENABLED
#include "serverlist.h"
#include "sharedfilelist.h"
#include "searchlist.h"
#ifdef OLD_SOCKETS_ENABLED
#include "listensocket.h"
#endif //OLD_SOCKETS_ENABLED
#include "uploadqueue.h"
#include "downloadqueue.h"
#include "clientlist.h"
#include "clientcredits.h"
#include "friendlist.h"
#include "filehashcontrol.h" // Lord KiRon
#ifdef OLD_SOCKETS_ENABLED
#include "clientudpsocket.h"
#endif //OLD_SOCKETS_ENABLED
#include "mdump.h"
#include "IPFilter.h"
#include "Loggable.h"
#include <afxmt.h>
#include "mmserver.h"
#include "smtp.h"

#include "BerkeleyDb/build_win32/db_cxx.h"
#include <afxcoll.h>

class CSearchList;
class CUploadQueue;
#ifdef OLD_SOCKETS_ENABLED
class CListenSocket;
#endif //OLD_SOCKETS_ENABLED
class CDownloadQueue;
class CWebServer;
class CEmEngine;

class CemuleApp : public CWinApp, public CLoggable
{
public:
	enum AppState{APP_STATE_RUNNING=0,APP_STATE_SHUTINGDOWN,APP_STATE_DONE};	// moved from emuldlg [FoRcHa]

	CemuleApp();
	CemuleDlg*			emuledlg;
	CClientList*		clientlist;
	CKnownFileList*		knownfiles;
	CPreferences*		glob_prefs;
#ifdef OLD_SOCKETS_ENABLED
	CServerConnect*		serverconnect;
#endif //OLD_SOCKETS_ENABLED
	CServerList*		serverlist;	
	CSharedFileList*	sharedfiles;
	CSearchList*		searchlist;
#ifdef OLD_SOCKETS_ENABLED
	CListenSocket*		listensocket;
#endif //OLD_SOCKETS_ENABLED
	CUploadQueue*		uploadqueue;
	CDownloadQueue*		downloadqueue;
	CClientCreditsList*	clientcredits;
	CFriendList*		friendlist;
#ifdef OLD_SOCKETS_ENABLED
	CClientUDPSocket*	clientudp;
#endif //OLD_SOCKETS_ENABLED
	CWebServer*			webserver; // kuchin
	virtual BOOL		InitInstance();
	CString*			pendinglink;
	tagCOPYDATASTRUCT  	sendstruct; // Cax2
	CFileHashControl 	m_FilesHashing; // Lord KiRon
	CMutex				hashing_mut;
	CIPFilter*			ipfilter;
#ifdef MOBILE_MULE
	CMMServer*			mmserver;
#endif //MOBILE_MULE
	CSMTPConnection*	smtpconnection;

#ifdef NEW_SOCKETS
	CEmEngine*			pEngine;
#endif //NEW_SOCKETS
	
	uint64				stat_sessionReceivedBytes;
	uint64				stat_sessionSentBytes;
	uint16				stat_reconnects;
	DWORD				stat_transferStarttime;
	DWORD				stat_serverConnectTime;
	DWORD				stat_starttime;
	uint16				stat_filteredclients;
	DWORD				m_dwCommCtrlMjr;
	DWORD				m_dwCommCtrlMin;

	DbEnv*	pDbEnv;	// Database environment
	Db*		pDbSLS;	// Stored sources

	CArray<CString,CString> webservices;

	// DonGato: ICC customization
	void			InitURLs();
	CStringArray	g_Urls;

// Implementierung
	// ed2k link functions
	CString		StripInvalidFilenameChars(CString strText, bool bKeepSpaces = true);	
	bool		CopyTextToClipboard(const CString& strText) const;
	CString		CopyTextFromClipboard(); //<<-- enkeyDEV(Ottavio84) -Download all ed2ks in clipboard-
	void		OnlineSig(); 
	void		UpdateReceivedBytes(int32 bytesToAdd);
	void		UpdateSentBytes(int32 bytesToAdd);

	DECLARE_MESSAGE_MAP()
protected:
	bool ProcessCommandline();
	void SetTimeOnTransfer();
	static BOOL CALLBACK SearchEmuleWindow(HWND hWnd, LPARAM lParam);
	void OnHelp();

	HIMAGELIST m_hSystemImageList;
	CMapStringToPtr m_aExtToSysImgIdx;
	CSize m_sizSmallSystemIcon;

public:
	AppState	m_app_state; // defines application state for shutdown 
	virtual int ExitInstance();

	int			GetFileTypeSystemImageIdx(LPCTSTR pszFilePath, int iLength = -1);
	HIMAGELIST	GetSystemImageList() { return m_hSystemImageList; }
	CSize		GetSmallSytemIconSize() { return m_sizSmallSystemIcon; }

private:
	MiniDumper *m_pDump;
	HANDLE m_hMutexOneInstance;
};
extern CemuleApp theApp;