//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"
#include "packets.h"
#include "otherfunctions.h"
#include "memcpy_amd.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

Packet::Packet(uint8 protocol){
	m_bSplitted = false;
	m_bLastSplitted = false;
	size = 0;
	pBuffer = 0;
	completebuffer = 0;
	tempbuffer = 0;
	prot = protocol;
	m_bPacked = false;
}

Packet::Packet(char* header){
	m_bSplitted = false;
	m_bPacked = false;
	m_bLastSplitted = false;
	tempbuffer = 0;
	pBuffer = 0;
	completebuffer = 0;
	Header_Struct* head = (Header_Struct*) header;
	size = head->packetlength-1;
	opcode = head->command;
	prot = head->eDonkeyID;
}

// -khaos--+++> Slightly modified for our stats uses...
Packet::Packet(char* pPacketPart, uint32 nSize ,bool bLast, bool bFromPF)
{
	m_bFromPF = bFromPF;
	m_bSplitted = true;
	m_bPacked = false;
	m_bLastSplitted = bLast;
	tempbuffer = 0;
	pBuffer = 0;
	completebuffer = pPacketPart;
	size = nSize-6;
}

// -khaos--+++> Slightly modified for our stats uses...
//				If m_bFromPF = true then packet was formed from a partfile
//				If m_bFromPF = false then this packet was formed from a complete shared file.
Packet::Packet(int8 in_opcode,uint32 in_size,uint8 protocol,bool bFromPF)
{
	m_bFromPF = bFromPF;
	m_bSplitted = false;
	m_bPacked = false;
	m_bLastSplitted = false;
	tempbuffer = 0;
	if (in_size){
		completebuffer = new char[in_size+10];
		pBuffer = completebuffer+6;
		memset(completebuffer,0,in_size+10);
	}
	else{
		pBuffer = 0;
		completebuffer = 0;
	}
	opcode = in_opcode;
	size = in_size;
	prot = protocol;
}

Packet::Packet(CMemFile* datafile,uint8 protocol)
{
	m_bSplitted = false;
	m_bPacked = false;
	m_bLastSplitted = false;
	size = datafile->GetLength();
	completebuffer = new char[datafile->GetLength()+10];
	pBuffer = completebuffer+6;
	BYTE* tmp = datafile->Detach();
	#ifndef AMD
	memcpy(pBuffer,tmp,size);
	#else
	memcpy_amd(pBuffer,tmp,size);
	#endif
	free(tmp);
	tempbuffer = 0;
	prot = protocol;
}

Packet::~Packet(){
	if (completebuffer)
		delete[] completebuffer;
	else if (pBuffer)
		delete [] pBuffer;
	if (tempbuffer)
		delete[] tempbuffer;
}

char* Packet::GetPacket(){
	if (completebuffer){
		if (!m_bSplitted)
			#ifndef AMD
			memcpy(completebuffer,GetHeader(),6);
			#else
			memcpy_amd(completebuffer,GetHeader(),6);
			#endif
		return completebuffer;
	}
	else{
		if (tempbuffer)
			delete[] tempbuffer;
		tempbuffer = new char[size+10];
		#ifndef AMD
		memcpy(tempbuffer,GetHeader(),6);
		memcpy(tempbuffer+6,pBuffer,size);
		#else
		memcpy_amd(tempbuffer,GetHeader(),6);
		memcpy_amd(tempbuffer+6,pBuffer,size);
		#endif
		return tempbuffer;
	}
}

char* Packet::DetachPacket(){
	if (completebuffer){
		if (!m_bSplitted)
			#ifndef AMD
			memcpy(completebuffer,GetHeader(),6);
			#else
			memcpy_amd(completebuffer,GetHeader(),6);
			#endif
		char* result = completebuffer;
		completebuffer = 0;
		pBuffer = 0;
		return result;
	}
	else{
		if (tempbuffer)
			delete[] tempbuffer;
		tempbuffer = new char[size+10];
		#ifndef AMD
		memcpy(tempbuffer,GetHeader(),6);
		memcpy(tempbuffer+6,pBuffer,size);
		#else
		memcpy_amd(tempbuffer,GetHeader(),6);
		memcpy_amd(tempbuffer+6,pBuffer,size);
		#endif
		char* result = tempbuffer;
		tempbuffer = 0;
		return result;
	}
}

char* Packet::GetHeader(){
	ASSERT ( !m_bSplitted );
	Header_Struct* header = (Header_Struct*) head;
	header->command = opcode;
	header->eDonkeyID =  prot;
	header->packetlength = size+1;
	return head;
}

char* Packet::GetUDPHeader(){
	ASSERT ( !m_bSplitted );
	memset(head,0,6);
	UDP_Header_Struct* header = (UDP_Header_Struct*) head;
	header->command = opcode;
	header->eDonkeyID =  prot;
	return head;
}

void Packet::PackPacket(){
	ASSERT (!m_bSplitted);
	BYTE* output = new BYTE[size+300];
	uLongf newsize = size+300;
	uint16 result = compress2(output,&newsize,(BYTE*)pBuffer,size,9);
	if (result != Z_OK || size <= newsize){
		delete[] output;
		return;
	}
	prot = OP_PACKEDPROT;
	#ifndef AMD
	memcpy(pBuffer,output,newsize);
	#else
	memcpy_amd(pBuffer,output,newsize);
	#endif
	delete[] output;
	m_bPacked = true;
}

bool Packet::UnPackPacket(){
	ASSERT ( prot == OP_PACKEDPROT); 
	uint32 nNewSize = size*10+300;
	if (nNewSize > 50000)
		nNewSize = 50000;
	BYTE* unpack = new BYTE[nNewSize];
	uLongf unpackedsize = nNewSize;
	uint16 result = uncompress(unpack,&unpackedsize,(BYTE*)pBuffer,size);
	if (result == Z_OK){
		ASSERT ( completebuffer == NULL );
		ASSERT ( pBuffer != NULL );
		size = unpackedsize;
		delete[] pBuffer;
		pBuffer = (char*)unpack;
		prot =  OP_EMULEPROT;
		return true;				
	}
	delete[] unpack;
	return false;
}


CTag::CTag(LPCTSTR name,uint32 i){	
	floatvalue = 0;
	specialtag = 0;	
	tagname = name;
	type = 3;
	intvalue = i;
}

CTag::CTag(int8 special, uint32 i){
	floatvalue = 0;
	specialtag = 0;
	type = 3;
	intvalue = i;
	specialtag = special;
}

CTag::CTag(LPCTSTR name,LPCTSTR strvalue){	
	intvalue = 0;
	floatvalue = 0;
	specialtag = 0;
	tagname = name;
	type = 2;
	stringvalue = strvalue;
}

CTag::CTag(int8 special, LPCTSTR strvalue){	
	intvalue = 0;
	floatvalue = 0;	
	type = 2;
	stringvalue = strvalue;
	specialtag = special;
}

CTag::CTag(CFile& in_data){	
	intvalue = 0;
	floatvalue = 0;
	specialtag = 0;
	in_data.Read(&type,1);
	uint16 length;
	in_data.Read(&length,2);
	if (length == 1)
		in_data.Read(&specialtag,1);
	else {				
		in_data.Read(tagname.GetBuffer(length+1),length);
		tagname.ReleaseBuffer(length);
	}
	if (type == 2){
		in_data.Read(&length,2);		
		in_data.Read(stringvalue.GetBuffer(length+1),length);
		stringvalue.ReleaseBuffer(length);
	}
	else if (type == 3)
		in_data.Read(&intvalue,4);
	else if (type == 4) // FLOAT (used by Hybrid 0.48)
		in_data.Read(&floatvalue,4);
		
}

CTag::CTag(FILE* in_data){	
	intvalue = 0;
	floatvalue = 0;
	specialtag = 0;
	fread(&type,1,1,in_data);
	uint16 length;
	fread(&length,2,1,in_data);
	if (length == 1)
		fread(&specialtag,1,1,in_data);
	else {		
		fread(tagname.GetBuffer(length+1),length,1,in_data);
		tagname.ReleaseBuffer(length);
	}
	if (type == 2){
		fread(&length,2,1,in_data);		
		fread(stringvalue.GetBuffer(length+1),length,1,in_data);
		stringvalue.ReleaseBuffer(length);
	}
	else if (type == 3)
		fread(&intvalue,4,1,in_data);
	else if (type == 4)
		fread(&floatvalue,4,1,in_data);
}

bool CTag::WriteTagToFile(CFile& file){
	file.Write(&type,1);
	if (!tagname.IsEmpty()){
		uint16 taglen = tagname.GetLength();
		file.Write(&taglen,2);
		file.Write(tagname,taglen);
	}
	else{
		uint16 taglen = 1;
		file.Write(&taglen,2);
		file.Write(&specialtag,taglen);
	}
	if (type == 2){
		uint16 len = stringvalue.GetLength();
		file.Write(&len,2);
		file.Write(stringvalue,len);
	}
	else if (type == 3)
		file.Write(&intvalue,4);
	return true;
}

bool CTag::WriteTagToFile(FILE* file){
	fputc(type,file);
	if (!tagname.IsEmpty() && (!specialtag)){
		uint16 taglen= tagname.GetLength();
		fwrite(&taglen,2,1,file);
		fwrite(tagname,taglen,1,file);
	}
	else{
		uint16 taglen = 1;
		fwrite(&taglen,2,1,file);
		fwrite(&specialtag,taglen,1,file);
	}
	if (type == 2){
		uint16 len = stringvalue.GetLength();
		fwrite(&len,2,1,file);
		fwrite(stringvalue,len,1,file);
	}
	else if (type == 3)
		fwrite(&intvalue,4,1,file);
	return ferror(file);
}